/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_X3M1BTemple.h"
#include "TransportTeam.h"
#include "SunTeam.h"
#include "EnemyShip.h"

const float CHECK_TRANSMISSION_INTERVAL = 5.0f;

tShipInfo* tBaseVictoryState::GetCurrentShipTarget(tShipInfo* ship)
{
	int32 ID = ship->mGetShipTargetID();
//	Assert(NULL != ID);
	tShipInfo* shipTarget = fMissionInfo->mFindShipID(ID);
//	Assert(NULL != shipTarget);
	return shipTarget;
}

iTruth tBaseVictoryState::OrderEnemyToFollowTransport()
{
	int iNumShipsOrdered = 0;
	tTeamInfo* teamTransport = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTransportTeam));
	tShipInfo* pTransport = teamTransport->mFindShipCustomID(tTransportTeam::kSevrinTransportID);
	if (NULL == pTransport)
		return kFalse;

	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

	for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
	{
		if (i->mInPlay())
		{
			i->mFollowEntity(pTransport, kAbsolutePriority);
			++iNumShipsOrdered;
		}
	}

	return (iNumShipsOrdered == NUM_ENEMY_SHIPS) ? kTrue : kFalse;
}

float tBaseVictoryState::CalculateAverageEnemyDamage()
{
	float fTotalDamage = 0.0f;
	int iNumShips = 0;
	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

	for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
	{
		if ((*i)->mInPlay())
		{
			fTotalDamage += (*i)->mGetDamagePercent();
			iNumShips++;
		}
	}

	return fTotalDamage / static_cast<float>(iNumShips);
}	

void tBaseVictoryState::EndMission(eGameStatus Status)
{
	/*
		A prestige bonus for victory is called for by the mission specs.
		It is only to be awarded once, however, and since this function
		could conceivably get called more than once, the following static
		variable provides a guard against multiple awards.
	*/

	static iTruth fPrestigeAwarded = kFalse;
	if (!fPrestigeAwarded)
	{
		if ((iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS) && !fEnemyGotTheSecret && fSecretSecured)
			mAddToBonusPrestige(PRESTIGE_VICTORY);
	}

	gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, Status);
}

void tBaseVictoryState::CheckEndMission()
{
	tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	tTeamInfo* teamTransport = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTransportTeam));
	tShipInfo* pTransport = teamTransport->mFindShipCustomID(tTransportTeam::kSevrinTransportID);

	if (!teamPlayer->mShipsRemainingInPlay())
	{
		/*
			If the player disengages leaving the transport and at least one enemy
			ship behind, we can assume the enemy gets the secret.
		*/

		if ((pTransport != NULL) && pTransport->mInPlay() && 
			(iNumEnemyShipsDefeated < NUM_ENEMY_SHIPS) && !fTractoringTransport)
		{
			fEnemyGotTheSecret = kTrue;
		}

		/*
			The mission must be ended but with the proper result code to queue the
			proper music and so forth.
		*/

		eVictoryLevels VictoryLevel = mCalcVictoryStatus(static_cast<eTeamID>(kPlayerTeam));
		if (VictoryLevel == kAstoundingVictory || VictoryLevel == kVictory)
			EndMission(kPlayerWins);
		else
			EndMission(kPlayerLoses);
	}

	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
	if ((!teamEnemy->mShipsRemainingInPlay() || fEnemySurrendered) && fSecretSecured && fTalkedToSun)
		EndMission(kPlayerWins);

	/*
		If the transport goes out of play without the player learning the secret,
		then the mission is a failure.
	*/

	if (pTransport != NULL && !pTransport->mInPlay() && !fSecretSecured && !fTractoringTransport)
		EndMission(kPlayerLoses);
}

void tBaseVictoryState::DoHailEnemy(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		tEnemyShip* pshipEnemy = static_cast<tEnemyShip*>(targetShip);
		if (fEnemySurrendered)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Enemy_Already_Surrendered_msg);
		else if (pshipEnemy->IsCaptured())
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Nothing_To_Talk_About_msg);
		else
		{
			int32 iRandomResponse = fMissionInfo->mRandomInt32(k_Enemy_Hail_Random1_msg, k_Enemy_Hail_Random3_msg);
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iRandomResponse);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Pulsar_Interference_msg);
	}
}	

void tBaseVictoryState::DoDemandSurrenderEnemy(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		tEnemyShip* pshipEnemy = static_cast<tEnemyShip*>(targetShip);

		if (fEnemySurrendered || pshipEnemy->IsCaptured())
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Done_Enough_msg);
		else if (CalculateAverageEnemyDamage() >= SURRENDER_THRESHOLD)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Enemy_Surrenders_msg);
			tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
			// TODO: figure out the float value's meaning
			teamEnemy->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamNeutral, kGoalIgnore, 1.0);
			teamEnemy->mSetRelations(static_cast<eTeamID>(kTransportTeam), kTeamNeutral, kGoalIgnore, 1.0);

			for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
			{
				if (i->mInPlay())
				{
					targetShip->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
					targetShip->mSetShipMaxSpeed(0);
				}
			}

			fEnemySurrendered = kTrue;
		}
		else
		{
			int32 iRandomResponse = fMissionInfo->mRandomInt32(k_Enemy_No_Surrender_Random1_msg, k_Enemy_No_Surrender_Random3_msg);
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iRandomResponse);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Pulsar_Interference_msg);
	}
}	

void tBaseVictoryState::DoHailTransport(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (fTransportSurrendered)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Hail_Sevrin_After_Surrender_msg);
		else if (fTransportCaptured)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Nothing_To_Talk_About_msg);
		else if (fSecretSecured)
		{
			tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

			if (!teamEnemy->mShipsRemainingInPlay())
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Hail_Sevrin_No_Enemy);
			else
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Hail_Sevrin_Enemy_Exists);
		}
		else
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Hail_Sevrin_Get_Closer);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Pulsar_Interference_msg);
	}
}	

void tBaseVictoryState::DoDemandSurrenderTransport(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (fTransportSurrendered)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Sevrin_Already_Surrendered_msg);
		else if (fTransportCaptured)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Nothing_To_Talk_About_msg);
		else
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Sevrin_Surrenders_msg);
			tTeamInfo* teamTransport = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTransportTeam));
			// TODO: figure out the float value's meaning
			teamTransport->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamNeutral, kGoalIgnore, 1.0);	
			targetShip->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
			targetShip->mSetShipMaxSpeed(0);
			fTransportSurrendered = kTrue;
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Pulsar_Interference_msg);
	}
}	

void tBaseVictoryState::DoHailSun(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (fSecretSecured)
		{
			if (fTalkedToSun)
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Before_Random_Hail_Sun_msg);	
				int32 iRandomResponse = fMissionInfo->mRandomInt32(k_Hail_Sun_Random1_msg, k_Hail_Sun_Random3_msg);
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iRandomResponse);
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_After_Random_Hail_Sun_msg);
			}
			else
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_First_Sun_Hail_msg);

				mSetTimedInterval(	kTimerSecondsSet, kPrimaryObjectiveCompleteID, 15.0f );
									
				mAddToBonusPrestige(PRESTIGE_HAILED_SUN);
				fTalkedToSun = kTrue;
			}
		}
		else
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Are_You_Ok_msg);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_Pulsar_Interference_msg);
	}
}	

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat ),
	iNumEnemyShipsEscapedWithSecret(0),
	iNumEnemyShipsDefeated(0),
	fSecretSecured(kFalse),
	fEnemyGotTheSecret(kFalse),
	fTalkedToSun(kFalse),
	fPlayerDefeated(kFalse),
	fTractoringTransport(kFalse),
	fScanning(kFalse),
	fEnemySetToFollowTransport(kFalse),
	fEnemySurrendered(kFalse),
	fTransportSurrendered(kFalse),
	fTransportCaptured(kFalse),
	fTransportHailed(kFalse),
	fDeepScannedSun(kFalse),
	fDeepScannedTransport(kFalse)
{}

void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval(kTimerSecondsSet, kCheckSecretTransmission, CHECK_TRANSMISSION_INTERVAL);
}

void tBaseVictoryState::mEndState()
{}

/*
	This method calculates the victory status for the mission. The conditions
	necessary for each victory condition may be enumerated as follows.

	kAstoundingVictory	The player got the secret, prevented the Feds from
						getting it, defeated all of the Fed ships and tested
						the secret by talking to the sun.

	kVictory			The player got the secret, prevented the Feds from
						getting it and defeated all of the Fed ships.

	kDraw				Anything not covered under the specific conditions
						listed for the other victory levels.

	kDefeat				The Feds got the secret and the player failed to
						defeat all the Fed ships. Also, this condition is
						assigned if the player is defeated.

	kDevastatingDefeat	The Feds got the secret, the player failed to defeat
						all the Fed ships and the player was defeated.
*/

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if (fSecretSecured &&
		(!fEnemyGotTheSecret || iNumEnemyShipsEscapedWithSecret == 0) && 
		(iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS))
	{
		if (fTalkedToSun)
			return kAstoundingVictory;
		else
			return kVictory;
	}
	else if ((fEnemyGotTheSecret && (iNumEnemyShipsDefeated < NUM_ENEMY_SHIPS)) || fPlayerDefeated)
		return kDefeat;
	else if (fEnemyGotTheSecret && (iNumEnemyShipsDefeated < NUM_ENEMY_SHIPS) && fPlayerDefeated)
		return kDevastatingDefeat;
	else
		return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	eTeamID teamOriginal = ship->mGetOriginalTeam();
	tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	if (teamOriginal == kPlayerTeam)
	{
		if (!teamPlayer->mShipsRemainingInPlay())
			fPlayerDefeated = kTrue;
	}
	else if (teamOriginal == kTransportTeam)
	{
		if (fSecretSecured)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportDestroyedSecret_msg);
		}
		else
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportDestroyedSansSecret_msg);
		}
	}
	else if (teamOriginal == kEnemyTeam && ship->mGetTeam() == kEnemyTeam )
	{
		if (++iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_2nd_Objective_Complete);

			if (fTalkedToSun)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_All_Objectives_Complete);
		}
	}

	CheckEndMission();
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	eTeamID teamCurrent = ship->mGetTeam();
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if (teamOriginal == kPlayerTeam)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		if (!teamPlayer->mShipsRemainingInPlay())
			fPlayerDefeated = kTrue;
	}
	else if (teamOriginal == kTransportTeam)
	{
		tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

		if (teamCurrent == kPlayerTeam)
		{
			if (!fSecretSecured)
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kCaptured_msg);
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kSecretSecured_msg);
				mAddToBonusPrestige(PRESTIGE_SECRET_SECURED);
				fSecretSecured = kTrue;

				/*
					Change the enemy team relations to attempt to capture the transport
					and destroy the player.
				*/
		
				teamEnemy->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamAtWar, kGoalDestroy, 1.0);
				teamEnemy->mSetRelations(static_cast<eTeamID>(kTransportTeam), kTeamAtWar, kGoalCapture, 1.0);

				/*
					Lift the speed restrictions on the enemy ships so that they have a
					better chance to get the secret and waste the player.
				*/

				for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
				{
					if (i->mInPlay())
						i->mSetShipMaxSpeed(MAX_SHIP_SPEED);
				}

				/*
					Order the transport to get the hell outta dodge, lifting any 
					prior "speed limit" that might have been placed upon it.
				*/

				ship->mSetShipMaxSpeed(MAX_SHIP_SPEED);
				ship->mDisengage(kByMapEdge, kAbsolutePriority);
			}
		}
		else
		{
			if (!fEnemyGotTheSecret)
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemyCapturedTheSecret_msg);
				fEnemyGotTheSecret = kTrue;

				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemyDisengaging_msg);
				ship->mDisengage(kByMapEdge, kAbsolutePriority);
				
				for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
				{
					if (i->mInPlay())
						i->mDisengage(kByMapEdge, kAbsolutePriority);
				}
			}
		}
	}
	else if (teamOriginal == kEnemyTeam)
	{
		tEnemyShip* pshipEnemy = static_cast<tEnemyShip*>(ship);
		pshipEnemy->SetCaptured();

		if (++iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_2nd_Objective_Complete);

			if (fTalkedToSun)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_All_Objectives_Complete);
		}
	}

	CheckEndMission();
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID teamCurrent = ship->mGetTeam();
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if ((teamCurrent == kEnemyTeam) && fEnemyGotTheSecret)
		iNumEnemyShipsEscapedWithSecret++;
	else if ((teamOriginal == kTransportTeam) && !fSecretSecured && !fTractoringTransport)
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportLeftFailure_msg);

	CheckEndMission();
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}

void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	// TODO: remove this hack once the problem is fixed.

	if (ship == NULL)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tShipInfo* shipPlayer= teamPlayer->mFindShipCustomID(tPlayerTeam::kPlayerShip1);
		Assert(shipPlayer != NULL);
		ship = shipPlayer;
	}

	switch (buttonID)
	{
		case k_HAIL_ENEMY_btn:
			DoHailEnemy(ship, targetShip);
			break;
		case k_DS_ENEMY_btn:
			DoDemandSurrenderEnemy(ship, targetShip);
			break;
		case k_HAIL_Transport_btn:
			DoHailTransport(ship, targetShip);
			break;
		case k_Demand_Surrender_Transport_btn:
			DoDemandSurrenderTransport(ship, targetShip);
			break;
		case k_HAIL_Sun_btn:
			DoHailSun(ship, targetShip);
			break;
	}
}

void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if (deepScanOn)
	{
		fScanning = true;
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kScanning_msg);
		mSetTimedInterval(kTimerTurnsSet, kScanTimer, SCAN_TIMER_INTERVAL);
	}
	else
	{
		fScanning = false;
		mSetTimedInterval(kTimerKillTimer, kScanTimer, 0.0f);
	}
}

void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}

void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	int32 iShipID = tractoringShip->mGetCustomID();
	int32 iCargoID = tractorTarget->mGetCustomID();

	if (iShipID == tPlayerTeam::kPlayerShip1 && iCargoID == tTransportTeam::kSevrinTransportID)
		fTractoringTransport = kTrue;
}

void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	int32 iShipID = tractoringShip->mGetCustomID();
	int32 iCargoID = tractorTarget->mGetCustomID();

	if (iShipID == tPlayerTeam::kPlayerShip1 && iCargoID == tTransportTeam::kSevrinTransportID)
		fTractoringTransport = kFalse;
}

void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
/*
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat2_msg );	break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat3_msg );	break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat4_msg );	break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat5_msg );	break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat6_msg );	break;
		}

		fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID,
										( float )( fMissionInfo->mRandomInt32( 30, 50 ) ) );
	}
#endif
*/
	if( timerID == kPrimaryObjectiveCompleteID )
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_1st_Objective_Complete);
		if ( iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k_All_Objectives_Complete);
		}
	}

	if (timerID == kCheckSecretTransmission)
	{
		if (!fEnemySetToFollowTransport)
			fEnemySetToFollowTransport = OrderEnemyToFollowTransport();

		iTruth fSevrinTransmits = kFalse;
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tTeamInfo* teamTransport = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTransportTeam));
		tShipInfo* pTransport = teamTransport->mFindShipCustomID(tTransportTeam::kSevrinTransportID);

		if (NULL != pTransport && pTransport->mGetTeam() == kTransportTeam)
		{
			for (tShipIterator i = teamPlayer->mGetFirstShip(); *i != NULL; i++)
			{
				if (i->mInPlay())
				{
					float fDistance = i->mGetTacticalDistance(pTransport);
					if (fDistance <= SECRET_TRANSMISSION_RANGE)
						fSevrinTransmits = kTrue;
				}
			}
		}

		if (fSevrinTransmits)
		{
			fSecretSecured = kTrue;
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kSecretSecured_msg);

			// TODO: figure out if we ought to lift the enemy speed limit.

			/*
			if (!fEnemySurrendered)
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemyDisengaging_msg);
				
				for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
				{
					if (i->mInPlay())
						i->mDisengage(kByMapEdge, kAbsolutePriority);
				}
			}
			*/

			// Set Sevrin's ship to disengage unless he's surrendered.

			if (!fTransportSurrendered)
			{
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportDisengaging_msg);
				pTransport->mDisengage(kByMapEdge, kAbsolutePriority);
			}
		}
		else
		{
			mSetTimedInterval(kTimerSecondsSet, kCheckSecretTransmission, CHECK_TRANSMISSION_INTERVAL);
		}
	}

	/*
		This case handles the scan completion notification. The player is
		awarded some points for their first deep scan of relevant objects in
		the system.
	*/

	else if ((timerID == kScanTimer) && fScanning)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tShipInfo* shipPlayer = teamPlayer->mCurrentlyControlledShip();
		Assert(NULL != shipPlayer);
		tShipInfo* shipTarget = GetCurrentShipTarget(shipPlayer);

		if( shipTarget )
		{
			if (shipTarget->mGetCustomID() == tTransportTeam::kSevrinTransportID)
			{
				if (!fDeepScannedTransport)
				{
					fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDeepScannedTransport_msg);
					mAddToBonusPrestige(PRESTIGE_DEEP_SCAN);
					fDeepScannedTransport = true;
				}
			}
			else if (shipTarget->mGetCustomID() == tSunTeam::kSunShip1)
			{
				if (!fDeepScannedSun)
				{
					fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDeepScannedSun_msg);
					mAddToBonusPrestige(PRESTIGE_DEEP_SCAN);
					fDeepScannedSun = true;
				}
			}
		}
		// Reset the timer.
		mSetTimedInterval(kTimerTurnsSet, kScanTimer, SCAN_TIMER_INTERVAL);
	}

	CheckEndMission();
}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
