/*
	TransportBaseState.cpp

	This module contains the definition for the tTransportBaseState class,
	which handles the capturing and damage of transports.
*/

#include <limits>
#include "ISC_X4M1BThief.h"
#include "TransportBaseState.h"
#include "MissionText.h"
#include "PlayerTeam.h"

/*
	This method assumes that the transport has been captured by the player. It
	checks the current level of damage and sets the current comm state for
	the transport accordingly.
*/

void tTransportBaseState::SetCommStateForDamage(tTransport* pTransport)
{
	float fDamage = pTransport->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	// Note: when testing for zero damage, it is necessary to test against
	// the C++ library defined lower limit due to float precision. In short,
	// the first case of the if statement is for zero damage.

	if (fDamage < std::numeric_limits<float>::epsilon())
		playerTeam->mGotoCommState(pTransport->mGetShipID(), tTransport::kCommState3);
	else if (fDamage < 30.0f)
		playerTeam->mGotoCommState(pTransport->mGetShipID(), tTransport::kCommState4);
	else if (fDamage < 60.0f)
		playerTeam->mGotoCommState(pTransport->mGetShipID(), tTransport::kCommState5);
	else
		playerTeam->mGotoCommState(pTransport->mGetShipID(), tTransport::kCommState6);
}	

/*
	This method assumes that the transport has been captured by the player. It
	checks any remaining defense platforms and sends the transport to the
	nearest one. If no defense platforms remain, then it orders the transport
	to follow the player's ship for safety's sake.
*/

void tTransportBaseState::SendTransportToSafety(tTransport* pTransport)
{
	tShipInfo* pNearestPlatform = NULL;
	tTeamInfo* teamPlatform = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlatformTeam));
	float fNearestDistance = FLT_MAX;

	for (tShipIterator i = teamPlatform->mGetFirstShip(); *i != NULL; i++)
	{
		if ((*i)->mInPlay())
		{
			float fDistance = pTransport->mGetTacticalDistance((*i));
			if (fDistance < fNearestDistance)
			{
				fNearestDistance = fDistance;
				pNearestPlatform = (*i);
			}
		}
	}

	if (pNearestPlatform != NULL)
	{
		pTransport->mFollowEntity(pNearestPlatform, kAbsolutePriority);
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportHeadingForCover);
	}
	else
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tShipInfo* pPlayerShip = teamPlayer->mFindShipCustomID(tPlayerTeam::kPlayerShip);
		Assert(pPlayerShip != NULL);
		pTransport->mFollowEntity(pPlayerShip, kAbsolutePriority);
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportFollowingForCover);
	}
}

void tTransportBaseState::mBeginState()
{}
void tTransportBaseState::mEndState()
{}
void tTransportBaseState::mOnDeath( tShipInfo* ship )
{}

/*
	This method is responsible for handling the capture of a transport.
*/

void tTransportBaseState::mOnCapture(tShipInfo* ship)
{
	if (fChildActor->mGetCustomID() == ship->mGetCustomID())
	{
		/*
			If this is the first time the transport has been captured, then
			we need to tend to a few things:

			(1) The ship maximum speed must be restored to 31 so that it can move.
			(2) The ship shields must be restored.
			(3) A message should be provided to acknowledge the capture.
		*/

		iTruth bInitialCapture = fChildActor->IsInitialCapture();

		if (bInitialCapture)
		{
			fChildActor->SetInitialCapture();
			fChildActor->mSetShipMaxSpeed(31);
			fChildActor->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 1.0f);
		}

		// Get the current team, which should be either the player or the enemy.

		eTeamID teamCurrent = fChildActor->mGetTeam();
		Assert(teamCurrent == kPlayerTeam || teamCurrent == kEnemyTeam);

		/*
			If the capture is by the player team, then we need to configure the
			ship to head to safety and defend itself if needed. We must also
			set the proper comm state.
		*/

		if (teamCurrent == kPlayerTeam)
		{
			if (fChildActor->WasPreviouslyCapturedByPlayer())
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportCapturedByPlayerSubsequent);
			else
			{
				// TODO: should we fortify the transport with extra marines?
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportCapturedByPlayerInitial);
				fChildActor->SetCapturedByPlayer();
			}

			tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
			SetCommStateForDamage(fChildActor);
			SendTransportToSafety(fChildActor);
		}

		/*
			On the other hand, if the capture is by the enemy team, then we need
			to tell the ship to disengage at best possible speed and set the
			proper comm state.
		*/

		else if (teamCurrent == kEnemyTeam)
		{
			tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
			fChildActor->mDisengage(kByMapEdge, kAbsolutePriority);
			playerTeam->mGotoCommState(fChildActor->mGetShipID(), tTransport::kCommState2);
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportCapturedByEnemy);
		}
	}
}

void tTransportBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tTransportBaseState::mAtDistance( tShipInfo* ship )
{}
void tTransportBaseState::mAtDestination( tShipInfo* ship )
{}
void tTransportBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}

/*
	This method is responsible for handling damage to the transports after
	they've been captured.
*/

void tTransportBaseState::mOnDamage(tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType)
{
	if (fChildActor->mGetCustomID() == damagedShip->mGetCustomID() &&
		fChildActor->mGetTeam() == kPlayerTeam)
	{
		SetCommStateForDamage(fChildActor);
		damagedShip->mSetShipCallbackEvents(kNotifyDamage);
	}
}

void tTransportBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTransportBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTransportBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnFlashingButtonPressed()
{}
void tTransportBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTransportBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tTransportBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tTransportBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tTransportBaseState::mOnHostChanged()
{}
void tTransportBaseState::mOnTimedInterval(int32 timerID)
{}
void tTransportBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTransportBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
