/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	for ( int i = 0; i < kNumStartingPositions; i++ )
	{
		fShipsDestroyed[i] = 0;
		fTeamWon[i] = true;
		fPosUsed[i] = kFalse;
		fStartPos[i] = -1;
		fNumAdjacentEmptySlots[i] = -1;
	}
	fNumTeams = 0;
}

/*

	Purpose:	Will end the game upon when only one team is left.  Essentially, all players (human or
				otherwise) are initialized as already won.  As ships are knocked out of the game, they
				are set to a losing condition and their game ends and should theorectically end when
				there is only one ship left standing.  However, since teams can be dynamically changed
				according to the player, rather than check for one ship standing, we need to check if all
				of the ships standing are from the same team tag.  If so, then we run through each of those
				remaining teams and set a winning victory condition for each.

*/
void tWorldConditions::mCheckForGameEnd( void )
{

	iTruth	remainingShipsOnSameTeam = true;
	int32	lastWinningTeam = -1;

	tMissionInfo*	thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	Find out if all the ships left standing are from the same team.
	//
	for ( int i = 0; i < kMaxTeams; i++ )
	{
		//
		//	If we already have found a winning team, then lastWinningTeam must be the most recent winning
		//	team we have found.
		//
		if ( lastWinningTeam != -1 )
		{
			tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			Assert( currentTeam );
			if ( !currentTeam )
				continue;
			tTeamInfo* previousTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( lastWinningTeam ) );
			Assert( previousTeam );
			if ( !previousTeam )
				continue;

			//
			//	If the two ships do not match team tags, then we know that a fight is still brewing and
			//	we should not end any games as winners yet.
			//
			if ( mTeamWon( i ) )
			{
				if ( currentTeam->mGetTag() != previousTeam->mGetTag() )
				{
					remainingShipsOnSameTeam = false;
					break;
				}
			}
		}

		//
		//	If the current team still has the won flag standing, then save the ID so that we can compare
		//	on the next pass through.
		//
		if ( mTeamWon( i ) )
		{
			lastWinningTeam = i;
		}
	}

	//
	//	If we have hit a winning condition (all ships remaining on same team), then we should end the game
	//	for all remaining players with a winning condition.
	//
	if ( remainingShipsOnSameTeam )
	{
		const int maxPlayers = 6;
		i = 0;
		int TeamCounter = 0;
		while( i < fNumTeams && TeamCounter < kMaxTeams )
		{
			tTeamInfo* Team = gScriptInterface.fScript->mMission()->mGetTeamHandle( eTeamID(TeamCounter) );
			if ( mTeamWon( i ) )
			{
				if(Team)
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( TeamCounter ), kPlayerWins );
					i++;
				}
			}
			else
			{
				if(Team)
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( TeamCounter ), kPlayerLoses );
					i++;
				}
			}
			TeamCounter++;
		}
	}
}

/*

	Purpose:	Returns a random starting position for each of the ships.  Only once all starting positions
				are used up (G - Z), will a starting position be available for reuse.

*/
iMapPosition tWorldConditions::mGenerateRandomStartPosition( void )
{
	//
	//	Make certain that there is at least one starting point that is still valid.  If not, then reset
	//	all starting points to be valid.
	//
	iTruth	valid = kFalse;
	for ( int i = 0; ( i < kNumStartingPositions ) && ( !valid ) ; i++ )
	{
		if ( !fPosUsed[i] )
			valid = kTrue;
	}
	if ( !valid )
	{
		for ( i = 0; i < kNumStartingPositions; i++ )
		{
			fPosUsed[i] = kFalse;
		}
	}

	//
	//	Now find a valid starting point.
	//
	while ( true )
	{
		int32 guess = gScriptInterface.mMission()->mRandomInt32( 0, kNumStartingPositions - 1 );
		Assert ( ( guess >= 0 ) && ( guess < kNumStartingPositions ) );
		if ( !fPosUsed[guess] )
		{
			fPosUsed[guess] = kTrue;
			return static_cast< iMapPosition >( guess + kStartPosition_G );
		}
	}

	//
	//	Should never get here.
	//
	return -1;
}

/*

	Purpose:	Assigns an initial spawning position for each team based on each teams' side.  The order of
				starting positions is 'G' - 'I' - 'J' - 'H' - 'K' - 'L'.

*/
iMapPosition tWorldConditions::mGenerateStartPosition( int team )
{
	//
	//	Search for a team match in the starting positions.
	//
	int match = -1;
	for ( int i = 0; i < kNumStartingPositions; i++ )
	{
		if ( fStartPos[i] == team )
		{
			match = i;
			break;
		}
	}

	//
	//	If there is a match, then look before and after the match for the next empty slot.
	//
	int arrayPosition = 0;
	int posTry = 0;
	if ( match != -1 )
	{
		int count = 0;
		while ( count <= ( kNumStartingPositions >> 1 ) )
		{
			//
			//	Before.
			//
			posTry = ( ( ( match + kNumStartingPositions ) - count ) % kNumStartingPositions );
			if ( fStartPos[posTry] == -1 )
			{
				arrayPosition = posTry;
				break;
			}

			//
			//	After.
			//
			posTry = ( ( match + count ) % kNumStartingPositions );
			if ( fStartPos[posTry] == -1 )
			{
				arrayPosition = posTry;
				break;
			}

			//
			//	If we got this far, then there was no empty slot, and the count should be incremented.
			//
			count++;
		}
	}
	//
	//	If there is not a match, then find a slot that is surrounded by the most empty slots.
	//
	else
	{
		//
		//	Find what the greatest number of equally empty slots is next to any given empty slot.
		//
		int highestNumEmptySlots = 1;		
		for ( int i = 0; i < kNumStartingPositions; i++ )
		{
			fNumAdjacentEmptySlots[i] = -1;
			if ( fStartPos[i] != -1 )
				continue;

			int curNumEmptySlots = 0;
			int offsetCount = 0;
			int beforeGood = 1;
			int afterGood = 1;
			while ( ( offsetCount < kNumStartingPositions ) && beforeGood && afterGood )
			{
				beforeGood = afterGood = 0;

				//
				//	Before.
				//
				posTry = ( ( ( i + kNumStartingPositions ) - offsetCount ) % kNumStartingPositions );
				if ( fStartPos[posTry] == -1 )
				{
					curNumEmptySlots++;
					beforeGood = 1;
				}

				//
				//	After.
				//
				posTry = ( ( i + offsetCount ) % kNumStartingPositions );
				if ( fStartPos[posTry] == -1 )
				{
					curNumEmptySlots++;
					afterGood = 1;
				}

				//
				//	If we got this far, then there was no empty slot, and the count should be incremented.
				//
				offsetCount++;

				fNumAdjacentEmptySlots[i] = curNumEmptySlots;
				if ( curNumEmptySlots >= highestNumEmptySlots )
				{
					highestNumEmptySlots = curNumEmptySlots;
				}
			}
		}

		//
		//	Now that we know the max amount of empty slots, then we should find it and make that the
		//	new array position.
		//
		for ( i = 0; i < kNumStartingPositions; i++ )
		{
			if ( fNumAdjacentEmptySlots[i] >= highestNumEmptySlots )
			{
				arrayPosition = i;
				break;
			}
		}
	}

	//
	//	Take the slot number that was figured out and convert it to the proper iMapPosition type.
	//
	iMapPosition mapPosition = kStartPosition_G;
	switch ( arrayPosition )
	{
	case 0:
		mapPosition = kStartPosition_G;		break;
	case 1:
		mapPosition = kStartPosition_I;		break;
	case 2:
		mapPosition = kStartPosition_J;		break;
	case 3:
		mapPosition = kStartPosition_H;		break;
	case 4:
		mapPosition = kStartPosition_K;		break;
	case 5:
		mapPosition = kStartPosition_L;		break;
	}
	fStartPos[arrayPosition] = team;
	return mapPosition;
}

int32 tWorldConditions::mShipsDestroyed( int x )
{
	return ( x >= 0 && x <= 5 )
		? fShipsDestroyed[x] : 0 ;
}
bool tWorldConditions::mTeamWon( int x )
{
	return ( x >= 0 && x <= 5 )
		? fTeamWon[x] : 0;
}

void tWorldConditions::mIncrementShipsDestroyed( int x )
{
	if ( x < 0 || x > 5 )
		return;
	fShipsDestroyed[x]++;
}

void tWorldConditions::mSetTeamWon( int x, bool y )
{
	if ( x < 0 || x > 5 )
		return;
	fTeamWon[x] = y;
}