/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackTeam.h"
#include "BaseVictoryState.h"
#include "DefendTeam.h"
#include "MissionText.h"
#include "Mul_2BaseAssault.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "WorldConditions.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( ID );
	Assert( team );
	if ( team )
	{
		//
		//	Find out which team the argument ID is on.
		//
		if ( team->mGetTag() == kAttackTag )
		{
			if( gWC.mStarbaseDestroyed() )
			{
				return kVictory;
			}
			else
			{
				return kDefeat;
			}
		}
		else if ( team->mGetTag() == kDefendTag )
		{
			if ( gWC.mAttackShipsDestroyed() >= gWC.mTotalAttackShips() )
			{
				if ( gWC.mDefendShipsDestroyed() >= gWC.mTotalDefendShips() )
				{
					return kDraw;
				}
				else
				{
					return kVictory;
				}
			}
			else
			{
				return kDefeat;
			}
		}
	}

	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kDefenderStarbase:
		{
			//
			//	The starbase has been destroyed, so the game should end.
			//
			gWC.mSetStarbaseDestroyed( true );			
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			tAttackTeam* attackTeam = dynamic_cast< tAttackTeam* >( team );
			Assert( attackTeam );
			if ( !attackTeam )
				return;

			attackTeam->fNumShips--;
			gWC.mIncrementAttackShipsDestroyed();

			if ( !attackTeam->mShipsRemainingInPlay() )
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			tDefendTeam* defendTeam = dynamic_cast< tDefendTeam* >( team );
			Assert( defendTeam );
			if ( !defendTeam )
				return;

			defendTeam->fNumShips--;
			gWC.mIncrementDefendShipsDestroyed();

			if ( !defendTeam->mShipsRemainingInPlay() )
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );

			//
			//	We cannot end the game for the defending team if that team is out of ships because
			//	the could not be destroyed yet and if all of the defenders are considered done and
			//	there is only 1 human-controlled attacker left, then the game would end prematurely
			//	before the attacker could have a chance to destroy the starbase.  As a result, a
			//	defender's game is ended in gWC.mCheckForGameEnd() below.
			//
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kDefenderStarbase:
		{
			//
			//	There is no way in the world that a starbase is going to disengage.  If it does,
			//	get a good night's rest.
			//			
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			//
			//	Disengaging is considered to be the same as death in this mission.  However, sometimes
			//	the AI could fly outside the boundaries into the "disengaging" area and then come
			//	back in.  In this case, we do not want to count this as a legitimate disengage.
			//
			tAttackTeam* attackTeam = dynamic_cast< tAttackTeam* >( team );
			Assert( attackTeam );
			if ( !attackTeam )
				return;

			attackTeam->fNumShips--;
			gWC.mIncrementAttackShipsDestroyed();

			if ( !attackTeam->mShipsRemainingInPlay() )
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );

			if ( !team->mIsTeamHumanControlled() )
			{
				ship->mDisengage( kByMapEdge, kAbsolutePriority );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			//
			//	Disengaging is considered to be the same as death in this mission.  However, sometimes
			//	the AI could fly outside the boundaries into the "disengaging" area and then come
			//	back in.  In this case, we do not want to count this as a legitimate disengage.
			//
			tDefendTeam* defendTeam = dynamic_cast< tDefendTeam* >( team );
			Assert( defendTeam );
			if ( !defendTeam )
				return;

			defendTeam->fNumShips--;
			gWC.mIncrementDefendShipsDestroyed();

			if ( !defendTeam->mShipsRemainingInPlay() )
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );

			//
			//	We cannot end the game for the defending team if that team is out of ships because
			//	the could not be destroyed yet and if all of the defenders are considered done and
			//	there is only 1 human-controlled attacker left, then the game would end prematurely
			//	before the attacker could have a chance to destroy the starbase.  As a result, a
			//	defender's game is ended in gWC.mCheckForGameEnd() below.
			//

			if ( !team->mIsTeamHumanControlled() )
			{
				ship->mDisengage( kByMapEdge, kAbsolutePriority );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( TeamID );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
