/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "EnemyShip.h"
#include "Star.h"
#include "NPC1Team.h"
#include "AlienDerelict.h"
#include "Uni_X3M3AStar.h"

tShipInfo* tBaseVictoryState::GetCurrentShipTarget(tShipInfo* ship)
{
	int32 ID = ship->mGetShipTargetID();
	assert(NULL != ID);
	tShipInfo* shipTarget = fMissionInfo->mFindShipID(ID);
	assert(NULL != shipTarget);
	return shipTarget;
}

void tBaseVictoryState::DestroyAllEnemyShips()
{
	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

	for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
	{
		if (i->mInPlay())
			i->mDestroyShip();
	}
}

void tBaseVictoryState::EndMission(eGameStatus Status)
{
	gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, Status);
}

void tBaseVictoryState::CheckEndMission()
{
	tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	if (!teamPlayer->mShipsRemainingInPlay())
	{
		/*
			The mission must be ended but with the proper result code to queue the
			proper music and so forth.
		*/

		eVictoryLevels VictoryLevel = mCalcVictoryStatus(static_cast<eTeamID>(kPlayerTeam));
		if (VictoryLevel == kAstoundingVictory || VictoryLevel == kVictory)
			EndMission(kPlayerWins);
		else
			EndMission(kPlayerLoses);
	}

	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
	if (!teamEnemy->mShipsRemainingInPlay() && fChoiceMade)
		EndMission(kPlayerWins);
}

/*
	This method calculates the average damage level of the enemy fleet at
	the time it is called.
*/

float tBaseVictoryState::CalculateAverageEnemyDamage()
{
	// TODO: consider revising this to account for the number of ships destroyed.

	float fTotalDamage = 0.0f;
	int iNumShips = 0;
	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

	for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
	{
		if ((*i)->mInPlay())
		{
			fTotalDamage += (*i)->mGetDamagePercent();
			iNumShips++;
		}
	}

	return fTotalDamage / static_cast<float>(iNumShips);
}	

/*
	This method handles pressing the hail button when an enemy ship is targeted.
*/

void tBaseVictoryState::DoHailEnemy(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		tEnemyShip* pshipEnemy = static_cast<tEnemyShip*>(targetShip);
		assert(pshipEnemy != NULL);

		if (fEnemySurrendered)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemy_Already_Surrendered_msg);
		else if (pshipEnemy->IsCaptured())
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kNothing_To_Talk_About_msg);
		else
		{
			int32 iRandomResponse = fMissionInfo->mRandomInt32(kEnemy_Hail_Random1_msg, kEnemy_Hail_Random3_msg);
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iRandomResponse);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}	

/*
	This method handles pressing the demand surrender button when an enemy 
	ship is targeted.
*/

// TODO: should be add prestige awards to this somehow?

void tBaseVictoryState::DoDemandSurrenderEnemy(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (fEnemySurrendered)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDone_Enough_msg);
		else if (CalculateAverageEnemyDamage() >= SURRENDER_THRESHOLD)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemy_Surrenders_msg);
			tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
			// TODO: figure out the float value's meaning
			teamEnemy->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamNeutral, kGoalIgnore, 1.0);	

			for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
			{
				if (i->mInPlay())
				{
					targetShip->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
					targetShip->mSetShipMaxSpeed(0);
				}
			}

			fEnemySurrendered = kTrue;
		}
		else
		{
			int32 iRandomResponse = fMissionInfo->mRandomInt32(kEnemy_No_Surrender_Random1_msg, kEnemy_No_Surrender_Random3_msg);
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iRandomResponse);
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}	

/*
	This method handles pressing the hail button when a star is targeted.
*/

void tBaseVictoryState::DoHailStar(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		tStar* pStar = static_cast<tStar*>(targetShip);
		assert(NULL != pStar);

		if (pStar->IsTalkedTo())
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kStar_Same_Sound_msg);
		}
		else
		{
			int32 iMsg = static_cast<int32>(kStar_Hail1_msg) + iNumStarsTalkedTo;
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iMsg);
			pStar->SetTalkedTo();

			/* 
				If the player has now talked to all the stars, then re-configure the
				comm panel for the alien derelict.
			*/

			if (++iNumStarsTalkedTo == NUM_PULSARS)
			{
				tTeamInfo* teamDerelict = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kNPC1Team));
				tShipInfo* pshipDerelict = teamDerelict->mFindShipCustomID(tNPC1Team::kAlienDerelict);
				assert(NULL != pshipDerelict);
				int32 iShipID = pshipDerelict->mGetShipID();
				tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
				teamPlayer->mGotoCommState(iShipID, tAlienDerelict::kCommState2);
			}
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}	

/*
	This method handles pressing the hail button when the alien derelict 
	is targeted.
*/

void tBaseVictoryState::DoHailDerelict(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (fChoiceMade)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDerelict_Choice_Made_msg);
		else if (iNumStarsTalkedTo < NUM_PULSARS)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDerelict_Hail_msg);
		else
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDerelict_Hail_Suggest_Code_msg);
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}

/*
	This method handles pressing the transmit code button when the alien derelict
	is targeted.
*/

void tBaseVictoryState::DoTransmitCodeToDerelict(tShipInfo* ship, tShipInfo* targetShip)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		if (!fTalkedToDerelict)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDerelict_Transmit_Code_msg);
			mAddToBonusPrestige(PRESTIGE_TRANSMIT_CODE);
			fTalkedToDerelict = kTrue;
		}
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}	

void tBaseVictoryState::DoChoice(tShipInfo* ship, tShipInfo* targetShip, int32 iButtonID)
{
	float fDistance = ship->mGetTacticalDistance(targetShip);
	if (fDistance <= COMMUNICATION_RANGE)
	{
		int32 iChoiceMsg;
		int32 iShipID = targetShip->mGetShipID();
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

		switch (iButtonID)
		{
			case kDerelict_Youth_btn:
				iChoiceMsg = kDerelict_Chose_Youth_msg;
				// TODO: set the immortality flag somehow.
				break;
			case kDerelict_YthPres_btn:
//			case kDerelict_Prestige_btn:
				iChoiceMsg = kDerelict_Chose_Prestige_msg;
				mAddToBonusPrestige(PRESTIGE_CHOSE_PRESTIGE);
				break;
			case kDerelict_Knowledge_btn:
				iChoiceMsg = kDerelict_Chose_Knowledge_msg;
				// TODO: figure out how to bump crew capabilities.
				break;
			case kDerelict_KnowVic_btn:
//			case kDerelict_Victory_btn:
				/*
					If the player has already defeated all of the ISC ships,
					then give him a break. Don't let him waste his choice
					on blowing up things that aren't there.
				*/
				if (iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
				{
					iChoiceMsg = kDerelict_Already_Victorious_msg;
					fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iChoiceMsg);
					playerTeam->mGotoCommState(iShipID, tAlienDerelict::kCommState3);
					return;
				}
				else
				{
					iChoiceMsg = kDerelict_Chose_Victory_msg;
					DestroyAllEnemyShips();
				}
				break;
		}

		// Re-configure the comm panel for the derelict.

		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iChoiceMsg);
		playerTeam->mGotoCommState(iShipID, tAlienDerelict::kCommState1);

		// Show a mission objective complete message.

		if (iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			/* 
				If the mission is complete, fire a timer to give the player time
				to read the comm traffic.
			*/

			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAll_Objectives_Complete_msg);
			mSetTimedInterval(kTimerSecondsSet, kCheckEndMissionTimer, 7.0f);
		}
		else
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k2nd_Objective_Complete_msg);
		}

		fChoiceMade = kTrue;
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kPulsar_Interference_msg);
	}
}

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat ),
	fEnemySurrendered(kFalse),
	fTalkedToDerelict(kFalse),
	fChoiceMade(kFalse),
	fPlayerDefeated(kFalse),
	fScanning(kFalse),
	iNumStarsTalkedTo(0),
	iNumEnemyShipsDefeated(0)
{}

void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	iTruth fDefeatedTheEnemy = (iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS) ? kTrue : kFalse;
	if (fChoiceMade)
		return fDefeatedTheEnemy ? kVictory : kDraw;
	else
		return fDefeatedTheEnemy ? kDefeat : kDevastatingDefeat;
}

//
//	Ship events
//

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	eTeamID teamOriginal = ship->mGetOriginalTeam();
	tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	if (teamOriginal == kPlayerTeam)
	{
		if (!teamPlayer->mShipsRemainingInPlay())
			fPlayerDefeated = kTrue;
	}
	else if (teamOriginal == kEnemyTeam)
	{
		mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED);

		if (++iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			// Give an extra prestige boost for defeating all the enemy ships.

			mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED);

			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k1st_Objective_Complete_msg);
			if (fChoiceMade)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAll_Objectives_Complete_msg);
		}
	}

	CheckEndMission();
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	eTeamID teamCurrent = ship->mGetTeam();
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if (teamOriginal == kPlayerTeam)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		if (!teamPlayer->mShipsRemainingInPlay())
			fPlayerDefeated = kTrue;
	}
	else if (teamOriginal == kEnemyTeam)
	{
		mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED);

		if (++iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
		{
			// Give an extra prestige boost for defeating all the enemy ships.

			mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED);

			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), k1st_Objective_Complete_msg);
			if (fChoiceMade)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAll_Objectives_Complete_msg);
		}
	}

	CheckEndMission();
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	CheckEndMission();
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}

void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}

void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	// TODO: remove this hack once the problem is fixed.

	if (ship == NULL)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tShipInfo* shipPlayer= teamPlayer->mFindShipCustomID(tPlayerTeam::kPlayerShip1);
		assert(shipPlayer != NULL);
		ship = shipPlayer;
	}

	switch (buttonID)
	{
		// Comm handling for the enemy ships.

		case kEnemy_Hail_btn:
			DoHailEnemy(ship, targetShip);
			break;
		case kEnemy_Surrender_btn:
			DoDemandSurrenderEnemy(ship, targetShip);
			break;

		// Comm handling for the stars.
		case kStar_Hail_btn:
			DoHailStar(ship, targetShip);
			break;

		// Comm handling for the alien derelict.

		case kDerelict_Hail_btn:
			DoHailDerelict(ship, targetShip);
			break;
		case kDerelict_Code_btn:
			DoTransmitCodeToDerelict(ship, targetShip);
			break;

//		case kDerelict_Youth_btn:
//		case kDerelict_Prestige_btn:
//		case kDerelict_Knowledge_btn:
//		case kDerelict_Victory_btn:
		case kDerelict_YthPres_btn:
		case kDerelict_KnowVic_btn:
			DoChoice(ship, targetShip, buttonID);
			break;
	}
}

void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if (deepScanOn)
	{
		fScanning = true;
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kScanning_msg);
		mSetTimedInterval(kTimerTurnsSet, kScanTimer, SCAN_TIMER_INTERVAL);
	}
	else
	{
		fScanning = false;
		mSetTimedInterval(kTimerKillTimer, kScanTimer, 0.0f);
	}
}

void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kCheckEndMissionTimer)
		CheckEndMission();
	else if ((timerID == kScanTimer) && fScanning)
	{
		tTeamInfo* teamPlayer = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		tShipInfo* shipPlayer = teamPlayer->mCurrentlyControlledShip();
		assert(NULL != shipPlayer);
		tShipInfo* shipTarget = GetCurrentShipTarget(shipPlayer);

		if (shipTarget->mGetTeam() == kNPC1Team)
		{
			if (shipTarget->mGetCustomID() == tNPC1Team::kAlienDerelict)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDerelict_Scan_Result_msg);
			else
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kStar_Scan_Result_msg);
		}

		// Reset the timer.
		//mSetTimedInterval(kTimerTurnsSet, kScanTimer, SCAN_TIMER_INTERVAL);
	}

}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
