//	<Name>	Actor.cpp
//
//	<Info>	DESCRIPTION
//

#pragma warning( disable:4800 )
#include "tclient.h"
#include "State.h"
#include "sfcAssert.h"
#include "ScriptInterface.h"

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tActor::tActor( void ) :
	fCurState( NULL ), 
	fNextStateID( 0 )
{
	for( int32 i = 0; i < kMaxStates; i++ )
	{
		fStateIDLookup[i] = NULL;
	}

	fMissionInfo = gScriptInterface.mMission();
}

/*
	Purpose:	Destructor

	Inputs:		N/A
	Outputs:	N/A
*/
tActor::~tActor( void )
{
	mEndCurState();

	for( tStateMap::iterator i = fStateMap.begin();
		 i != fStateMap.end();
		 i++ )
	{
		if( i->second )
		{
			delete i->second; 
		}
	}

	for( int32 index = 0; index < kMaxStates; index++ )
	{
		fStateIDLookup[index] = NULL;
	}
}

/*
	Purpose:	Although, this does nothing now, it can be overridden to register states.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tActor::mSetupState( void )
{}

/*
	Purpose:	Although, this does nothing now, it can be overridden to register comm states.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tActor::mSetupCommState( void )
{}

/*
	Purpose:	By registering a state, all actors will add the intended state to their list.

	Inputs:		( tState* )	-	The state in question
	Outputs:	N/A
*/
void
tActor::mRegisterState( tState* State )
{
	Assert( fNextStateID < kMaxStates );

	if( fNextStateID >= kMaxStates )
	{
		return;
	}

	State->mSetup( this, fNextStateID );
	
	fStateIDLookup[fNextStateID] = State;
	const type_info& stateTypeID = typeid( *State );
	fStateMap[ stateTypeID.raw_name() ] = State;
	
	fNextStateID++;
}

/*
	Purpose:	Accessor for the current state of an actor.

	Inputs:		( iStateID )	-	The ID associated with the state in question
	Outputs:	( tState* )		-	The pointer to the state requested
*/
tState*
tActor::mGetState( iStateID StateID ) const
{
	return fStateIDLookup[StateID];
}

/*
	Purpose:	Tells an actor to link itself to a state.

	Inputs:		( type_info& )	-	Constant reference to the "typeid()" associated with the state
									in question
	Outputs:	N/A
*/
void
tActor::mGotoState( const type_info& StateTypeID )
{
	const char* stateName = StateTypeID.raw_name();

	//
	//	Tell the world that we are leaving this state.
	//
	if ( fCurState )
	{
		tEventMgr::tEventInfo prevEventInfo;
		prevEventInfo.fEvent = kNotifyStateChanged;
		prevEventInfo.fEventItem[0] = ( int32 )this;
		prevEventInfo.fEventItem[1] = fCurState->mGetStateID();
		prevEventInfo.fEventItem[2] = tState::kEndingState;
		gEventMgr.mPostEvent( prevEventInfo );
	}
	
	mEndCurState();
	
	fCurState = fStateMap[stateName];
	Assert( fCurState );
	
	fCurState->mBeginState();

	//
	//	Tell the world that we our starting this state
	//
	tEventMgr::tEventInfo eventInfo;
	eventInfo.fEvent = kNotifyStateChanged;
	eventInfo.fEventItem[0] = (int32)this;
	eventInfo.fEventItem[1] = fCurState->mGetStateID();
	eventInfo.fEventItem[2] = tState::kStartingNewState;
	gEventMgr.mPostEvent( eventInfo );
}

/*
	Purpose:	Sets up a timer only for the actor that uses this.

	Inputs:		( eTimerIntervalTypes )	-	This can be one of several values such as "kTimerSecondsSet"
	Outputs:	N/A
*/
void
tActor::mSetTimedInterval(	eTimerIntervalTypes	Type,
							int32				TimerID,
							float				Interval,
							iStateID			StateID )
{
	tEventMgr::tTimerID eventTimerID = gEventMgr.mSetTimedInterval( Type, TimerID, Interval, fActorID, StateID );	
	fCurTimers.push_back( eventTimerID );
}

/*
	Purpose:	Informs the state associated with the actor about an event that occurred.

	Inputs:		( tShipInfo* )	-	A ship that may have participated in the event
				( tEventInfo& )	-	A structure that contains information about which
									event went off as well as the objects involved
	Outputs:	N/A
*/
void
tActor::mHandleShipEvent(	tShipInfo*						Ship,
							const tEventMgr::tEventInfo&	EventInfo )
{
	if( fCurState )
	{
		fCurState->mHandleShipEvent( Ship, EventInfo );
	}
}

/*
	Purpose:	Accessor for the ID associated with an actor.

	Inputs:		N/A
	Outputs:	( iActorID )	-	The ID associated with the actor in question
*/
iActorID
tActor::mGetActorID( void )
{
	return fActorID;
}

/*
	Purpose:	Everytime a state is killed, all of the timers as well as other cleanup
				tasks must be killed.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tActor::mEndCurState( void )
{
	if( fCurState )
	{
		//
		//	Diable all the timers associated with this state
		//
		for( iTimers::iterator i = fCurTimers.begin();
			 i != fCurTimers.end();
			 i++ )
		{
			gEventMgr.mKillTimer( *i );
		}

		fCurState->mEndState();
		
		fCurState = 0;
	}	
}


//	8Nov99	zdrummond	Added Comment Header and Footer's
//	8Jul00	dsuleski	Documentation update