//	<Name>	EventMgr.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "EventMgr.h"
#include "sfcAssert.h"
#include "ScriptInterface.h"

tEventMgr gEventMgr;

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tEventMgr::tEventMgr( void )
	: fCurActorIndex(0)
{
	for( int32 i = 0; i < kMaxActors; i++ )
	{
		fActorLookup[i] = NULL;
	}
}

/*
	Purpose:	Will set a timer that will be triggered dependant upon the following parameters.

	Inputs:		( eTimerIntervalTypes )	-	Can be one of several constants such as "kTimerTurnsSet"
	Outputs:	( tTimerID )			-	The timer object
*/
tEventMgr::tTimerID
tEventMgr::mSetTimedInterval(	eTimerIntervalTypes	Type,
								int32				TimerID,
								float				Interval,
								iActorID			ActorID,
								iStateID			StateID )
{
	tTimerID eventTimerID;
	eventTimerID.fTimerID = TimerID;
	eventTimerID.fActorID = ActorID;
	eventTimerID.fStateID = StateID;

	tShipSystemsInfo info;

	info.fAction = kSetTimedInterval;
	info.fInt32[0] = Type;
	info.fInt32[1] = eventTimerID.fActorID;
	info.fInt32[2] = eventTimerID.fStateID;
	info.fInt32[3] = eventTimerID.fTimerID;
	info.fFloat[0] = Interval;
	gScriptInterface.mEntityMgr( info );

	return eventTimerID;	
}

/*
	Purpose:	Will end a timer.

	Inputs:		( tTimerID )	-	The ID associated with the timer in question
	Outputs:	N/A
*/
void
tEventMgr::mKillTimer( const tTimerID& TimerID )
{
	tShipSystemsInfo info;

	info.fAction = kSetTimedInterval;
	info.fInt32[0] = kTimerKillTimer;
	info.fInt32[1] = TimerID.fActorID;
	info.fInt32[2] = TimerID.fStateID;
	info.fInt32[3] = TimerID.fTimerID;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	By registering an actor, we place the actor object into a list that is processed
				when new event come around.

	Inputs:		( tActor* )		-	The actor in question
				( iActorID )	-	The ID of the actor in question
	Outputs:	
*/
iActorID
tEventMgr::mRegisterActor(	tActor*		Actor,
							iActorID	ParentActor )
{
	//
	//	Find an empty actor to fill.
	//
	iTruth	iFoundEmptySlot = kFalse;
	for ( int32 i = 0; i < kMaxActors; i++ )
	{
		if ( fActorLookup[i] == NULL )
		{
			iFoundEmptySlot = kTrue;
			break;
		}
	}

	Assert( iFoundEmptySlot );
	if ( !iFoundEmptySlot )
	{
		return kInvalidActorID;
	}

	fActorLookup[i] = Actor;
	Actor->fActorID = i;
	
	Actor->mSetupState();
	Actor->mSetupCommState();

	return i;
}

/*
	Purpose:	Removes an actor from the "master" list of actors.

	Inputs:		( iActorID )	-	The ID of the actor in question
	Outputs:	N/A
*/
void
tEventMgr::mRemoveActor( iActorID Actor )
{
	Assert( fActorLookup[Actor] );
	
	if( fActorLookup[Actor] )
	{
		fActorLookup[Actor]->mEndCurState();
		delete fActorLookup[Actor];
		fActorLookup[Actor] = NULL;
	}
}

/*
	Purpose:	Will send an event to the area of the API that will eventually
				examine its contents and act on them.

	Inputs:		( tEventInfo& )	-	Constant reference of a structure that contains
									information about the event in question
	Outputs:	N/A
*/
void
tEventMgr::mPostEvent( const tEventInfo& EventInfo )
{
	//
	//	This is slow and lame.. but it makes sure all event go though the same logic
	//
	tShipSystemsInfo systemEventInfo;
	systemEventInfo.fTeam = kMaxTeams;
	systemEventInfo.fAction = EventInfo.fEvent;
	if( EventInfo.fShip )
	{
		systemEventInfo.fShipEntityID = EventInfo.fShip->mGetShipID();
	}
	else
	{
		systemEventInfo.fShipEntityID = -1;
	}

	for( int32 i = 0; i < tEventInfo::kEventItemCount; i++ )
	{
		systemEventInfo.fInt32[i] = EventInfo.fEventItem[i];
	}

	

	mEntityEvent( systemEventInfo );
}

/*
	Purpose:	Will examine an incoming event and act on it accordingly

	Inputs:		( tShipSystemsInfo& )	-	Constant reference to a structure that contains
											the information on an event for processing
	Outputs:	N/A
*/
void
tEventMgr::mEntityEvent( const tShipSystemsInfo& Info )
{
	tMissionInfo* missionInfo = gScriptInterface.mMission();
	tShipInfo* ship = NULL;
	eTeamID teamID = eTeamID( Info.fTeam );

	if( teamID < kMaxTeams )
	{
		//
		//	The ship that is found hereis used for most of the below actions.  There are special cases that
		//	are taken care of in each of the cases if needed.
		//
		tTeamInfo* team = missionInfo->mGetTeamHandle( teamID );
		ship = team->mFindShipID ( Info.fShipEntityID );		
		if( ship == NULL )
		{
			//
			//	The ship was NULL, so we should now search through all of the teams for the ships.  This
			//	is done under the assumption that all shipIDs are unique (primary key).
			//
			for ( int32 i = kTeam1; i < kMaxTeams; i++ )
			{
				tTeamInfo* team = missionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
				if ( team )
				{
					ship = team->mFindShipID( Info.fShipEntityID );
					if ( ship )
						break;
				}
			}
			if ( !ship )
				return;
		}
	}

	//
	//	Pre action.
	//
	switch ( Info.fAction )
	{
	case kNotifyOnCapture:
		ship->mSetShipStatus( kShipCaptured, ( Info.fInt32[0] != ship->mGetOriginalTeam() ) );
		break;

	case kNotifyAtDeath:
		ship->mSetShipStatus( kShipDestroyed );
		ship->mClearAllCommButtons();
		break;

	case kNotifyDamage: 
		if( ship->mTestShipStatus( kShipInternalsTaken ) == kFalse && ship->mGetDamagePercent() > 0.0f )
		{
			ship->mSetShipStatus( kShipInternalsTaken );
		}
		break;

	case kNotifyDisengaged:
		ship->mSetShipStatus( kShipDisengaged );
		break;

	case kNotifyAtDestination:
		ship->mSetShipStatus( kShipAtFinalDestination );
		break;

	case kNotifyChangingShip:
		{
			tTeamInfo* team = missionInfo->mGetTeamHandle( static_cast< eTeamID >( teamID ) );
			if ( team )
			{
				tShipInfo* toShip = team->mFindShipID ( Info.fInt32[0] );
				team->mSetCurrentlyControlledShip( toShip );
			}
		}
		break;

	case kNotifyDeepScan:
		ship->mSetDeepScanStatus( Info.fInt32[0] );
		break;

	case kNotifyPlayerLeaving:
		{
			tTeamInfo* team = missionInfo->mGetTeamHandle( static_cast< eTeamID >( Info.fInt32[1] ) );
			if( team ) 
			{
				team->mSetTeamLeaving();				
			}
			gScriptInterface.mMission()->mInvokeGameStatus( kTrue, kNoTeam, kTrue, gScriptInterface.mIsHost() );
		}
		break;
	}

	//
	//	remove this action from this ships callback events
	//
	if (ship != NULL)
	{
		iNotifyEvents old_events = ship->mGetShipCallbackEvents();
		iNotifyEvents new_events = old_events & ~Info.fAction;
		if (new_events != old_events)
		{
			ship->mReplaceShipCallbackEvents( new_events );
		}
	}
	
	tEventInfo eventInfo;
	eventInfo.fEvent = Info.fAction;
	eventInfo.fShip = ship;
	for( int32 i = 0; i < tEventInfo::kEventItemCount; i++ )
	{
		eventInfo.fEventItem[i] = Info.fInt32[i];	
		eventInfo.fEventItem_Float[i] = Info.fFloat[i];
	}

	if( eventInfo.fEvent == kNotifyAtTimedInterval &&
		eventInfo.fEventItem[0] != kInvalidActorID )
	{
		tActor* actor = fActorLookup[ eventInfo.fEventItem[0] ];
		if( actor )
		{
			actor->mHandleShipEvent( ship, eventInfo );
		}			
		else
		{
			//
			//	A event meant for a particuler actor went off, but that actor does not exist
			//
			Assert( kFalse );
			return;
		}
	}
	else
	{
		for( int32 i = 0; i < kMaxActors; i++ )
		{
			if( fActorLookup[i] )
			{
				fActorLookup[i]->mHandleShipEvent( ship, eventInfo );
			}
		}
	}

	//
	//	Post action.
	//
	switch( eventInfo.fEvent )
	{
		//
		//	Remove a ship from the team list.
		//
		case kNotifyAtDeath:
			{
				tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( ship->mGetOriginalTeam() );
				Assert( team );
				if ( team )
				{
					team->mRemoveShipFromTeamList( ship );
				}
			}
			break;

		case kNotifyDisengaged:
			{
				if ( ship->mGetShipOptions() & kDeathUponDisengage )
				{
					ship->mDestroyShip();
				}
			}
			break;

		case kNotifyOnCapture:
		case kNotifyChangingShip:
			{
				ship->mSetShipCallbackEvents( eventInfo.fEvent );

				if ( eventInfo.fEvent == kNotifyOnCapture )
				{
					if ( ship->mGetShipOptions() & kDeathUponCapture )
					{
						ship->mDestroyShip();
					}
				}
			}
			break;
	}	
}

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tEventMgr::tEventInfo::tEventInfo( void )	:
	fEvent( kNotifyOff ),
	fShip( NULL )
{
	for( int32 i = 0; i < kEventItemCount; i++ )
	{
		fEventItem[i] = 0;
		fEventItem_Float[i] = 0.0f;
	}
}

void
tEventMgr::mCleanUpActors( void )
{
	for( int32 i = 0; i < kMaxActors; i++ )
	{
		if( gEventMgr.fActorLookup[i] )
		{
			gEventMgr.mRemoveActor( fActorLookup[i]->mGetActorID() );
		}
	}
}

//	11Nov99	zdrummond	Added Comment Header and Footer's
//	19May00	dsuleski	Added ship death messages support for ships that were captured and sent on another team
//	16Jun00	dsuleski	Actor index is no longer sequential -- can break a list using fCurActorIndex
//	8Jul00	dsuleski	Documentation update
