//	<Name>	MissionInfo.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "ScriptInterface.h"
#include "sfcAssert.h"

iTruth tMissionInfo::fLookupInitialized = false;
tMissionInfo::tCharLookup tMissionInfo::fRelationshipLookup;
tMissionInfo::tCharLookup tMissionInfo::fGoalLookup;

/*
	Purpose:	Constructor.

	Inputs:		( tScript* )	-	The script that should be connected to this mission
	Outputs:	N/A
*/
tMissionInfo::tMissionInfo( tScript* Script ) :
	fScript( Script ),
	fMissionEnd( kFalse )
{
	int32 index = kMaxTeams;
	while( index-- )
	{
		fTeams[index] = NULL;
		fRaces[index] = kUndefinedRace;
		fStatusArray[index] = kGameInProgress;
	}
	fTotalAlliedHumanTeams = 0;
	fTotalEnemyHumanTeams = 0;
	fTotalAlliedCPUTeams = 0;
	fTotalEnemyCPUTeams = 0;
}

/*
	Purpose:	Destructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tMissionInfo::~tMissionInfo( void )
{
	int32 index = kMaxTeams;
	while( index-- )
	{
		if( fTeams[index] != NULL )
		{
			delete fTeams[index];
			fTeams[index] = NULL;
		}
	}
}

/*
	Purpose:	Sets up the next mission to be used.

	Inputs:		( int32 )	-	The ID associated with the script to be scheduled
	Outputs:	N/A
*/
void
tMissionInfo::mScheduleMission( int32 MsgID )
{
	tShipSystemsInfo info;

	info.fAction = kScheduleMission;
	info.fText = Messages[MsgID].c_str();

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Determines if the the mission has ended.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the mission has ended; kFalse, otherwise
*/
iTruth
tMissionInfo::mCheckForMissionEnd( void )
{
	if( fMissionEnd == kFalse )
	{
		fMissionEnd = kTrue;

		//Check if all the human teams have finished playing
		for( int32 i = 0; i < kMaxTeams; ++i )
		{
			if( fTeams[i] &&
				fTeams[i]->mIsTeamHumanControlled() &&
				fTeams[i]->mGetGameStatus() == kGameInProgress )
			{
				fMissionEnd = kFalse;
				break;					
			}
		}

		if( fMissionEnd )
		{
			tShipSystemsInfo info;
			info.fAction = kMissionOver;

			gScriptInterface.mEntityMgr( info );	
		}
	}

	return fMissionEnd;
}

/*
	Purpose:	Modifies the current date.

	Inputs:		( tMissionDate& )	-	Constant reference of a structure that contains the
										current date
	Outputs:	N/A
*/
void
tMissionInfo::mSetDate( const tDynaverseScriptInfo::tMissionDate& Date )
{
	fDate = Date;
}

/*
	Purpose:	Modifies the current location.

	Inputs:		( eSectorName )	-	One of several constants that describes the current position
	Outputs:	N/A
*/
void
tMissionInfo::mSetLocation( eSectorName Location )
{
	fLocation = Location;
}


/*
	Purpose:	Modifies the system name.

	Inputs:		( eSystems )	-	One of several constants that describe the system name
	Outputs:	N/A
*/
void
tMissionInfo::mSetSystemName( eSystems SystemName )
{
	fSystemName = SystemName;
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tMissionInfo::mSetupTeam(	int32				Team,
							eTeamTag			Tag,
							eRaceName			Race,
							iTruth				HumanControlled,
							const type_info&	TeamClass )
{
	fRaces[Team] = Race;
	
	fTeams[Team] = dynamic_cast<tTeamInfo*>( tDynamicAllocator::Instance()->mAllocateClass( TeamClass ) );

	//
	//	If this assert fails, it means you did not define an allocator for your team, in each team you 
	//	define you must add tClassAllocator<tFillInTheTeamName> gFillInTheTeamNameAllocator; to the
	//	team source file; OR that you passed in an invalid type_info, such as a ship class typeid
	//
	Assert( fTeams[Team] );

	if( fTeams[Team] )
	{
		fTeams[Team]->mSetTeamInfoData( static_cast< eTeamID >( Team ), fRaces[Team], HumanControlled, Tag );
	}

	tEventMgr::tEventInfo eventInfo;
	eventInfo.fEvent = kNotifyNewTeam;
	eventInfo.fEventItem[0] = (int32)fTeams[Team];
	
	gEventMgr.mPostEvent( eventInfo );
}

/*
	Purpose:	Accessor for a usuable data structure that describes a team based on its ID.

	Inputs:		( eTeamID )		-	The ID associated with the team in question
	Outputs:	( tTeamInfo* )	-	The team requested
*/
tTeamInfo*
tMissionInfo::mGetTeamHandle( eTeamID Team )
{
	return fTeams[Team];
}

/*
	Purpose:	Accessor for the current status of the game.

	Inputs:		( eTeamID )		-	The ID associated then the team in question
	Outputs:	( eGameStatus )	-	One of several constants that describe what the
									status of the game currently is
*/
eGameStatus
tMissionInfo::mGetGameStatus( eTeamID ID )
{
	tTeamInfo*		team = mGetTeamHandle( ID );
	iNotifyEvents	event = team->mGetTeamStatus();

	switch ( event )
	{
	case kNotifyAtDeath:
		return kPlayerDestroyed;
	case kNotifyOnCapture:
		return kPlayerCaptured;
	case kNotifyDisengaged:
		return kPlayerDisengaged;
	}

	return kPlayerLoses;
}

/*
	Purpose:	Determines if all teams in the game are ready to go.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if everyone is ready; kFalse, otherwise
*/
iTruth
tMissionInfo::mAllTeamsReady( void )
{
	int32 index = kMaxTeams;
	while( index-- )
	{
		if( fTeams[index] != NULL )
		{
			if( fTeams[index]->mAllShipsReady() == kFalse )
			{
				return kFalse;
			}
		}
	}
	return kTrue;
}

/*
	Purpose:	Accessor for a random integer.

	Inputs:		( int32 )	-	The low range of the random integer
				( int32 )	-	The high range of the random integer
	Outputs:	( int32 )	-	The random integer requested
*/
int32
tMissionInfo::mRandomInt32(	int32	Lo,
							int32	Hi ) const
{
	return gScriptInterface.mRandomInt( Lo, Hi );
}

/*
	Purpose:	Accessor for a random bit.

	Inputs:		( int32 )		-	The series of bits
				( int32 )		-	The number of bits in the series
				( eBitRequest )	-	Can be kOnBits, kOffBits, or kAllBits
	Outputs:	( int32 )		-	The requested string of random bits
*/
int32
tMissionInfo::mGetRandomBitValue(	int32		String,
									int32		MaxBits,
									eBitRequest	BitRequest )
{
	int32	numOnBits = 0;
	int32	numOffBits = 0;
	int32	returnValue = -1;
	int32	onMask = String;
	int32	offMask = String ^ 0xFFFFFFFF;

	//
	//	Find out how many bits are on.
	//
	int32	temp = String;
	while ( temp = ( temp & ( temp - 1 ) ) )
		numOnBits++;
	numOffBits = MaxBits - ( ++numOnBits );

	//
	//	Left-pad the string with zeroes.
	//
	String <<= ( 32 - MaxBits );
	String >>= ( 32 - MaxBits );

	//
	//	Get a random number from 0 to maxBits - 1.
	//
	switch ( BitRequest )
	{
	case kOnBits:
		{
			while ( ( returnValue = ( 1 << ( rand() % MaxBits ) ) ) & onMask )
				;
		}
		break;

	case kOffBits:
		{
			while ( !( ( returnValue = ( 1 << ( rand() % MaxBits ) ) ) & offMask ) )
				;
		}
		break;

	case kAllBits:
		{
			returnValue = ( 1 << ( rand() % MaxBits ) );
		}
		break;
	}
	return returnValue;
}

/*
	Purpose:	Displays a message to the screen using std::string.

	Inputs:		( eTeamID )				-	The team that should receive the message
				( string )				-	The message
				( eScriptMessageTypes )	-	One of several constants that describe how the message should
											be displayed
				( tShipInfo* )			-	?
	Outputs:	N/A
*/
void
tMissionInfo::mDisplayMessage(	eTeamID				MsgForTeam,
								std::string			Msg,
								eScriptMessageTypes	Type,
								tShipInfo* Ship )
{
	mDisplayMessage( MsgForTeam, Msg.c_str(), Type, Ship );
}

/*
	Purpose:	Displays a message to the screen using the index into the messages.

	Inputs:		( eTeamID )				-	The team that should receive the message
				( int32 )				-	The index into the array of messages
				( eScriptMessageTypes )	-	One of several constants that describe how the message should
											be displayed
				( tShipInfo* )			-	?
	Outputs:	N/A
*/
void
tMissionInfo::mDisplayMessage(	eTeamID				MsgForTeam,
								int32				MsgID,
								eScriptMessageTypes	Type,
								tShipInfo*			Ship )
{
	mDisplayMessage( MsgForTeam, Messages[MsgID].c_str(), Type, Ship );
}

/*
	Purpose:	Displays a message to the screen using char*.

	Inputs:		( eTeamID )				-	The team that should receive the message
				( int32 )				-	The message
				( eScriptMessageTypes )	-	One of several constants that describe how the message should
											be displayed
				( tShipInfo* )			-	?
	Outputs:	N/A
*/
void
tMissionInfo::mDisplayMessage(	eTeamID				MsgForTeam,
								const char*			Msg,
								eScriptMessageTypes	Type,
								tShipInfo*			Ship )
{
	if( fTeams[MsgForTeam] && fTeams[MsgForTeam]->mIsTeamHumanControlled() )
	{
		int32 ShipID = 0;
		if( Ship )
			ShipID = Ship->mGetShipID();

		//
		//	Replace script-specific tokens.
		//
		mReplaceTokens( const_cast< char* >( Msg ) );

		//
		//	Send the message over to the source for display.
		//
		gScriptInterface.mPopupMessage( static_cast< eTeamID >( MsgForTeam ), Msg, Type, ShipID );
	}
}

/*
	Purpose:	Not used

	Inputs:		
	Outputs:	
*/
void
tMissionInfo::mReplaceTokens( char* Msg )
{}

/*
	Purpose:	Displays a debug message on the screen.  Will not be displayed in release -- or will it?

	Inputs:		( char* )	-	Constant pointer to a list of characters that describes the message
	Outputs:	N/A
*/
void
tMissionInfo::mDebugMessage( const char* Message )
{
	gScriptInterface.mPopupMessage( kTeam1, Message, kNonCommScriptMessage, 0 );
}

/*
	Purpose:	Will append text to the debrifing list for a team.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
				( int32 )	-	The message that should be appended
	Outputs:	N/A
*/
void
tMissionInfo::mAddToDebriefingList(	eTeamID	TeamID,
									int32	DebriefingIndex )
{
	mAddToDebriefingList( TeamID, Messages[DebriefingIndex].c_str() );
}

/*
	Purpose:	Will append text to the debrifing list for a team.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
				( char* )	-	The message that should be appended
	Outputs:	N/A
*/
void
tMissionInfo::mAddToDebriefingList(	eTeamID		TeamID,
									const char*	Debriefing )
{
	//
	//	Only add a debriefing if the team is human controlled.
	//
	tTeamInfo* team = mGetTeamHandle( TeamID );
	if ( team && team->mIsTeamHumanControlled() )
	{
		tShipSystemsInfo info;
		info.fAction = kPlayerEvent;
		info.fTeam = TeamID;
		info.fInt32[0] = kAddToDebriefing;
		info.fText = Debriefing;
		
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will append text to the debrifing list for a team.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
				( string )	-	The message that should be appended
	Outputs:	N/A
*/
void
tMissionInfo::mAddToDebriefingList(	eTeamID		TeamID,
									std::string	Debriefing)
{
	mAddToDebriefingList( TeamID, Debriefing.c_str() );
}

/*
	Purpose:	Clears the debriefing list for a team.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
	Outputs:	N/A
*/
void
tMissionInfo::mResetDebriefingCount( eTeamID TeamID )
{
	//
	//	Only reset the debriefing if the team is human controlled.
	//
	tTeamInfo* team = mGetTeamHandle( TeamID );
	if ( team && team->mIsTeamHumanControlled() )
	{
		tShipSystemsInfo info;
		info.fAction = kPlayerEvent;
		info.fTeam = TeamID;
		info.fInt32[0] = kResetDebriefing;

		gScriptInterface.mEntityMgr( info );
	}
}


/*
	Purpose:	Given an object on the original map, a position can be given.

	Inputs:		( char )	-	The object on the map
				( int32* )	-	Allocated memory to contain the x-position
				( int32* )	-	Allocated memory to contain the y-position
	Outputs:	( iTruth )	-	kTrue if the object exists on the map; kFalse, otherwise
*/
iTruth
tMissionInfo::mGetGridPosition(	unsigned char	MapObject,
								int32*			GridX,
								int32*			GridY ) const
{
	int32 subType;
	eMapObjectType objectType = tScriptMapData::mGetMapItemType( MapObject, reinterpret_cast< int* >( &subType ) );

	tShipSystemsInfo info;

	info.fAction = kGetMapEntryPosition;
	info.fInt32[0] = objectType;
	info.fInt32[1] = subType;

	gScriptInterface.mEntityMgr( info );

	//
	//	If the object is not found on the map, we need to return a -1 in both of the "out" parameters
	//	and return false to represent the absence of the map object.  Otherwise, we just return the
	//	values received with a true.
	//
	if ( info.fGridX < 0 && info.fGridY < 0 )
	{
		*GridX = -1, *GridY = -1;
		return kFalse;
	}
	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
	return kTrue;
}

/*
	Purpose:	Accessor for the distance between two objects on the original map.

	Inputs:		( char )	-	The first object on the map
				( char )	-	The second object on the map
	Outputs:	( float )	-	Distance between the 2 objects if it exists
*/
float
tMissionInfo::mGetTacticalDistance(	unsigned char	MapObject1,
									unsigned char	MapObject2 ) const
{
	int32 subType1;
	eMapObjectType objectType1 = tScriptMapData::mGetMapItemType( MapObject1, reinterpret_cast< int* >( &subType1 ) );
	int32 subType2;
	eMapObjectType objectType2 = tScriptMapData::mGetMapItemType( MapObject1, reinterpret_cast< int* >( &subType2 ) );

	tShipSystemsInfo info;
	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = objectType1;
	info.fInt32[2] = subType1;
	info.fInt32[3] = objectType2;
	info.fInt32[4] = subType2;
	info.fShipEntityID = 0;				// == 0 indicates that distance is between two map objects
	gScriptInterface.mEntityMgr( info );

	return( info.fFloat[0] );
}

/*
	Purpose:	Creates a map object that can be seen on the original map.

	Inputs:		( char )	-	New object to create
				( int32 )	-	X-position of the new object
				( int32 )	-	Y-position of the new object
	Outputs:	N/A
*/
void
tMissionInfo::mCreateMapObject(	unsigned char	MapObject,
								int32			GridX,
								int32			GridY )
{
	tShipSystemsInfo info;

	info.fAction = kCreateMapEntry;
	info.fInt32[0] = MapObject;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Accessor for the rank of the player.

	Inputs:		N/A
	Outputs:	( int32 )	-	A scalar that determines the level of the player
*/
int32
tMissionInfo::mGetPlayerRank( void ) const
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[0];
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
int32
tMissionInfo::mGetEliteOrganization( void ) const
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[1];
}

/*
	Purpose:	Accessor for the current difficulty level.

	Inputs:		N/A
	Outputs:	( int32 )	-	A scalar that describes the level
*/
int32
tMissionInfo::mGetDifficultyLevel( void ) const
{
	tShipSystemsInfo info;

	info.fAction = kGetPlayerRank;
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[2];
}
/*
	Purpose:	Accessor for the current era.

	Inputs:		N/A
	Outputs:	( int32 )	-	A scalar that describes the era
*/
int32
tMissionInfo::mGetEra( void ) const
{
	tShipSystemsInfo info;

	info.fAction = kOther;
	info.fInt32[0] = kGetEra;
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[1];
}
/*
	Purpose:	Plays a sound file according to its index.

	Inputs:		( int32 )	-	The index into the list of sound files
	Outputs:	N/A
*/
void
tMissionInfo::mPlaySound( int32 SoundID )
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = SoundID;
	info.fInt32[1] = 0;		// indicate its a non-voice sound
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Plays a sound file (specifically a voice).

	Inputs:		( int32 )	-	The index into the list of sound files
				( iTruth )	-	kTrue for sync; kFalse for async
	Outputs:	N/A
*/
void
tMissionInfo::mPlayVoiceSample(	int32	VoiceID,
								iTruth	StopPreviousVoice )
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = VoiceID;
	info.fInt32[1] = 1;			// indicate its a voice sound
	info.fInt32[2] = StopPreviousVoice;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Evaluates whether a voice file is still being played.

	Inputs:		N/A
	Outputs:	( int32 )	-	1 if a file is still playing; 0, otherwise
*/
int32
tMissionInfo::mVoiceSampleStillPlaying( void )
{
	tShipSystemsInfo info;

	info.fAction = kPlaySound;
	info.fInt32[0] = 0;
	info.fInt32[1] = 2;			// indicate return status request
	gScriptInterface.mEntityMgr( info );
	return info.fInt32[0];
}

/*
	Purpose:	Modifies a player's career totals.

	Inputs:		( eClassTypes )		-	One of several constants that describe a type of ship
				( eCareerEvents )	-	One of several constants that describe occurring events
				( int32 )			-	The amount to modify
	Outputs:	N/A
*/
void
tMissionInfo::mAddToCareerTotals(	eClassTypes		Type,
									eCareerEvents	Event,
									int32			Amount )
{
	tShipSystemsInfo info;

	info.fAction = kAddToCareerTotal;
	info.fInt32[0] = Amount;
	info.fInt32[1] = Event;
	info.fInt32[2] = Type;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Removes a ship as a target from the AI list from all teams.

	Inputs:		( tShipInfo* )	-	The ship that is being targetted
	Outputs:	N/A
*/
void
tMissionInfo::mRemoveShipAsTarget( tShipInfo* TargetShip )
{
	if( TargetShip )
	{
		eTeamID target_team = TargetShip->mGetTeam();

		//
		//	remove this ship as a target from all teams
		//
		for( int32 i = 0; i < kMaxTeams; i++ )
		{
			tTeamInfo* Team = fTeams[i];
			if( Team && i != target_team )
				Team->mRemoveShipAsTarget( TargetShip );
		}
	}
}

/*
	Purpose:	Will set relationships between teams according to their team tags.

	Inputs:		( int32 )	-	The list of tags
				( string )	-	The 2D array describing the relationships and goals
				( int32 )	-	The number of teams
	Outputs:	N/A
*/
void
tMissionInfo::mSetTagRelations(	int32*				TagList,
								const std::string*	RelationTable,
								int32				Size )
{
	mSetupRelationshipLookup();

	//
	//	Run through each of the kMaxTeams (20 currently) and find out if they exist in the multiplayer game.
	//
	for ( int32 teamIndex = 0; teamIndex < kMaxTeams; teamIndex++ )
	{
		tTeamInfo* team = NULL;
		team = mGetTeamHandle( static_cast< eTeamID >( teamIndex ) );

		//
		//	If the team exists, then the team can be given relations from the relationTable equivalent to their team tag.
		//
		if ( team )
		{
			eTeamTag teamTag = team->mGetTag();
			if ( teamTag != kNoTeamTag )
			{
				//
				//	Now that we know that the first team has a team tag, we should find another team that exists.
				//
				for ( int32 hasRelationshipWithIndex = 0; hasRelationshipWithIndex < kMaxTeams; hasRelationshipWithIndex++ )
				{
					tTeamInfo*	team2 = NULL;
					team2 = mGetTeamHandle( static_cast< eTeamID >( hasRelationshipWithIndex ) );

					//
					//	If the second team does not exist, then just continue on by ignoring this particular team.
					//
					if ( team2 )
					{
						eTeamTag teamTag2 = team2->mGetTag();
						if ( teamTag2 != kNoTeamTag )
						{
							//
							//	We know that both teams exist, go ahead and find that relationship string and start relating!
							//
							const std::string& relationStr = RelationTable[ (teamTag * Size) + teamTag2 ];

							Assert( fRelationshipLookup.find( relationStr[0] ) != fRelationshipLookup.end() );
							//If this assert fails then make sure your array has the correct number of elements.
							eTeamRelationship relationship = (eTeamRelationship)fRelationshipLookup[ relationStr[0] ];

							Assert( fGoalLookup.find( relationStr[1] ) != fGoalLookup.end() );
							//If this assert fails then make sure your array has the correct number of elements.
							eTargetGoals goal = (eTargetGoals)fGoalLookup[ relationStr[1] ];

							//
							//	Finally, set the relationship -- whew!
							//
							mSetTeamRelations( static_cast< eTeamID >( teamIndex ), static_cast< eTeamID >( hasRelationshipWithIndex ), relationship, goal, 1.0f, false );
						}
					}
				}
			}
		}
	}
}

/*
	Purpose:	Sets relationships and goals between teams.

	Inputs:		( int32 )	-	The teams
				( string )	-	A 2D array containing the info on the relationships and goals
				( in32 )	-	The number of teams
	Outputs:	N/A
*/
void
tMissionInfo::mSetTeamRelations(	int32*				TeamList,
									const std::string*	RelationTable,
									int32				Size )
{
	mSetupRelationshipLookup();

	for( int32 teamIndex = 0; teamIndex < Size; teamIndex++ )
	{
		int32 teamID = TeamList[teamIndex];
		for( int32 hasRelationshipWithIndex = 0; hasRelationshipWithIndex < Size; hasRelationshipWithIndex++ )
		{
			int32 hasRelationshipWithID = TeamList[hasRelationshipWithIndex];
			
			const std::string& relationStr = RelationTable[ (teamIndex * Size) + hasRelationshipWithIndex ];

			//
			//	If this assert fails then make sure your array has the correct number of elements.
			//
			Assert( fRelationshipLookup.find( relationStr[0] ) != fRelationshipLookup.end() );
			eTeamRelationship relationship = (eTeamRelationship)fRelationshipLookup[ relationStr[0] ];

			//
			//	If this assert fails then make sure your array has the correct number of elements.
			//
			Assert( fGoalLookup.find( relationStr[1] ) != fGoalLookup.end() );
			eTargetGoals goal = (eTargetGoals)fGoalLookup[ relationStr[1] ];

			mSetTeamRelations( static_cast< eTeamID >( teamID ), static_cast< eTeamID >( hasRelationshipWithID ), relationship, goal, 1.0f, false );   
		}
	}
}

/*
	Purpose:	Sets a relationship and a goal between two teams.

	Inputs:		( eTeamID )	-	The first team
				( eTeamID )	-	The second team
				( eTeamRelationship )	-	Relationship between the two teams
				( eTargetGoals )		-	Goals between the two teams
				( float )				-	?
				( iTruth )				-	kFalse if these parameters only apply from team 1 to
											team 2; kTrue, if they apply in both directions
	Outputs:	
*/
void
tMissionInfo::mSetTeamRelations(	eTeamID				Team1,
									eTeamID				Team2,
									eTeamRelationship	Relations,
									eTargetGoals		Goals,
									float				EPVFactor,
									iTruth				MutualRelations )
{
	if ( Team1 > kNoTeam && Team1 < kMaxTeams && Team2 > kNoTeam && Team2 < kMaxTeams )
	{
		//
		//	set Team1 relationship with Team2
		//
		tTeamInfo* team = NULL;
		team = mGetTeamHandle( Team1 );
		if ( !team )
			return;
		team->mSetRelations( Team2, Relations, Goals, EPVFactor );

		if( MutualRelations )
		{
			//
			//	set Team2 relationship with Team1
			//
			team = NULL;
			team = mGetTeamHandle( Team2 );
			if ( !team )
				return;
			team->mSetRelations( Team1, Relations, Goals, EPVFactor );
		}
	}
}

/*
	Purpose:	Accessor for the relationship between two teams.

	Inputs:		( eTeamID )				-	Team 1 ID
				( eTeamID )				-	Team 2 ID
	Outputs:	( eTeamRelationship )	-	One of several constants that describe a relationship
											between two teams
*/
eTeamRelationship
tMissionInfo::mGetTeamRelations(	eTeamID	Team1,
									eTeamID	Team2 ) const
{
	if( Team1 > kNoTeam && Team1 < kMaxTeams )
	{
		tTeamInfo* team = fTeams[Team1];
		if( team != NULL )
			return team->mGetRelations( Team2 );
	}
	return kUnknownRelationship;
}

/*
	Purpose:	Accessor for the ship that matches a particular ID.

	Inputs:		( int32 )		-	The ID of the requested ship
	Outputs:	( tShipInfo* )	-	The ship requested
*/
tShipInfo*
tMissionInfo::mFindShipID( int32 ShipID ) const
{
	for( int32 i = 0; i < kMaxTeams; i++ )
	{
		tTeamInfo* Team = fTeams[i];
		if( Team )
		{
			for( tShipIterator ship = Team->mGetFirstShip(); *ship != NULL; ++ship )
			{
				if( ship->mGetShipID() == ShipID )
					return *ship;
			}
		}
	}
	return NULL;
}

/*
	Purpose:	Accessor for the political relationship between two races.

	Inputs:		( eRaceName )			-	The first team's race
				( eRaceName )			-	The second team's race
	Outputs:	( ePoliticalRelation )	-	One of several constants that describes the
											political relationship between two teams
*/
ePoliticalRelation
tMissionInfo::mGetRaceRelationship(	eRaceName	Race1,
									eRaceName	Race2 ) const
{
	tShipSystemsInfo info;

	info.fAction = kGetRaceRelationship;
	info.fInt32[0] = Race1;
	info.fInt32[1] = Race2;
	gScriptInterface.mEntityMgr( info );

	return( ePoliticalRelation(info.fInt32[2]) );
}

/*
	Purpose:	Accessor for the target of a ship, if available.

	Inputs:		( tShipInfo* )	-	The ship in question
	Outputs:	( tShipInfo* )	-	The ship target in question
*/
tShipInfo*
tMissionInfo::mGetShipTarget( tShipInfo* Ship ) const
{
	if( Ship )
		return mFindShipID( Ship->mGetShipTargetID() );

	return NULL;
}

/*
	Purpose:	Starts to flash a button on the comm panel.

	Inputs:		( char* )	-	Constant pointer to a list of characters that describes
								the button to flash
	Outputs:	N/A
*/
void
tMissionInfo::mStartFlashingButton( const char* ButtonID )
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kTrue;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Stops flashing a button on the comm panel.

	Inputs:		( char* )	-	Constant pointer to a list of characters that describes
								the button to flash
	Outputs:	N/A
*/
void
tMissionInfo::mStopFlashingButton( const char* ButtonID )
{
	tShipSystemsInfo info;
	info.fAction = kStartFlashingButton;
	info.fTextPtr = ButtonID;
	info.fInt32[0] = kFalse;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Returns whether this script is single-human-player.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if single-human-player; kFalse, otherwise
*/
iTruth
tMissionInfo::mGetCampaignType( void )
{
	tShipSystemsInfo info;
	info.fAction = kOther;
	info.fInt32[0] = kRequestGameType;
	info.fInt32[1] = kTrue;
	gScriptInterface.mEntityMgr( info );
	return ( static_cast< iTruth >( info.fInt32[1] ) );
}

/*
	Purpose:	Returns the value associated with the given params.

	Inputs:		( int32 )	-	Category
				( int32 )	-	Element found under given category
	Outputs:	( int32 )	-	Value associated with given element
*/
int32
tMissionInfo::mGetINIFileInt(	int32	HeaderID,
								int32	LabelID ) const
{
	tShipSystemsInfo info;
	info.fAction = kGetINIFileEntry;
	info.fText = Messages[HeaderID].c_str();
	info.fText += "^";
	info.fText += Messages[LabelID].c_str();
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[0];
}

/*
	Purpose:	Toggles an arrow pointing to a predetermined point.

	Inputs:		( eTeamID )		-	The team that is to receive the arrow
				( iTruth )		-	kTrue to turn on the indicator; kFalse, otherwise
				( int32 )		-	X-coordinate of the predetermined point
				( int32 )		-	Y-coordinate of the predetermined point
	Outputs:	N/A
*/
void
tMissionInfo::mSetGoalIndicator(	eTeamID	TeamID,
									iTruth	IndicatorOn,
									int32	GridX,
									int32	GridY )
{
	tTeamInfo* team = mGetTeamHandle( TeamID );
	if ( team )
	{
		for ( tShipIterator ship = team->mGetFirstShip(); *ship != NULL; ship++ )
		{
			tShipSystemsInfo info;
			info.fAction = kStartFlashingButton;
			info.fInt32[0] = IndicatorOn;
			info.fInt32[1] = TeamID;
			info.fInt32[2] = ship->mGetShipID();
			info.fGridX = GridX - 1;
			info.fGridY = GridY - 1;
			info.fShipEntityID = -1;
			info.fTextPtr = NULL;
			gScriptInterface.mEntityMgr( info );
		}
	}
}

/*
	Purpose:	Toggles an arrow pointing to a ship.

	Inputs:		( eTeamID )		-	The team that is receive the arrow
				( tShipInfo* )	-	The predetermined ship
	Outputs:	N/A
*/
void
tMissionInfo::mSetGoalIndicator(	eTeamID		TeamID,									
									tShipInfo*	TargetShip )
{
	if( TargetShip )
	{
		tTeamInfo* team = mGetTeamHandle( TeamID );
		if ( team )
		{
			for ( tShipIterator ship = team->mGetFirstShip(); *ship != NULL; ship++ )
			{
				tShipSystemsInfo info;
				info.fAction = kStartFlashingButton;
				info.fInt32[0] = kTrue;
				info.fInt32[1] = TeamID;
				info.fInt32[2] = ship->mGetShipID();
				info.fShipEntityID = TargetShip->mGetShipID();
				info.fTextPtr = NULL;
				gScriptInterface.mEntityMgr( info );
			}
		}
	}
}

/*
	Purpose:	Toggles an arrow pointing to a predetermined point.

	Inputs:		( tShipInfo* )	-	The ship that will be displayed with the arrow
				( iTruth )		-	kTrue to turn on the indicator; kFalse, otherwise
				( int32 )		-	X-coordinate of the predetermined point
				( int32 )		-	Y-coordinate of the predetermined point
	Outputs:	N/A
*/
void
tMissionInfo::mSetGoalIndicator(	tShipInfo*	FromShip,
									iTruth	IndicatorOn,
									int32	GridX,
									int32	GridY )
{
	if ( FromShip )
	{
		tShipSystemsInfo info;
		info.fAction = kStartFlashingButton;
		info.fInt32[0] = IndicatorOn;
		info.fInt32[1] = FromShip->mGetTeam();
		info.fInt32[2] = FromShip->mGetShipID();
		info.fGridX = GridX - 1;
		info.fGridY = GridY - 1;
		info.fShipEntityID = -1;
		info.fTextPtr = NULL;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Toggles an arrow pointing to a ship.

	Inputs:		( tShipInfo* )	-	The ship that will be displayed with the arrow
				( tShipInfo* )	-	The predetermined ship
	Outputs:	N/A
*/
void
tMissionInfo::mSetGoalIndicator(	tShipInfo*	FromShip,
									tShipInfo*	TargetShip )
{
	if( FromShip )
	{
		tShipSystemsInfo info;
		info.fAction = kStartFlashingButton;
		info.fInt32[0] = kTrue;
		info.fInt32[1] = FromShip->mGetTeam();
		info.fInt32[2] = FromShip->mGetShipID();
		info.fShipEntityID = TargetShip->mGetShipID();
		info.fTextPtr = NULL;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Evaluates whether a team is leaving a game.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
	Outputs:	( iTruth )	-	kTrue if the team left; kFalse, otherwise
*/
iTruth
tMissionInfo::mIsTeamLeaving( eTeamID Team )
{
	tTeamInfo* team = mGetTeamHandle( Team );
	if( team )
		return team->mIsTeamLeaving();
	else
		return kFalse;
}

/*
	Purpose:	Not used.

	Inputs:		
	Outputs:	
*/
void
tMissionInfo::mAdjustDynaverse(	eDynaverseTableType	Type,
								int32				Modifier,
								eRaceName			Race )
{
	tShipSystemsInfo info;
	info.fAction = kDynaverseAdjustments;
	info.fInt32[0] = Type;
	info.fInt32[1] = Modifier;
	info.fInt32[2] = Race;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Determines the map dimensions in 2D space.

	Inputs:		( int32* )	-	Allocated memory to hold the max X
				( int32* )	-	Allocated memory to hold the max Y
	Outputs:	N/A
*/
void
tMissionInfo::mGetMapDimensions(	int32*	GridX,
									int32*	GridY ) const
{
	gScriptInterface.mGetMapDimensions( GridX, GridY );
}

/*
	Purpose:	Returns the type of hex kPrimaryTeam currently occupies.  This includes what sort of terrain
				is in the hex and which race owns the hex.

	Inputs:		( tMetaLocationSpec )	-	A class that contains the who and what of the hex
	Outputs:	( iTruth )				-	Returns kTrue if hex location is valid
*/
iTruth
tMissionInfo::mGetHexType( tMetaLocationSpec& Hex )
{
	tShipSystemsInfo info;
	info.fAction = kOther;
	info.fInt32[0] = kRequestHexType;
	info.fGridX = -1;	//	who
	info.fGridY = -1;	//	what

	gScriptInterface.mEntityMgr( info );

	Hex.fRange = 0;
	Hex.fLocationWhoHexFlags = info.fGridX;
	Hex.fLocationObjectFlags = info.fGridY;

	return ( ( ( info.fGridX != -1 ) && ( info.fGridY != -1 ) ) ? kTrue : kFalse );
}

/*
	Purpose:	Creates shortcuts for the relationships and goals assigned to teams.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tMissionInfo::mSetupRelationshipLookup( void )
{
	if( !fLookupInitialized )
	{
		fLookupInitialized = true;

		fRelationshipLookup['-'] = kTeamSelfAware;
		fRelationshipLookup['A'] = kTeamAllied;
		fRelationshipLookup['F'] = kTeamFriendly;
		fRelationshipLookup['N'] = kTeamNeutral;
		fRelationshipLookup['H'] = kTeamHostile;
		fRelationshipLookup['W'] = kTeamAtWar;
		fRelationshipLookup['P'] = kTeamPassive;

		fGoalLookup['-'] = kGoalIgnore;
		fGoalLookup['I'] = kGoalIgnore;
		fGoalLookup['C'] = kGoalCripple;
		fGoalLookup['X'] = kGoalCapture;
		fGoalLookup['D'] = kGoalDestroy;
	}							                 
}

/*

	Purpose:	Will set the game status for all teams that are not kGameInProgress.  This is usually
				called when a player leaves.

	Inputs:		( iTruth )	-	kTrue if this function is called in response to a host change
				( eTeamID )	-	If you only want to invoke the game status of one team, then pass it thru
				( iTruth )	-	kTrue if called as a result of a player leaving
				( iTruth )	-	kTrue if this guy is the host
	Outputs:	N/A

*/
void
tMissionInfo::mInvokeGameStatus(	iTruth	UponHostChange,
									eTeamID	TeamID,
									iTruth	PlayerLeaving,
									iTruth	AmIHost )
{
	for ( int32 i = kTeam1; i < kMaxTeams; i++ )
	{
		tTeamInfo* team = mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( ( team ) && ( fStatusArray[i] != kGameInProgress ) && ( ( TeamID == kNoTeam ) || ( TeamID == i ) ) )
		{
			team->fVictoryLevel = kDraw;

			if( team->fTeamLeaving )
			{
				team->fVictoryLevel = kUnknownVictoryLevel;
				team->fStatus = kLeftEarly;
			}
			else
			{
				team->fVictoryLevel = team->fVictoryCondition->mCalcVictoryStatus( team->mGetTeam() );

				//
				//	change a disengage status to player wins if victory level is victory or better
				//
				if( team->fStatus & kPlayerDisengaged && team->fVictoryLevel <= kVictory )
					team->fStatus = (eGameStatus)( (team->fStatus & kCampaignCompleted) | kPlayerWins );
			}
			
			team->mScheduleNextMission( team->fVictoryCondition );
			mSetGoalIndicator( static_cast< eTeamID >( i ), kFalse, 0, 0 );			

			if ( ( !team->fHasEndedHost ) )// && ( !team->fTeamLeaving ) )
			{
				//
				//	Let the source know that the status of this team has changed and that the proper dialog should
				//	appear as a result.
				//
				if ( team->mIsTeamHumanControlled() )
				{
					tShipSystemsInfo info;
					info.fTeam = static_cast< eTeamID >( i );
					info.fAction = kPlayerEvent;
					info.fInt32[0] = kPlayerStatusChange;
					info.fInt32[1] = team->fStatus;
					info.fInt32[2] = team->fVictoryLevel;
					info.fInt32[3] = team->fVictoryCondition->mGetPrestigePoints( team->fVictoryLevel );
					info.fInt32[4] = team->fVictoryCondition->mGetBonusPrestige();

					mResetDebriefingCount( static_cast< eTeamID >( i ) );
					team->mSetupDebriefingList( team->fVictoryLevel );

					gScriptInterface.mEntityMgr( info );

					if ( gScriptInterface.mIsHost() )
					{
						team->fHasEndedHost = kTrue;
						team->fHasEndedClient = kTrue;
					}
				}

				//
				//	The mission has ended for this team, so get both the results of this mission and the information
				//	that describes the next mission, and send it over to the source.
				//
				tTeamInfo::tMissionScheduler* nextMission = team->mGetNextMissionInfo();
				tShipSystemsInfo info;
				info.fTeam = static_cast< eTeamID >( i );
				info.fAction = kOther;
				info.fInt32[0] = kMissionResults;
				info.fInt32[1] = team->fVictoryLevel;				

				//
				//	Ignore the script's mission results if the client left early.
				//
				if ( team->fVictoryLevel == kUnknownVictoryLevel )
					info.fInt32[2] = team->fVictoryCondition->mLookupPrestigeValue( team->fVictoryLevel );
				else
					info.fInt32[2] = nextMission->fPrestige;

				info.fInt32[3] = team->fVictoryCondition->mGetBonusPrestige();
				info.fInt32[4] = nextMission->fNextMissionScore;
				info.fFloat[0] = static_cast< float >( nextMission->fMedal );
				info.fFloat[1] = static_cast< float >( nextMission->fCampaignEvent );
				info.fFloat[2] = static_cast< float >( PlayerLeaving );
				info.fFloat[3] = static_cast< float >( AmIHost );
				if ( !nextMission->fNextMissionTitle.empty() )
					info.fTextPtr = nextMission->fNextMissionTitle.c_str();
				else
					info.fTextPtr = NULL;

				gScriptInterface.mEntityMgr( info );
			}

			if( mCheckForMissionEnd() == kFalse )
			{
				tShipIterator ship = team->mGetFirstShip();
				tShipInPlayEval inPlayEval;
				ship.mAddEvaluator( &inPlayEval );
				for( ; *ship != NULL; ++ship )
				{
					ship->mSetShipCallbackEvents( 0 );	
				}
			}
		}
	}
}

//	4Nov99	zdrummond	Added Comment Header and Footer's
//	18Apr00	dsuleski	Added a check if team exists in mDisplayMessage(...)
//	25May00	dsuleski	Overload mDisplayMessage() to take a char string in addition
//	06Jun00	dsuleski	mGetGameStatus() now takes a team ID
//	08Jun00	dsuleski	New team tag relations setup
//	8Jul00	dsuleski	Documentation update
//	25Jul00	dsuleski	Drawing primitive functionality
//	03Oct00	dsuleski	Debriefings are now only sent for human-controlled teams