//	<Name>	NormalVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "NormalVictoryState.h"
#include "ScriptInterface.h"
#include "sfcAssert.h"

/*
	Purpose:	Constructor.

	Inputs:		( int32 )	-	Default amount awarded for capturing an object
				( int32 )	-	Default amount awarded for achieving an astounding victory
				( int32 )	-	Default amount awarded for achieving a victory
				( int32 )	-	Default amount awarded for achieving a draw
				( int32 )	-	Default amount awarded for achieving a defeat
				( int32 )	-	Default amount awarded for achieving a devastating defeat				
	Outputs:	N/A
*/
tNormalVictoryState::tNormalVictoryState(	int32	CaptureBonus,
											int32	AstoundingVictory,
											int32	Victory,
											int32	Draw, 
											int32	Defeat,
											int32	DevastatingDefeat ) :
	tVictoryState( AstoundingVictory, Victory, Draw, Defeat, DevastatingDefeat ), 
	fCaptureBonus( CaptureBonus ),
	fPlayerCount( 0 ),
	fEnemyCount( 0 ),
	fPlayerTeamID( -1 )
{}

/*
	Purpose:	Determines the score of a team dependant upon the status of the ships on a team.  These
				values are determined by the values contained within the victory state by default.

	Inputs:		( tTeamInfo* )	-	The team in question
	Outputs:	( float )		-	The total score as described above
*/
float
tNormalVictoryState::mPointsScoredAgainstTeam( tTeamInfo* Team )
{
	float score = 0.0f;

	for( tShipIterator ship = Team->mGetFirstShip(); *ship != NULL; ++ship )
	{
		if( ship->mTestShipStatus( kShipDestroyed ) )
		{
			score += mShipDestroyedValue( *ship );	
		}
		else if( ship->mTestShipStatus( kShipCaptured ) )
		{
			score += mShipCapturedValue( *ship );
		}
		else if( ship->mTestShipStatus( kShipDisengaged ) )
		{
			score += mShipDisengagedValue( *ship );
		}
		else
		{
			score += mShipDamagedValue( *ship );
		}
	}

	return score;
}

/*
	Purpose:	A ship's destroyed value is considered to be the same as its BPV.  In this sense,
				this function acts as an accessor.

	Inputs:		( tShipInfo* )	-	The ship in question
	Outputs:	( float )		-	The BPV of the input ship
*/
float
tNormalVictoryState::mShipDestroyedValue( tShipInfo* Ship )
{
	return Ship->mGetShipBPV();	
}

/*
	Purpose:	A ship's captured value is considered to be the same as its BPV, doubled.  In this
				sense, this function acts as an accessor.  However, it also is a modifier because
				it will increment the bonus prestige.

	Inputs:		( tShipInfo* )	-	The ship in question
	Outputs:	( float )		-	The BPV of the input ship * 2
*/
float
tNormalVictoryState::mShipCapturedValue( tShipInfo* Ship )
{
	if( fCaptureBonus > 0 && mGetSide( Ship->mGetTeam() ) == kEnemySide )
	{
		fBonusPrestige += fCaptureBonus; 
	}

	return Ship->mGetShipBPV() * 2.0f;	
}

/*
	Purpose:	A ship's disengaged value is considered to be the same as its BPV, quartered.  In
				this sense, this function acts as an accessor.

	Inputs:		( tShipInfo* )	-	The ship in question
	Outputs:	( float )		-	The BPV of the input ship / 4
*/
float
tNormalVictoryState::mShipDisengagedValue( tShipInfo* Ship )
{
	return Ship->mGetShipBPV() * 0.25f;	
}

/*
	Purpose:	A ship's damaged value is considered to be the same as its BPV with respect to
				the ratio of damage it has incurred.  In this sense, this function acts as an
				accessor.

	Inputs:		( tShipInfo* )	-	The ship in question
	Outputs:	( float )		-	The BPV of the input ship * ( ( 100 - health of ship ) / 100 )
*/
float
tNormalVictoryState::mShipDamagedValue( tShipInfo* Ship )
{
	float damageRatio = Ship->mGetDamagePercent() / 100.0f; 
	return Ship->mGetShipBPV() * damageRatio;;	
}

/*
	Purpose:	Determines which side a team falls into (player or enemy).

	Inputs:		( eTeamID )		-	The number associated with the team in question
	Outputs:	( iTeamSide )	-	Either kPlayerSide or kEnemySide
*/
tNormalVictoryState::iTeamSide
tNormalVictoryState::mGetSide( eTeamID Team )
{	
	for( int32 i = 0; i < fEnemySide.mGetSize(); i++ )
	{
		if( fEnemySide[i] == Team )
		{
			return kPlayerSide;
		}
	}

	for( i = 0; i < fPlayerSide.mGetSize(); i++ )
	{
		if( fPlayerSide[i] == Team )
		{
			return kEnemySide;
		}
	}

	//
	//	Didn't find the team
	//
	Assert( kFalse );

	return kEnemySide;
}

/*
	Purpose:	Calculates the total score for a team based on their enemies.

	Inputs:		( iTeams& )	-	Reference to the list of player teams
				( iTeams& )	-	Reference to the list of the enemies of the player teams
	Outputs:	( float )	-	The total score of all the teams described in 'teams'
*/
float
tNormalVictoryState::mCalcSideScore(	const iTeams&	Teams,
										const iTeams&	EnemyTeams )
{
	float sideScore = 0;

	//
	//	Total all points scored against enemyTeams 
	//
	for( int32 i = 0; i < EnemyTeams.mGetSize(); i++ )
	{
		tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( EnemyTeams[i] ) );
		sideScore += mPointsScoredAgainstTeam( enemyTeam );
	}

	return sideScore;
}

/*
	Purpose:	Calculates a "handicap" between the player and enemy sides.

	Inputs:		( iTeams& )	-	Reference to the list of player teams
				( iTeams& )	-	Reference to the list of the enemies of the player teams
	Outputs:	( float )	-	Total BPV of enemy side  - total BPV of player side
*/
float
tNormalVictoryState::mBalancePoints(	const iTeams&	PlayerTeams,
										const iTeams&	EnemyTeams )
{
	int32 playerBPV = 0;
	int32 enemyBPV = 0;

	for( int32 i = 0; i < EnemyTeams.mGetSize(); i++ )
	{
		tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( EnemyTeams[i] ) );
		enemyBPV += enemyTeam->mGetCombatBPV();
	}

	for( i = 0; i < PlayerTeams.mGetSize(); i++ )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( PlayerTeams[i] ) );
		playerBPV += playerTeam->mGetCombatBPV();
	}

	return enemyBPV - playerBPV;
}

/*
	Purpose:	Figures out the probable victory level dependant upon the scores of the player
				side and the enemy side.

	Inputs:		( float )			-	The total score of the player team
				( float )			-	The total score of the enemy team
	Outputs:	( eVictoryLevels )	-	Can be kAstoundingVictory, kVictory, kDraw, kDefeat, or
										kDevastatingDefeat
*/
eVictoryLevels
tNormalVictoryState::mGetVictoryLevel(	float	PlayerScore,
										float	EnemyScore )
{
	float scoreRatio = PlayerScore / EnemyScore;

	if( scoreRatio >= 2.0f )
	{
		return kAstoundingVictory;
	}
	else if( scoreRatio >= 1.2f )
	{
		return kVictory;
	}
	else if( scoreRatio >= 0.8f )
	{
		return kDraw;
	}
	else if( scoreRatio >= 0.5f )
	{
		return kDefeat;
	}
	else 
	{
		return kDevastatingDefeat;
	}
}

/*
	Purpose:	Determines the victory level of a team dependant upon its score and its enemies' score.

	Inputs:		( eTeamID )			-	The ID associated with the team in question
	Outputs:	( eVictoryLevels )	-	Can be kAstoundingVictory, kVictory, kDraw, kDefeat, or
										kDevastatingDefeat
*/
eVictoryLevels
tNormalVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 playerBPV = 0;
	int32 enemyBPV = 0;
	
	float playerScore = mCalcSideScore( fPlayerSide, fEnemySide );
	float enemyScore = mCalcSideScore( fEnemySide, fPlayerSide );
	
	playerScore += mBalancePoints( fPlayerSide, fEnemySide );

	//
	//	The min score is zero. This will stop a div by zero
	//
	if( playerScore < 1.0f )
	{
		playerScore = 1.0f;	
	}

	if( enemyScore < 1.0f )
	{
		enemyScore = 1.0f;
	}
	
	return mGetVictoryLevel( playerScore, enemyScore );
}

/*
	Purpose:	Will figure out the status of the script using this victory state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tNormalVictoryState::mCheckGameStatus( void )
{
	eGameStatus status = mCheckForWin();
	if( status != kGameInProgress ) 
	{
		fTeam->mSetGameStatus( status );		
		return;
	}

	status = mCheckForLoss();
	if( status != kGameInProgress ) 
	{
		fTeam->mSetGameStatus( status );		
		return;
	}
}

/*
	Purpose:	Checks if a team can claim victory.

	Inputs:		N/A
	Outputs:	( eGameStatus )	-	Can be one of many constants such as "kMissionFailed"
*/
eGameStatus
tNormalVictoryState::mCheckForWin( void )
{
	fEnemyCount = 0;
	for( int32 i = 0; i < fEnemySide.mGetSize(); i++ )
	{
		tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( fEnemySide[i] ) );
		fEnemyCount += enemyTeam->mGetShipsInPlayCount();
	}

	//
	//	Check if we won
	//
	if( fEnemyCount == 0 )
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}

/*
	Purpose:	Checks if a team can claim a loss.

	Inputs:		N/A
	Outputs:	( eGameStatus )	-	Can be one of many constants such as "kMissionFailed"
*/
eGameStatus
tNormalVictoryState::mCheckForLoss( void )
{
	Assert( fPlayerTeamID > -1 );
	fPlayerCount = 0;
	
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( fPlayerTeamID ) );
	fPlayerCount += playerTeam->mGetShipsInPlayCount();
		
	if( fPlayerCount == 0 )
	{
		return kPlayerLoses;
	}
	else
	{
		return kGameInProgress;
	}
}

/*
	Purpose:	Will add a team to the player side; as opposed to the enemy side.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
				( iTruth )	-	If set to kTrue, then this team will be considered the player
								team, which is useful for further references to the team
	Outputs:	N/A
*/
void
tNormalVictoryState::mAddPlayerSide(	eTeamID	Team,
										iTruth	PlayerTeam )
{
	if( PlayerTeam )
	{
		fPlayerTeamID = Team;
	}

	fPlayerSide.mAdd( Team );
}

/*
	Purpose:	Will add a team to the enemy side; as opposed to the player side.

	Inputs:		( eTeamID )	-	The ID associated with the team in question
	Outputs:	N/A
*/
void
tNormalVictoryState::mAddEnemySide( int32 Team )
{
	fEnemySide.mAdd( Team );
}

/*
	Purpose:	Will check if the game has ended when an object has been declared destroyed.

	Inputs:		( tShipInfo* )	-	The most recent ship to be destroyed
	Outputs:	N/A
*/
void
tNormalVictoryState::mOnDeath( tShipInfo* Ship )
{
	mCheckGameStatus();
}

/*
	Purpose:	Will check if the game has ended when an object has been declared captured.

	Inputs:		( tShipInfo* )	-	The most recent ship to be captured
	Outputs:	N/A
*/
void
tNormalVictoryState::mOnCapture( tShipInfo* Ship )
{
	mCheckGameStatus();
}

/*
	Purpose:	Will check if the game has ended when an object has been declared disengaged.

	Inputs:		( tShipInfo* )	-	The most recent ship to be disengaged
	Outputs:	N/A
*/
void
tNormalVictoryState::mOnDisengaged( tShipInfo* Ship )
{
	mCheckGameStatus();
}



//	15Nov99	zdrummond	Added Comment Header and Footer's
//	7Jul00	dsuleski	Added function purpose, inputs, and outputs
//	8Jul00	dsuleski	Documentation update