#include "scriptinterface.h"
#include "sfcAssert.h"
#include "ShipScriptDescription.h"
#include "scriptversion.h"
#include <time.h>



extern int32 TotalMaps;
extern const char* MapTypes[];
extern const char* MapLines[];

tScriptInterface gScriptInterface;
extern "C" __declspec( dllexport ) tScriptEntry* ScriptEntry( void )
{
	//
	//	Return a pointer to the interface for source use.
	//
	return &gScriptInterface;
}

extern "C" __declspec( dllexport ) const tMissionSpec* MissionSpecEntry( float Version )	
{
	//
	//	Since the server cannot grab pointers to gScriptInterface, this special entry point is created and
	//	returns the mission specs.
	//
	if( !gScriptInterface.mIsScriptCreated() )
	{
		gScriptInterface.mCreateScript();
	}

	if ( !gScriptInterface.mValidateVersion( Version ) )
	{
		return NULL;
	}

	return gScriptInterface.mGetScriptSpecs();
}

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tScriptInterface::tScriptInterface( void ) :
	fScript( NULL ),
	fMissionSpec( NULL ),
	fHost( kFalse )
{
	USE_CLASS( tTeamInfo, gTeamInfoAllocator );
	USE_CLASS( tShipInfo, gShipInfoAllocator );
}

/*
	Purpose:	Destructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tScriptInterface::~tScriptInterface( void )
{
	if( fScript )
	{
		delete fScript;
	}

	if( fMissionSpec )
	{
		delete fMissionSpec;
	}
}

/*
	Purpose:	Sets the status of the game for a team.

	Inputs:		( eTeamID )		-	The ID associated with the team in question
				( eGameStatus )	-	One of several constants that describe the intended team status
	Outputs:	N/A
*/
void
tScriptInterface::mSetGameStatus(	eTeamID		TeamID,
									eGameStatus	Status )
{
	tTeamInfo* endingTeam = fScript->mMission()->mGetTeamHandle( TeamID );
	if(endingTeam)
	{
		endingTeam->mSetGameStatus( Status );
	}
}

/*
	Purpose:	Accessor for the mission.

	Inputs:		N/A
	Outputs:	( tMissionInfo* )	-	The mission requested
*/
tMissionInfo*
tScriptInterface::mMission( void ) const
{
	if( fScript )
	{
		return fScript->mMission();
	}
	else
	{
		return NULL;
	}
}

/*
	Purpose:	Accessor for the script.

	Inputs:		N/A
	Outputs:	( tScript* )	-	The script requested
*/
tScript*
tScriptInterface::mScript( void ) const
{
	Assert( fScript );
	return fScript;
}

/*

	Purpose:	Will return the currently selected campaign title.  Should be used in conjunction with
				tTeamInfo::mScheduleNextMission(...)

	Inputs:		N/A
	Outputs:	( string )	-	The title of the campaign
*/
std::string
tScriptInterface::mGetCampaignTitle( void )
{
	tShipSystemsInfo	info;
	
	info.fAction = kOther;
	info.fInt32[0] = kRequestCampaignTitle;
	mEntityMgr( info );

	std::string temp = info.fTextPtr;
	return temp;
}

/*
	Purpose:	Evaluates whether or not a script has been created yet.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if a script has been created; kFalse, otherwise
*/
iTruth
tScriptInterface::mIsScriptCreated( void ) const
{
	if( fScript )
	{
		return kTrue;
	}
	else
	{
		return kFalse;
	}
}

/*
	Purpose:	Creates a new script.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mCreateScript( void )
{
	Assert( fScript == NULL );

	if( fScript )
	{
		delete fScript;
		fScript = 0;
	}
	
	fScript = ::AllocNewScript();
	
	if( fMissionSpec )
	{	
		delete fMissionSpec;
	}
	fMissionSpec = new tMissionSpec;
	fMissionSpec->fVersion = kCurrentScriptVersion;

	//
	//	Get the overrides for the ASCII map characters.
	//
	fScript->mASCIIOverrides( &fMissionSpec->fMapOverrides );

	//
	//	Read in the map lines and the terrain of each map.
	//
	for( int32 i = 0; i < TotalMaps; i++ )
	{
		fMissionSpec->fScriptMapData.push_back( tScriptMapData( MapTypes[i], MapLines, &fMissionSpec->fMapOverrides ) );
		fMissionSpec->fMapTerrainList.push_back( fScript->mAssignMapTerrainTypes( i ) );
	}

	//
	//	Get the briefing matrix.
	//
	fScript->mDefineBriefings();
	fScript->mSetBriefings();

	//
	//	Get the list of ships for each team.
	//
	fScript->mDefineTeamShipStrengths();

	//
	//	Get the mission specs for the script.
	//
	fScript->mScriptSpecs( fMissionSpec );
}

/*
	Purpose:	Assigns the function to create ships.

	Inputs:		( cbCreateShip* )	-	The function to fill in the function pointer that
										creates ships
	Outputs:	N/A
*/
void
tScriptInterface::mCreateShipCallback( cbCreateShip* Func )
{
	mCreateSpecShip = Func;
}

/*
	Purpose:	Assigns the function for entities.

	Inputs:		( cbEntityMgr* )	-	The function to fill in the functon pointer for entities
	Outputs:	N/A
*/
void
tScriptInterface::mEntityMgrCallback( cbEntityMgr* Func )
{
	mEntityMgr = Func;
}

/*
	Purpose:	Assigns the function for messages.

	Inputs:		( cbMessages* )	-	The function to fill in the function pointer for messages
	Outputs:	N/A
*/
void
tScriptInterface::mMessageCallback( cbMessages* Func )
{
	mPopupMessage = Func;
}
	
/*
	Purpose:	Assigns the function for random numbers.

	Inputs:		( cbRandom* )	-	The function to fill in the function pointer for random numbers
	Outputs:	N/A
*/
void
tScriptInterface::mRandomCallback( cbRandom* Func )
{
	mRandomInt = Func;
}

/*
	Purpose:	Assigns the function setting primitives.

	Inputs:		( cbPrimitive* )	-	The function to fill in the function pointer
	Outputs:	N/A
*/
void
tScriptInterface::mPrimitiveCallback( cbPrimitive* Func )
{
	mPrimitive = Func;
}

/*
	Purpose:	Initialization for mission.

	Inputs:		( tDynaverseScriptInfo& )	-	?
	Outputs:	N/A
*/
void
tScriptInterface::mInitialize( tDynaverseScriptInfo& Info )
{
	//
	//	Seed the random number.
	//
	srand( time( NULL ) );

	//
	//	This function is only processed at the beginning of the mission by the host, so set this local script
	//	to be the host.
	//
	if ( Info.fHosting )
		mSetHost();

	//
	//	Only the host runs the script
	//
	if ( mIsHost() )
	{
		Assert( fMissionSpec );

		//
		//	pick allied race
		//
		Info.fAlliedRace = mChooseRace( Info.fAlliedRaces, kSpecAnyRace );

		//
		//	pick enemy race
		//
		Info.fEnemyRace = mChooseRace( Info.fEnemyRaces, fMissionSpec->fEnemyRace );

		//
		//	pick system
		//
		Info.fSystemName = mChooseSystem( Info.fSystemNames );

		//
		//	set title of mission
		//
		Assert( fMissionSpec->fMissionTitle.size() < Info.fMissionTitle.capacity() );
		Info.fMissionTitle.erase();
		Info.fMissionTitle.append( fMissionSpec->fMissionTitle );
		Info.fMenuStates = fMissionSpec->fMenuStates;

		tMissionInfo* missionInfo = fScript->mMission();	
	}

	Info.fMapPicked = Info.fPreferedMap;

	//
	//	let the script change things if it needs to
	//
	if ( Info.fMissionType == kMultiplayerMission )
	{
		;
	}
	else
	{
		Info.fMapPicked = fScript->mInitializeStart( Info ) - 1;
	}
	
	if( Info.fPreferedMap > -1 )
	{
		Info.fMapPicked = Info.fPreferedMap;
		if( Info.fMapPicked >= TotalMaps )
		{
			Info.fMapPicked = 0;
		}
	}

	if ( mIsHost() )
	{
		Assert( strlen( MapTypes[Info.fMapPicked] ) < Info.fMapName.capacity() );
		Info.fMapName.erase();
		Info.fMapName.append( MapTypes[Info.fMapPicked] );
		
		mInitMapForRead( Info.fMapPicked );
	}
	fMapPicked = Info.fMapPicked;

	//
	//	Let the script set the drawing primitives.
	//
	fScript->mInitializeDraw( fMapPicked + 1 );

	if ( mIsHost() )
	{
		//
		//	Finally, set all of the above traits in the script.
		//
		fScript->mSetupScript( mIsHost(), Info.fPlayerTeam, Info.fMissionDate, Info.fMissionLocation, Info.fSystemName );
	}
}

/*
	Purpose:	Gives 50% chance of selecting a random race.

	Inputs:		( eRaceName )	-	The list of valid races
				( int32 )		-	The number of races
	Outputs:	( eRaceName )	-	The randomly chosen race
*/
eRaceName
tScriptInterface::mChooseRace(	eRaceName	RaceArray[],
								int32		ValidRaces )
{
	//
	//	make certain one of the valid races exists in the array - if not then just return
	//	the first one
	//
	iTruth valid = kFalse;
	int32 index = 0;
	while( RaceArray[index] != kNoRace )
	{
		if(( 1 << RaceArray[index]) & ValidRaces  )
		{
			valid = kTrue;
			break;
		}
		index++;
	}

	index = 0;		// default to first race
	while( valid )
	{
		if( mRandomInt( 1, 100) < 10  )
		{
			//
			//	count number of races in array
			//
			while( RaceArray[index] != kNoRace )
			{
				index++;
			}
			//
			//	pick a random race
			//
			index = mRandomInt( 0, index - 1 );
		}
		if(( 1 << RaceArray[index]) & ValidRaces  )
		{
			break;			// found one - we're done
		}
		index = 0;		// default to first race again
	}
	if(RaceArray[index] < kNoRace)
	{
		return RaceArray[index];
	}
	else
	{
		return kNoRaceType;
	}
}

/*
	Purpose:	Gives 75% chance of selecting a random system.

	Inputs:		( eSystems )	-	The list of valid systems
	Outputs:	( eSystems )	-	The randomly chosen system
*/
eSystems
tScriptInterface::mChooseSystem( eSystems SystemsArray[] )
{
	int32 index = 0;	// default to first system
	if( mRandomInt( 1, 100) < 75  )
	{
		// count number of systems in array
		while( SystemsArray[index] != kMaxSystems )
		{
			index++;
		}
		// pick a random system
		index = mRandomInt( 0, index - 1 );
	}
	return SystemsArray[index];
}

/*
	Purpose:	Once all ships are created for all teams, this is called.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mFinishSetupShips( void )
{
	//
	//	Most of the ships should already be setup so give the teams one last chance to create
	//	any ships they want before we start the game ask each team to create ships, and then
	//	setup it's victory condition
	//
	for( int32 teamIndex = 0; teamIndex < kMaxTeams; teamIndex++ )
	{
		tTeamInfo* team = fScript->mMission()->mGetTeamHandle( static_cast< eTeamID >( teamIndex ) );
		if( team )
		{
			team->mCreateShipsForTeam();			
			team->mSetupVictoryCondition();
		}
	}
	
	//
	//	create the rest of the ships
	//
	fScript->mCreateExternalShips( 0 );
}

#if 1	///DANHM -- this block was changed to reflect host migration
/*
	Purpose:	When all teams are created, the host script broadcasts a message to all scripts that it is
				now OK to do initialization for ships and such.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mScriptStartMission( void )
{
	if ( fScript )
		fScript->mStartMission();
}

/*
	Purpose:	This function is called if host migration takes place.  This will only be called on the script
				that is to be the new host.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mSetNewHost( void )
{
	fHost = true;
}
#endif

/*
	Purpose:	Whenever a ship is created, a copy of it must be saved elsewhere.

	Inputs:		( eTeamID )			-	The ID associated with the team that the ship in question is on
				( int32 )			-	The callback ID associated with the ship in question
				( int32 )			-	The ID associated with the ship in question
				( iAllocatorID )	-	A predetermined ID that describes a piece of memory for this ship
	Outputs:	N/A
*/
iTruth
tScriptInterface::mSaveEntityID(	eTeamID			Team,
									int32			CallbackID,
									int32			EntityID,
									iAllocatorID	AllocID )
{
#if 1	///DANHM -- this block was changed to reflect host migration
	//
	//	Only the host deals with ships
	//
//	if( !fScript->mHosting() )
//	{
//		return kFalse;
//	}
#endif

	tTeamInfo* fTeam = fScript->mMission()->mGetTeamHandle( eTeamID(Team) );

	tShipInfo* Ship = fTeam->mFindShipIndex( CallbackID );
	if( Ship == NULL )
	{
		//
		//	a ship has been created that did not come from this script (most likely it came from
		//	a remote script). we need to create a shadow version of it in this script.
		//
		tCreateShipInfo info;
		info.fShipData.fTeam				= eTeamID(Team);
		info.fShipData.fScriptCallbackID	= CallbackID;
		info.fShipData.mSetShipPosition( kStartPosition_Specified );
		info.fShipData.fShadowShipID		= EntityID;
		info.fShipData.fCreateShadowShip	= kTrue;
		info.fShipData.fCustomName			= NULL;
		info.fShipData.fShipInfoClassID		= AllocID;
		if( fTeam->mAddShipToTeamList( info )  )
		{
			Ship = fTeam->mFindShipIndex( CallbackID );
		}
	}

	if( Ship )
	{
		Ship->mSetShipID( EntityID );
		Ship->mSetShipStatus( kShipCreated );
		Ship->mReplaceShipCallbackEvents( kNotifyDefaultEvents );
		
		if( fScript->mHasMissionBegun() )
		{
			fScript->mNewInternalShip( Ship );
			tTeamInfo* team = fScript->mMission()->mGetTeamHandle( Ship->mGetTeam() );
			Assert( team );
			if( team && team->mIsTeamSetup() )
			{
				Ship->mSetupShip();
				gEventMgr.mRegisterActor( Ship );				
			}
		}
		//
		//	If the mission has not started yet, then initialize each team's ships.  This must be done on all
		//	client scripts as well, so send off a message to them.
		//
		else if( fScript->mIsMissionReadyToStart() && mIsHost() )
		{
#if 1	///DANHM -- this block was changed to reflect host migration
			tShipSystemsInfo info;

			info.fAction = kOther;
			info.fInt32[0] = kScriptStartMission;

			mEntityMgr( info );
//			fScript->mStartMission();
#endif
		}
	}

	return fScript->mHasMissionBegun();
}

/*
	Purpose:	An in-game event has occurred.

	Inputs:		( tShipSystemsInfo& )	-	Reference of a structure that contains the information about
											the event
	Outputs:	N/A
*/
void
tScriptInterface::mEntityEvent( tShipSystemsInfo& Info )
{
	gEventMgr.mEntityEvent( Info );
}

/*
	Purpose:	Initializes a general file that contains the maps for a script.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mInitMapForRead( void )
{
	fMapLineIndex = 0;
}

/*
	Purpose:	Initializes a file that contains the maps for a script.

	Inputs:		( int32 )	-	The map to look at in the script
	Outputs:	( iTruth )	-	kTrue if the seek was successful;
								kFalse, otherwise
*/
iTruth
tScriptInterface::mInitMapForRead( int32 Map )
{
	mInitMapForRead();
	const char* MapName = MapTypes[Map];

	for( ;; )
	{
		const char* mapLine = mGetNextMapLine();
		if( mapLine == NULL )
		{
			break;		// No matching map found
		}
		tString CurLine = mapLine;
		if( CurLine.substr(0,2) == "//" )
		{
			if( CurLine.find (MapName) >= 0 )
			{
				return kTrue;
			}
		}
	}
	return kFalse;
}

/*
	Purpose:	Reads in the next line of the current map.

	Inputs:		N/A
	Outputs:	( char* )	-	Constant pointer to a list of characters that contains
								the requested line
*/
const char*
tScriptInterface::mGetNextMapLine( void )
{
	if( MapLines[fMapLineIndex] == 0 )
	{
		return NULL;
	}
	return MapLines[fMapLineIndex++];
}

/*
	Purpose:	Determines the briefing message for a script.

	Inputs:		N/A
	Outputs:	( string )	-	The briefing message
*/
std::string&
tScriptInterface::mBriefingText( void )
{
	return( Messages[fBriefingIndex] );
}

/*
	Purpose:	Accessor for the specifications of a script.  If the current script is
				not up to date with the interface, then assertion will occur.

	Inputs:		N/A
	Outputs:	( tMissionSpec* )	-	The structure containing the mission specs
*/
const tMissionSpec*
tScriptInterface::mGetScriptSpecs( void )
{
	Assert( fMissionSpec );

	return fMissionSpec;
}

/*
	Purpose:	Validates the script version with the version from the source.  The version should be updated
				whenever the mission spec or the script interface changes.

	Inputs:		( float )	-	Version number
	Outputs:	( iTruth )	-	Returns true if the version numbers match
*/
iTruth
tScriptInterface::mValidateVersion( float Version )
{
	Assert( Version == kCurrentScriptVersion );
	if ( Version == kCurrentScriptVersion )
	{
		return kTrue;
	}

	return kFalse;
}

/*
	Purpose:	Whenever a new team is created from the engine, this will be called.

	Inputs:		( tTeamScriptDescription& )	-	Constant reference of a data structure
												containing the new team information
	Outputs:	N/A
*/
void
tScriptInterface::mNewTeam( const tTeamScriptDescription& TeamScriptDescription )
{
	fScript->mNewTeam( TeamScriptDescription );
}

/*
	Purpose:	Whenever a new ship is created from the engine, this will called.

	Inputs:		( tShipScriptDescription& )	-	Constant reference of a data structure
												containing the new ship information
	Outputs:	N/A
*/
void
tScriptInterface::mNewShip( const tShipScriptDescription& ShipScriptDescription )
{
	tTeamInfo* teamInfo = fScript->mMission()->mGetTeamHandle( ShipScriptDescription.fTeamID );
	Assert( teamInfo );

#if 1	///DANHM -- this block was changed to reflect host 
//	Assert( fScript->mHosting() );
//
//	//
//	//	We only make ships for the team we control 
//	//
//	if( teamInfo == NULL ||	!fScript->mHosting() )
//	{
//		return;
//	}
#endif

	teamInfo->mCreateNewShip( ShipScriptDescription );
}

/*
	Purpose:	Last step before the mission is initialized.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScriptInterface::mBeginMission( void )
{
	//
	//	now create the ships used in this mission
	//
	mFinishSetupShips();

	fScript->mReadyToStartMission();	
}

/*
	Purpose:	Accessor for the relationship between two teams.

	Inputs:		( eTeamID )				-	Team 1
				( eTeamID )				-	Team 2
	Outputs:	( eTeamRelationship )	-	One of several constants that describe
											the relationship between two teams
*/
eTeamRelationship
tScriptInterface::mGetTeamRelations(	eTeamID	Team1,
										eTeamID	Team2 )
{
	return( fScript->mMission()->mGetTeamRelations (Team1, Team2) );
}

/*
	Purpose:	Will find a given object on a map.  Please note that the indices into
				the map are 1-based.

	Inputs:		( char )	-	The object to look for on the original map.
				( int32* )	-	Allocated memory to hold the X-position of the object
				( int32* )	-	Allocated memory to hold the Y-position of the object
	Outputs:	N/A
*/
void
tScriptInterface::mFindMapObject(	unsigned char	MapObject,
									int32*			GridX,
									int32*			GridY )
{
	//
	//	find the start of the map
	//
	if( mInitMapForRead (fMapPicked) )
	{
		*GridY = 1;
		for( ;; )
		{
			const char* mapLine = mGetNextMapLine();

			if( mapLine == NULL ) break;			// Map too short
			if(  *mapLine == '/'  ) break;		// Map too short
			while(  *mapLine == ' '  ) mapLine++;

			if(  *mapLine != '+'  )
			{
				while(  *mapLine != '|'  )
				{
					if(  *mapLine == '\0'  ) break;		// Bad map line - missing '|'
					mapLine++;
				}
				if(  *mapLine != '|'  ) break;		// Bad map line - missing '|'
				*GridX = 1;
				for( ;; )
				{
					mapLine++;
					if( *mapLine == MapObject ) return;	// object found
					if( *mapLine == '|' ) break;			// end of line
					if( *mapLine == '\0' ) break;		// Bad map line - missing '|'
					*GridX += 1;
				}
				*GridY += 1;
			}
		}
	}

	//
	//	didn't find object
	//
	*GridX = -1;
	*GridY = -1;
}

/*
	Purpose:	Will find the maximum dimensions of a map.  Please note that the indices into
				the map are 1-based.

	Inputs:		( int32* )	-	Allocated memory to hold the X-position of the object
				( int32* )	-	Allocated memory to hold the Y-position of the object
	Outputs:	N/A
*/
void
tScriptInterface::mGetMapDimensions(	int32*	GridX,
										int32*	GridY )
{
	//
	//	find the start of the map
	//
	if( mInitMapForRead (fMapPicked) )
	{
		*GridY = 0;
		for( ;; )
		{
			const char* mapLine = mGetNextMapLine();

			if( mapLine == NULL ) break;			// Map too short
			if(  *mapLine == '/'  ) break;		// Map too short
			while(  *mapLine == ' '  ) mapLine++;

			if(  *mapLine != '+'  )
			{
				while(  *mapLine != '|'  )
				{
					if(  *mapLine == '\0'  ) break;		// Bad map line - missing '|'
					mapLine++;
				}
				if(  *mapLine != '|'  ) break;		// Bad map line - missing '|'
				*GridX = 0;
				for( ;; )
				{
					mapLine++;
					if( *mapLine == '|' ) break;			// end of line
					if( *mapLine == '\0' ) break;		// Bad map line - missing '|'
					*GridX += 1;
				}
				*GridY += 1;
			}
		}
	}
	else	// didn't find map
	{
		*GridX = 0;
		*GridY = 0;
	}
}

/*
	Purpose:	Returns whether this local script is acting as the host.

	Inputs:		N/A				
	Outputs:	( iTruth )	-	kTrue means that this script is the host
*/
iTruth
tScriptInterface::mIsHost( void )
{
	return fHost;
}

/*
	Purpose:	Sets this script to be the host.

	Inputs:		( iTruth )	-	kTrue is the default				
	Outputs:	N/A
*/
void
tScriptInterface::mSetHost( iTruth x )
{
	fHost = x;
}

// ************************************************************************************************
//					public DLL routines
//			This is now defined in each indivudal script
// ************************************************************************************************
/*
extern "C" __declspec( dllexport ) tScriptEntry* ScriptEntry(void)
{
	return &gScript;
}
*/

//	8Jul00	dsuleski	Documentation update
//	10Jul00	dsuleski	mRandomInt() is now its own function
//	28Jul00	dsuleski	mInitializeDraw() added it mInitialize()