//	<Name>	ShipInfo.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "ScriptInterface.h"
#include "sfcAssert.h"

tClassAllocator<tShipInfo> gShipInfoAllocator;

/*
	Purpose:	Constructor.

	Inputs:		( iterator )	-	tShipContainer
				( iterator )	-	tShipContainer
	Outputs:	N/A
*/
tShipIterator::tShipIterator(	tShipContainer::iterator	ShipIter,
								tShipContainer::iterator	End ) :
	fCurShip( ShipIter ),
	fEndOfShipList( End )
{}

/*
	Purpose:	Will append an evaluator to a ship, so when the "iterator" traits are used, the
				list of evaluators will be taken into account.

	Inputs:		( tShipEvaluator* )	-	Can be for "in play", "custom ID", or "ship class"
	Outputs:	N/A
*/
void
tShipIterator::mAddEvaluator( tShipEvaluator* Eval )
{
	fEvaluators.push_back( Eval );

	//
	//	Re-Eval the cur selection
	//
	if( mEvaluate( fCurShip ) == kFalse )
	{
		++( *this );
	}
}

/*
	Purpose:	Determines if an iterator element is valid.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the iterator is valid; kFalse, otherwise
*/
iTruth
tShipIterator::mValid( void )
{
	return fCurShip != fEndOfShipList;
}

/*
	Purpose:	Will jump to the next node in an iterator list and return the original node.

	Inputs:		N/A
	Outputs:	( tShipIterator& )	-	Reference of a ship that describes the next node requested
*/
tShipIterator&
tShipIterator::operator++( void )
{
	iTruth foundNext = kFalse;
	while( mValid() && !foundNext )
	{
		fCurShip++;
		if ( **this != NULL )
		{
			if( mEvaluate( fCurShip ) )
			{
				foundNext = kTrue;
			}		
		}
	}  

	return *this;
}

/*
	Purpose:	Will jump to the next node in an iterator list and return that node.

	Inputs:		( int )				-	This is just convention for postfix operators
	Outputs:	( tShipIterator )	-	The iterator of the ship requested
*/
tShipIterator
tShipIterator::operator++( int )
{
	tShipIterator tmp = *this;
	
	++(*this);	 

	return tmp;
}

/*
	Purpose:	Acts as a "dereferencer" for a tShipIterator.  The result of this is a tShipInfo*.

	Inputs:		N/A
	Outputs:	( tShipInfo* )	-	The result of the dereferencing if valid
*/
tShipInfo*
tShipIterator::operator*( void )
{
	if ( fCurShip == fEndOfShipList )
		return NULL;
	return *fCurShip; 
}

/*
	Purpose:	Acts as a "dereferencer" for a tShipIterator while invoking the elements of a
				structure.  The result of this indirection is a tShipInfo*.

	Inputs:		N/A
	Outputs:	( tShipInfo* )	-	The result of the indirection if valid
*/
tShipInfo*
tShipIterator::operator->( void )
{
	if ( fCurShip == fEndOfShipList )
		return NULL;
	return *fCurShip;
}

/*
	Purpose:	Evaluator for 2 tShipIterators.

	Inputs:		( iterator )	-	The right side of the evaluation
	Outputs:	( iTruth )		-	kTrue if both iterators are the same; kFalse, otherwise
*/
iTruth
tShipIterator::operator==( tShipContainer::iterator ShipIter )
{
	return ShipIter == fCurShip;
}

/*
	Purpose:	Will determine if a ship is valid.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the ship does not exist; kFalse, otherwise
*/
iTruth
tShipIterator::operator !( void )
{
	return !( fCurShip == fEndOfShipList );
}

/*
	Purpose:	Evaluates whether a ship is valid or not.

	Inputs:		( iterator )	-	The ship in question
	Outputs:	( iTruth )		-	kTrue if the ship in question is valid; kFalse, otherwise
*/
iTruth
tShipIterator::mEvaluate( tShipContainer::iterator Ship )
{
	iTruth valid = kTrue;
	
	if ( Ship == fEndOfShipList )
	{
		valid = kFalse;
	}
	else
	{
		for( tEvaluators::iterator eval = fEvaluators.begin();
			 eval != fEvaluators.end();
			 ++eval )
		{
			if( (*eval)->mEvaluate( Ship ) == kFalse )
			{
				valid = kFalse;
				break;
			}
		}
	}

	return valid;
}

/*
	Purpose:	This will be called when a tShipInPlayEval object is added to a
				tShipIterator list.

	Inputs:		( iterator )	-	The ship in question
	Outputs:	( iTruth )		-	kTrue if the ship in question is in play; kFalse, otherwise
*/
iTruth
tShipInPlayEval::mEvaluate( tShipContainer::iterator ShipIter ) const
{
	return ( *ShipIter )->mInPlay(); 
}

/*
	Purpose:	Constructor.

	Inputs:		( int32 )	-	The custom ID requested
	Outputs:	N/A
*/
tShipCustomIDEval::tShipCustomIDEval( int32 CustomID ) :
	fCustomID( CustomID )
{}

/*
	Purpose:	This will be called when a tShipCustomIDEval object is added to a
				tShipIterator list.

	Inputs:		( iterator )	-	The ship in question
	Outputs:	( iTruth )		-	kTrue if the ship in question is that custom ID; kFalse, otherwise
*/
iTruth
tShipCustomIDEval::mEvaluate( tShipContainer::iterator ShipIter ) const
{
	return ( *ShipIter )->mGetCustomID() == fCustomID; 
}

/*
	Purpose:	Constructor.

	Inputs:		( type_info& )	-	The "typeid()" associated with the ship class in question
	Outputs:	N/A
*/
tShipClassEval::tShipClassEval( const type_info& ShipClass ) :
	fShipClass( ShipClass )	
{}

/*
	Purpose:	This will be called when a tShipClassEval object is added to a
				tShipIterator list.

	Inputs:		( iterator )	-	The ship in question
	Outputs:	( iTruth )		-	kTrue if the ship in question is that ship class; kFalse,
									otherwise
*/
iTruth
tShipClassEval::mEvaluate( tShipContainer::iterator ShipIter ) const
{
	return *( *ShipIter ) == fShipClass; 
}

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tShipInfo::tShipInfo( void ) :
	fTeam( kNoTeam ),
	fOriginalTeam( kNoTeam ),
	fRace( kNoRaceType ),
	fStatus( kUndefined ),
	fShipBPV( 0 ),
	fCallbackID( -1 ),
	fStartPositionIndex( -1 ),
	fStartX( 0 ),
	fStartY( 0 ),
	fEndX( -1 ),
	fEndY( -1 ),
	fShipID( -1 ),
	fCustomID( -1 ),
	fCustomName( "" )
{}

/*
	Purpose:	Destructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tShipInfo::~tShipInfo( void )
{}

/*
	Purpose:	Initializes information for a ship that was not known when it was first created.

	Inputs:		( eTeamID )		-	The team this ship is on
				( eRaceName )	-	The race of this ship
				( int32 )		-	The ID used when sending messages out to the ship
				( int32 )		-	The position of the ship
				( int32 )		-	The custom ID of the ship
				( char* )		-	Constant pointer to a list of characters that make up its custom name
				( int32 )		-	Its x-starting position
				( int32 )		-	Its y-starting position
				( int32 )		-	Its x-ending position
				( int32 )		-	Its y-ending position
				( iTruth )		-	Whether or not this ship can be added to another team's fleet upon capture
	Outputs:	
*/
void
tShipInfo::mSetupShipInfo(	eTeamID			Team,
							eRaceName		Race,
							int32			CallbackID,
							int32			Position,
							int32			CustomID,
							const char*		CustomName,
							int32			StartX,
							int32			StartY,
							int32			EndX,
							int32			EndY,
							eShipOptions	ShipOptions )
{
	fTeam = Team;
	fOriginalTeam = Team;
	fRace = Race;
	fStatus = kUndefined;
	fShipBPV = 0;
	fCallbackID = CallbackID;
	fStartPositionIndex = Position;
	fStartX = StartX;
	fStartY = StartY;
	fEndX = EndX;
	fEndY = EndY; 
	fCustomID = CustomID;
	if(CustomName)
	{
		fCustomName = CustomName;
	}else
	{
		fCustomName = "";
	}
	fShipOptions = ShipOptions;
}

/*
	Purpose:	Zeroes out all commands for a ship that could have been created
				through AI.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mClearCommands( void )
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIClearCommands;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will set the target (depending upon its relations) for an AI-controlled
				ship to another ship.

	Inputs:		( tShipInfo* )	-	The new ship target
				( eAIPriority )	-	Can be one of many constants that determine the likelyhood
									of the AI actually carrying out this order
	Outputs:	N/A
*/
void
tShipInfo::mSetShipTarget(	tShipInfo*	TargetShip,
							eAIPriority	Priority )
{
	if( TargetShip )
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAITargetObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will remove the target (depending upon its relations) to another ship.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mRemoveTargets( void )
{
	tShipSystemsInfo info;

	info.fAction = kOther;
	info.fInt32[0] = kRemoveTargets;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will tell an AI-controlled ship to disengage.

	Inputs:		( eDisngageMethods )	-	One of two constants that describe how the
											ship in question should go about disengaging
				( eAIPriority )			-	Can be one of many constants that determine
											the likelyhood of the AI actually carrying
											out this order
	Outputs:	N/A
*/
void
tShipInfo::mDisengage(	eDisengageMethods	Method,
						eAIPriority			Priority )
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIDisengage;
	info.fInt32[1] = Priority;
	info.fInt32[2] = Method;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will tell an AI-controlled ship to tractor another ship.

	Inputs:		( tShipInfo* )	-	The ship to tractor
				( int32 )		-	The x-coordinate of the tractoree's destination
				( int32 )		-	The y-coordinate of the tractoree's destination
				( eAIPriority )	-	Can be one of many constants that determine the
									likelyhood of the AI actually carrying out this
									order
	Outputs:	N/A
*/
void
tShipInfo::mTractorEntity(	tShipInfo*	Entity,
							int32		EndX,
							int32		EndY,
							eAIPriority	Priority )
{
	if( Entity )
	{
		fShipEvents |= kNotifyEndTractor | kNotifyBeginTractor;

		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIMoveObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fInt32[3] = fShipEvents;
		info.fGridX = EndX - 1;
		info.fGridY = EndY - 1;
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will tell an AI-controlled ship to defend another ship.

	Inputs:		( tShipInfo* )	-	The ship that is to be defended
				( float )		-	The radius around the defended ship that will trigger the
									defender to initiate chase
				( float )		-	The distance that the defender should circle around the
									defended ship
				( eAIPriority )	-	Can be one of many constants that determine the
									likelyhood of the AI actually carrying out this
									order
	Outputs:	N/A
*/
void
tShipInfo::mDefendEntity(	tShipInfo*	Entity,
							float		ThreatRadius,
							float		FollowRadius,
							eAIPriority	Priority )
{
	if( Entity )
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIDefendObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fFloat[0] = ThreatRadius;
		info.fFloat[1] = FollowRadius;
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will tell an AI-controlled ship to follow another ship.

	Inputs:		( tShipInfo* )	-	The ship is to be followed.
				( eAIPriority )	-	Can be one of many constants that determine the
									likelyhood of the AI actually carrying out this
									order
	Outputs:	N/A
*/
void
tShipInfo::mFollowEntity(	tShipInfo*	Entity,
							eAIPriority	Priority )
{
	if( Entity )
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFollowObject;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Entity->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will tell an AI-controlled ship to fire a probe.

	Inputs:		( tShipInfo* )	-	The ship that the probe is intended for
				( eAIPriority )	-	Can be one of many constants that determine the
									likelyhood of the AI actually carrying out this
									order
	Outputs:	N/A
*/
void
tShipInfo::mFireProbe(	tShipInfo*	Target,
						eAIPriority	Priority )
{
	if( Target )
	{
		tShipSystemsInfo info;

		info.fAction = kIssueAICommand;
		info.fInt32[0] = kAIFireProbe;
		info.fInt32[1] = Priority;
		info.fInt32[2] = Target->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );
	}
}

/*
	Purpose:	Will go to a general setting for weapons on a ship

	Inputs:		( eWeaponStates )	-	Can be one of many constants that determine
										the state of the ship's weapons
	Outputs:	N/A
*/
void
tShipInfo::mSetWeaponState( eWeaponStates weaponState )
{
	tShipSystemsInfo info;

	info.fAction = kOther;
	info.fInt32[0] = kSetWeaponState;
	if ( weaponState < 0 || weaponState >= kNumWeaponStates )
		info.fInt32[1] = static_cast< int >( kWeaponState_Default );
	else
		info.fInt32[1] = static_cast< int >( weaponState );
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will tell an AI-controlled ship to patrol around a predetermined
				number of checkpoints.

	Inputs:		( float )		-	The range at which the ship should be at for each
									point in question
				( int32 )		-	Number of checkpoints
				( tMapPoint* )	-	A list of the positions of the checkpoints
				( eAIPriority )	-	Can be done of many constants that determine the
									likelyhood of the AI actually carrying out this
									order
	Outputs:	N/A
*/
void
tShipInfo::mPatrol(	float		DetectionRange,
					int32		PatrolPointCount,
					tMapPoint*	PatrolPoints,
					eAIPriority	Priority )
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIPatrol;
	info.fInt32[1] = Priority;
	info.fInt32[2] = PatrolPointCount;
	info.fFloat[0] = DetectionRange;
	info.fTextPtr =( const char* )PatrolPoints;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Sets the state of the officers on an AI-controlled ship.

	Inputs:		( eAIOfficers )	-	The officer in requested
				( iTruth )		-	kTrue for the officer to be active; kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mChangeAIOfficerState(	eAIOfficers	Officer,
									iTruth		State )
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAIChangeOfficerState;
	info.fInt32[1] = Officer;
	info.fInt32[2] = State;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will tell an AI-controlled ship to cloak.  Please note that
				"kCloakOfficer" must be turned off first with the above
				function, "mChangeAIOfficerState()" before using this function.

	Inputs:		( iTruth )	-	kTrue to turn on cloaking; kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mCloak( iTruth State )
{
	tShipSystemsInfo info;

	info.fAction = kIssueAICommand;
	info.fInt32[0] = kAICloakShip;
	info.fInt32[1] = State;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will tell an AI-controlled ship what it's current speed should
				be set to.

	Inputs:		( int32 )	-	The requested new speed of the ship
	Outputs:	N/A
*/
void
tShipInfo::mSetShipThrust( int32 NewThrust )
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = NewThrust;
	info.fInt32[0] = 0;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Sets the maximum speed for a ship.

	Inputs:		( int32 )	-	The requested max speed of the ship
	Outputs:	N/A
*/
void
tShipInfo::mSetShipMaxSpeed( int32 MaxSpeed )
{
	tShipSystemsInfo info;

	info.fAction = kSetNewThrust;
	info.fFloat[0] = MaxSpeed;
	info.fInt32[0] = -1;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Sets the immediate speed for a ship.

	Inputs:		( int32 )	-	The requested immediate speed of the ship
	Outputs:	N/A
*/
void
tShipInfo::mSetShipImmediateSpeed( int32 ImmediateSpeed )
{
	tShipSystemsInfo info;

	info.fAction = kOther;
	info.fFloat[0] = ImmediateSpeed;
	info.fInt32[0] = kSetShipSpeedImmediate;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Find out this ship's current speed.

	Inputs:		N/A
	Outputs:	( int32 )	-	The current speed of this ship
*/
int32
tShipInfo::mGetShipSpeed( void )
{
	tShipSystemsInfo info;

	info.fAction = kOther;
	info.fInt32[0] = kGetShipSpeed;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return info.fInt32[1];
}

/*
	Purpose:	Sets an AI-controlled ship to human-controlled.  The human
				that is set to control this is the human that "owns" the
				team that this ship is on.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mSetShipPlayerControlled( void )
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = -1;
	gScriptInterface.mEntityMgr( info );

	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( mGetTeam() );
	Assert( team );
	if ( team )
		if ( team->mIsTeamHumanControlled() )
			team->mSetCurrentlyControlledShip( this );
}

/*
	Purpose:	Sets a ship to AI-controlled; thereby, taking control
				away from any human players.

	Inputs:		( iTruth )	-	kTrue if you want the ship to be
								AI-controlled; kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mSetShipAIControlled( iTruth AIControlled )
{
	tShipSystemsInfo info;

	info.fAction = kSetPlayerControlledShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = AIControlled;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Additional state event calls can be added here.

	Inputs:		( iNotifyEvents )	-	Can be one of several constants such as
										"kNotifyDamage" or "kNotifyDeepScan"
	Outputs:	N/A
*/
void
tShipInfo::mSetShipCallbackEvents( iNotifyEvents NewEvents )
{
	mReplaceShipCallbackEvents( fShipEvents | NewEvents );
}

/*
	Purpose:	State event calls can be added here; however, any state
				calls that existed here are not gone.

	Inputs:		( iNotifyEvents )	-	Can be one of several constants such as
										"kNotifyDamage" or "kNotifyDeepScan"
	Outputs:	N/A
*/
void
tShipInfo::mReplaceShipCallbackEvents( iNotifyEvents NewEvents )
{
	tShipSystemsInfo info;

	info.fAction = kSetNewEvents;
	info.fInt32[0] = NewEvents;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	fShipEvents = NewEvents;
}

/*
	Purpose:	An accessor for those events that are currently attached to a
				particular ship.

	Inputs:		N/A
	Outputs:	( iNotifyEvents )	-	Can be a combination of several constant
										such as "kNotifyDamage" or "kNotifyAtDeath"
*/
iNotifyEvents
tShipInfo::mGetShipCallbackEvents( void )
{
	return fShipEvents;
}

/*
	Purpose:	Will set the destination of an AI-controlled ship.  Please note
				that X and Y are 1-based, not 0-based!

	Inputs:		( int32 )	-	The x-coordinate of the destination
				( int32 )	-	The y-coordinate of the destination
	Outputs:	N/A
*/
void
tShipInfo::mSetFinalDestination(	int32	EndX,
									int32	EndY )
{
	fShipEvents |= kNotifyAtDestination;

	tShipSystemsInfo info;

	info.fAction = kSetFinalDestination;
	info.fInt32[0] = mGetStartPositionIndex() - kStartPosition_G;
	info.fInt32[1] = fShipEvents;
	info.fGridX = EndX - 1;
	info.fGridY = EndY - 1;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Will track when a certain distance is reached from a ship to a
				point in space.  Please note that X and Y are 1-based.

	Inputs:		( float )	-	The requested distance from a predetermined point
				( int32 )	-	The x-coordinate of the predetermined point
				( int32 )	-	The y-coordinate of the predetermined point
				( iTruth )	-	kTrue if an triggered event is requested when the
								ship enters or exits across the radial boundary
								determined by the requested distance; kFalse,
								if it triggers only upon entrance
	Outputs:	N/A
*/
void
tShipInfo::mTrackTacticalDistance(	float	Distance,
									int32	GridX,
									int32	GridY,
									iTruth	NotifyInOut )
{
	fShipEvents |= kNotifyAtDistance;
	fTrackingDistance = Distance;
	if( NotifyInOut )
	{
		Distance = -Distance;	// a neg. dist means the game should detect boundry crossings both ways
	}

	tShipSystemsInfo info;

	info.fAction = kTrackDistanceFromTarget;
	info.fInt32[0] = fShipEvents;
	info.fFloat[0] = Distance;
	info.fGridX = GridX - 1;
	info.fGridY	= GridY - 1;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Tells a ship which button goes where.  Please note that the button
				number is 1-based.

	Inputs:		( eTeamID )	-	The ID of the team associated with the ship
				( int32 )	-	The index into the array of messages that were
								registered with the buttons
				( iTruth )	-	?
				( int32 )	-	The index into the array of buttons that were
								registered with the messages
	Outputs:	N/A
*/
void
tShipInfo::mSetCommButton(	eTeamID	TeamID,
							int32	MsgID,
							iTruth	SetAsTarget,
							int32	ButtonNumber )
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = kTrue;	
	info.fInt32[1] = MsgID;
	info.fInt32[2] = ButtonNumber;
	info.fInt32[3] = SetAsTarget;
	info.fInt32[4] = TeamID;
	info.fText = Messages[MsgID].c_str();
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Physically erases the comm buttons from a ship's comm panel.

	Inputs:		( eTeamID )	-	The ID of the team associated with the ship
				( int32 )	-	The index into the array of buttons that were
								registered with the messages
	Outputs:	N/A
*/
void
tShipInfo::mClearCommButton(	eTeamID	TeamID,
								int32	ButtonNumber )
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = kFalse;	
	info.fInt32[1] = -1;
	info.fInt32[2] = ButtonNumber;
	info.fInt32[3] = kFalse;
	info.fInt32[4] = TeamID;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Erases the message associated with a comm button.

	Inputs:		( eTeamID )	-	The ID of the team associated with the ship
				( int32 )	-	The index into the array of messages that were
								registered with the buttons
	Outputs:	N/A
*/
void
tShipInfo::mClearCommMsg(	eTeamID	TeamID,
							int32	MsgID )
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = kFalse;	
	info.fInt32[1] = MsgID;
	info.fInt32[2] = -1;
	info.fInt32[3] = kFalse;
	info.fInt32[4] = TeamID;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Physically erases everything about the comm buttons on the ship.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mClearAllCommButtons( void )
{
	tShipSystemsInfo info;

	info.fAction = kSetCommButton;
	info.fInt32[0] = kFalse;	
	info.fInt32[1] = -1;
	info.fInt32[2] = -1;
	info.fInt32[3] = kFalse;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the status of a ship.

	Inputs:		( eShipStatus )	-	Can be one of several constants such as
									"kPlayerDisengaged" or "kPlayerCaptured"
	Outputs:	N/A
*/
void
tShipInfo::mSetShipStatus(	eShipStatus	Status,
							iTruth		SetState )
{
	if( SetState )
	{
		fStatus |= Status;
	}
	else
	{
		fStatus &= ~Status;
	}
}

/*
	Purpose:	Evaluator for ship status.

	Inputs:		( int32u )	-	The bitwise status requested for testing
	Outputs:	( iTruth )	-	kTrue if the requested status is contained somewhere
								within the ship's actual status
*/
iTruth
tShipInfo::mTestShipStatus( int32u Status )
{
	if( fStatus & Status )
	{
		return kTrue;
	}
	return kFalse;
}

/*
	Purpose:	Assigns an ID to a ship.

	Inputs:		( int32 )	-	The ID to assign
	Outputs:	N/A
*/
void
tShipInfo::mSetShipID( int32 ShipID )
{
	fShipID = ShipID;
}

/*
	Purpose:	Assigns the team to a ship.

	Inputs:		( eTeamID )	-	The ID of the team to assign
	Outputs:	N/A
*/
void
tShipInfo::mSetTeam( eTeamID NewTeam )
{
	fTeam = NewTeam;
}

/*
	Purpose:	Accessor for the options for this ship.

	Inputs:		N/A
	Outputs:	( eShipOptions )	-	The options for this ship
*/
eShipOptions
tShipInfo::mGetShipOptions( void )
{
	return fShipOptions;
}

/*
	Purpose:	Accessor for the team that the ship is currently on.

	Inputs:		N/A
	Outputs:	( eTeamID )	-	The ID associated with the team the ship
								is currently on
*/
eTeamID
tShipInfo::mGetTeam( void )
{
	return fTeam;
}

/*
	Purpose:	Accessor for the team that the ship originally started on.

	Inputs:		N/A
	Outputs:	( eTeamID )	-	The ID associated with the team the ship
								originally came from
*/
eTeamID
tShipInfo::mGetOriginalTeam( void )
{
	return fOriginalTeam;
}

/*
	Purpose:	Accessor for the ID of a ship.

	Inputs:		N/A
	Outputs:	( int32 )	-	The requested ID of the ship
*/
int32
tShipInfo::mGetShipID( void )
{
	return fShipID;
}

/*
	Purpose:	Assigns a class type to a ship.

	Inputs:		( eClassTypes )	-	Can be one of several constants such as
									"kClassFrigate" that describe the ship type
	Outputs:	N/A
*/
void
tShipInfo::mSetClassType( eClassTypes ClassType )
{
	fClassType = ClassType;
}

/*
	Purpose:	Accessor for the class type of a ship.

	Inputs:		N/A
	Outputs:	( eClassTypes )	-	Can be one of several constants such as
									"kClassFrigate" that describe the ship type
*/
eClassTypes
tShipInfo::mGetClassType( void )
{
	return fClassType;
}

/*
	Purpose:	Accessor for the class name of a ship.

	Inputs:		N/A
	Outputs:	( char* )	-	Constant pointer to a list of characters that
								make up the class name of a ship
*/
const char*
tShipInfo::mGetClassName( void )
{
	tShipSystemsInfo info;

	info.fAction = kGetShipClassName;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return( info.fTextPtr );
}

/*
	Purpose:	Accessor for the current target ID of a ship, if any.

	Inputs:		N/A
	Outputs:	( int32 )	-	The ID associated with the ship that is its
								current target, if any
*/
int32
tShipInfo::mGetShipTargetID( void )
{
	tShipSystemsInfo info;

	info.fAction = kGetShipTargetID;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return( info.fInt32[0] );
}

/*
	Purpose:	Accessor for the positioning of a ship on the map.

	Inputs:		N/A
	Outputs:	( iMapPosition )	-	One of several constants that describe where
										the ship has started from.
*/
iMapPosition
tShipInfo::mGetStartPositionIndex( void )
{
	return fStartPositionIndex;
}

/*
	Purpose:	Accessor for the custom ID of a ship.

	Inputs:		N/A
	Outputs:	( int32 )	-	The custom ID of a ship, if applicable;
								otherwise, -1
*/
int32
tShipInfo::mGetCustomID( void )
{
	return fCustomID;
}

/*
	Purpose:	Accessor for the custom name of a ship.

	Inputs:		N/A
	Outputs:	( char* )	-	Constant pointer to a list of characters that
								makes up the custom name of a ship, if applicable;
								otherwise, NULL
*/
const char*
tShipInfo::mGetCustomName( void )
{
	return fCustomName.c_str();
}

/*
	Purpose:	Accessor for the X-coordinate starting point of a ship.  Please
				note that the return value is 1-based.

	Inputs:		N/A
	Outputs:	( int32 )	-	The X-coordinate of the start position
*/
int32
tShipInfo::mGetStartX( void )
{
	return fStartX + 1;
}

/*
	Purpose:	Accessor for the Y-coordinate starting point of a ship.  Pleae
				note that the return value is 1-based.

	Inputs:		N/A
	Outputs:	( int32 )	-	The Y-coordinate of the start position
*/
int32
tShipInfo::mGetStartY( void )
{
	return fStartY + 1;
}

/*
	Purpose:	Accessor for the X-coordinate ending point of a ship.  Please
				note that the return value is 1-based.

	Inputs:		N/A
	Outputs:	( int32 )	-	The X-coordinate of the end point
*/
int32
tShipInfo::mGetEndX( void )
{
	return fEndX + 1;
}

/*
	Purpose:	Accessor for the Y-coordinate ending point of a ship.  Please
				note that the return value is 1-based.

	Inputs:		N/A
	Outputs:	( int32 )	-	The Y-coordinate of the end point
*/
int32
tShipInfo::mGetEndY( void )
{
	return fEndY + 1;
}

/*
	Purpose:	Accessor for the current tracking distance from the ship
				to a predetermined point.


	Inputs:		N/A
	Outputs:	( float )	-	The distance between the ship and the point
*/
float
tShipInfo::mGetTrackingDistance( void )
{
	return fTrackingDistance;
}

/*
	Purpose:	Assigns a BPV to a ship.

	Inputs:		( int32 )	-	The BPV to be assigned
	Outputs:	N/A
*/
void
tShipInfo::mSetShipBPV( int32 ShipBPV )
{
	fShipBPV = ShipBPV;
}

/*
	Purpose:	Accessor for the BPV of a ship.

	Inputs:		N/A
	Outputs:	( int32 )	-	The BPV of the ship
*/
int32
tShipInfo::mGetShipBPV( void )
{
	return fShipBPV;
}

/*
	Purpose:	Evaluates whether a ship is still in play or not.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the ship is alive and has not been
								captured or disengaged
*/
iTruth
tShipInfo::mInPlay( void )
{
	if( fMissionInfo->mIsTeamLeaving(fTeam) )
	{
		return kFalse;
	}

	if( mTestShipStatus ( kShipDisengaged | kShipDestroyed) )
	{
		return kFalse;
	}
	return kTrue;
}

/*
	Purpose:	Determines the damage of a ship per BPV.

	Inputs:		( eRaceName )	-	The race of the enemy of the ship
	Outputs:	( int32 )		-	The BPV for damage
*/
int32
tShipInfo::mAssessDamage( eRaceName EnemyRace )
{
	int32 DamageBPV = 0;

	if( fStatus & kShipCaptured )
	{
		if( fClassType == kClassShuttle )
		{
			DamageBPV = fShipBPV;
		}
		else if( EnemyRace == kOrion )
		{
			DamageBPV = fShipBPV * 5;
		}
		else
		{
			DamageBPV = fShipBPV * 2;
		}
	}
	else if( fStatus & kShipDestroyed )
	{
		DamageBPV = fShipBPV;
	}
	else if( fStatus & kShipDisengaged )
	{
		if( EnemyRace != kOrion )
		{
			DamageBPV = float(fShipBPV * .5);
		}
	}

	return DamageBPV;
}

/*
	Purpose:	Accessor for a ship's callback ID.

	Inputs:		N/A
	Outputs:	( int32 )	-	The callback ID of the ship
*/
int32
tShipInfo::mGetCallbackID( void )
{
	return fCallbackID;
}

/*
	Purpose:	Accessor for the 2D position of a ship.

	Inputs:		( int32* )	-	The memory allocated for the X-position
				( int32* )	-	The memory allocated for the Y-position
	Outputs:	N/A
*/
void
tShipInfo::mGetGridPosition(	int32*	GridX,
								int32*	GridY )
{
	tShipSystemsInfo info;

	info.fAction = kGetShipPosition;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	*GridX = info.fGridX + 1;
	*GridY = info.fGridY + 1;
}

/*
	Purpose:	Determines the distance between two ships.

	Inputs:		( tShipInfo* )	-	The "other" ship
	Outputs:	( float )		-	The distance between this ship and the "other" ship
*/
float
tShipInfo::mGetTacticalDistance( tShipInfo* TargetShip )
{
	if( TargetShip )
	{
		tShipSystemsInfo info;

		info.fAction = kGetTacticalDistance;
		info.fInt32[0] = kGetShipPosition;
		info.fInt32[1] = TargetShip->mGetShipID();
		info.fShipEntityID = fShipID;
		gScriptInterface.mEntityMgr( info );

		return( info.fFloat[0] );
	}
	return 0.0;
}

/*
	Purpose:	Determines the distance from the ship to a letter on the map.

	Inputs:		( char )	-	The letter on the ASCII map requested
	Outputs:	( float )	-	The distance between the ship and the predetermined point
*/
float
tShipInfo::mGetTacticalDistance( unsigned char MapObject )
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fInt32[1] = MapObject;
	info.fGridX = -1;					// == -1 indicates that distance is between ship and map object
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return( info.fFloat[0] );
}

/*
	Purpose:	Determines the distance from the ship to a 2D point.

	Inputs:		( int32 )	-	The x-coordinate of the predetermined 2D point
				( int32 )	-	The y-coordinate of the predetermined 2D point
	Outputs:	( float )	-	The distance between the ship and the predetermined point
*/
float
tShipInfo::mGetTacticalDistance(	int32	GridX,
									int32	GridY )
{
	tShipSystemsInfo info;

	info.fAction = kGetTacticalDistance;
	info.fInt32[0] = kGetMapEntryPosition;
	info.fGridX = GridX - 1;
	info.fGridY = GridY - 1;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return( info.fFloat[0] );
}

/*
	Purpose:	Accessor for the status of the human controlling the ship.  Remember,
				alert status is only associated with a human player, not necessarily
				a ship.

	Inputs:		N/A
	Outputs:	( eAlertLevels )	-	Can be kAlertRed, kAlertYellow, or kAlertGreen
*/
eAlertLevels
tShipInfo::mGetAlertStatus( void )
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = -1;
	gScriptInterface.mEntityMgr( info );

	return( eAlertLevels(info.fInt32[0]) );
}

/*
	Purpose:	Modifies the alert status of a human player.

	Inputs:		( eAlertLevels )	-	Can be kAlertRed, kAlertYellow, or kAlertGreen
	Outputs:	N/A
*/
void
tShipInfo::mSetAlertStatus( eAlertLevels Status )
{
	tShipSystemsInfo info;

	info.fAction = kGetAlertStatus;
	info.fInt32[0] = Status;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Accessor for the race of the ship.

	Inputs:		N/A
	Outputs:	( eRaceName )	-	Can be one of several constants such as "kFederation"
									that describe the race of the ship
*/
eRaceName
tShipInfo::mGetRace( void )
{
	return fRace;
}

/*
	Purpose:	Will destroy a ship.  Completely clear it off of the map.  Kaput.  Please
				remember that the "this" pointer memory is invalid after this call.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mDestroyShip( void )
{
	mSetSystemHealthFromOrig( kNoWeaponHardPoint, 0.0f );

	//
	//	Remember to set "this" to NULL after you call this function...the memory is no longer
	//	guaranteed to be valid at this point.
	//
}

/*
	Purpose:	Will destroy a ship without an explosion.  Completely clear it off of the map.  Kaput.  Please
				remember that the "this" pointer memory is invalid after this call.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mDestroyShipSilently( void )
{
	mSetSystemHealthFromOrig( kNoWeaponHardPoint, 0.0f );

	//
	//	This really isn't so silent.
	//



	//
	//	Remember to set "this" to NULL after you call this function...the memory is no longer
	//	guaranteed to be valid at this point.
	//
}

/*
	Purpose:	Initialization for a state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mSetupState( void )
{}

/*
	Purpose:	Initialization for a comm state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mSetupCommState( void )
{}

/*
	Purpose:	Initialization for a ship.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tShipInfo::mSetupShip( void )
{}


/*
	Purpose:	Accessor for the amount of damage that a ship has incurred.  Please
				note that this is not the amount of health remaining, but really the
				amount of damage incurred.

	Inputs:		N/A
	Outputs:	( float )	-	The percentage of damage on the ship
*/
float
tShipInfo::mGetDamagePercent( void )
{
	tShipSystemsInfo info;

	info.fAction = kGetShipDamagePercent;
	info.fShipEntityID = fShipID;
	info.fFloat[0] = 100.0;				// assume 100% damage if no ship
	gScriptInterface.mEntityMgr( info );

	return info.fFloat[0];
}

/*
	Purpose:	Will change the custom name of a ship.

	Inputs:		( int32 )	-	The index into the array of messages that
								describes the ship
	Outputs:	N/A
*/
void
tShipInfo::mChangeName( int32 MsgID )
{
	tShipSystemsInfo info;

	info.fAction = kSetObjectName;
	info.fTextPtr = Messages[MsgID].c_str();
	info.fShipEntityID = fShipID;

	//
	//	Set the custom name on the source side.
	//
	gScriptInterface.mEntityMgr( info );

	//
	//	Set the custom name on the script side.
	//
	mSetCustomName( info.fTextPtr );
}

/*
	Purpose:	Accessor for the rank of a specific officer onboard the ship.

	Inputs:		( eOfficerTypes )	-	One of several constants that describe the
										various officers onboard
	Outputs:	( eOfficerRank )	-	One of several constants that describe the
										rankings of an officer
*/
eOfficerRank
tShipInfo::mGetOfficerRank( eOfficerTypes Officer )
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kUnknownRank;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	return( (eOfficerRank)info.fInt32[2] );
}

/*
	Purpose:	Can set the rank of an officer onboard the ship.

	Inputs:		( eOfficerTypes )	-	One of several constants that describe the
										various officers onboard
				( eOfficerRank)		-	One of several constants that describe the
										rankings of an officer
	Outputs:	N/A
*/
void
tShipInfo::mSetOfficerRank(	eOfficerTypes	Officer,
							eOfficerRank	Rank )
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = Rank;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Can set the name of an officer.

	Inputs:		( eOfficerTypes )	-	One of several constants that describe the
										various officers onboard
				( int32 )			-	The index into the messages that describes
										the officer's name
	Outputs:	N/A
*/
void
tShipInfo::mSetOfficerName(	eOfficerTypes	Officer,
							int32			MsgID )
{
	tShipSystemsInfo info;

	info.fAction = kGetSetOfficerRank;
	info.fInt32[0] = Officer;
	info.fInt32[1] = kMaxOfficerRank;
	info.fShipEntityID = fShipID;
	info.fText = Messages[MsgID].c_str();
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Can set or "unset" an object as targetable.

	Inputs:		( iTruth )	-	kTrue if targetable; kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mSetTargetable( iTruth State )
{
	tShipSystemsInfo info;

	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipTargetable;
	info.fInt32[1] = State;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );

	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipAbleToTarget;
	info.fInt32[1] = State;
	info.fShipEntityID = fShipID;
	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the state of the "Deep Scan" bar.

	Inputs:		( iTruth )	-	kTrue if the bar is filled or being filled;
								kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mSetDeepScanStatus( iTruth State )
{
	fDeepScanStatus = State;
}

/*
	Purpose:	Modifier for the custom ID of a ship.

	Inputs:		( int32 )	-	The new custom ID of the ship
	Outputs:	N/A
*/
void
tShipInfo::mSetCustomID( int32 X )
{
	fCustomID = X;
}

/*
	Purpose:	Modifier for the custom name of a ship.

	Inputs:		( char* )	-	Constant pointer to a list of characters that
								describes the custom name of the ship
	Outputs:	N/A
*/
void
tShipInfo::mSetCustomName( const char* Name )
{
	fCustomName = Name;
}

/*
	Purpose:	Accessor for the state of the "Deep Scan" bar

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the bar is on; kFalse, otherwise
*/
iTruth
tShipInfo::mGetDeepScanStatus( void ) const
{
	return fDeepScanStatus;
}

/*
	Purpose:	Allows ability to change the hit points of a hardpoint on a ship.

	Inputs:		( eSystemHardPoint )	-	One of several constants that determine
											which parts of a ship are affected
				( float )				-	The percentage requested for the hardpoint
				( iTruth )				-	kTrue if the hardpoint can be healed;
											kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mSetSystemHealthFromOrig(	eSystemHardPoint	System,
										float				Percentage,
										iTruth				AllowHeal )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;
	
	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kSetPercentageFromOrig;
	info.fInt32[2] = System;
	info.fInt32[3] = AllowHeal;
	
	info.fFloat[0] = Percentage;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Allows ability to change the hit points of a hardpoint on a ship based
				on its current health.

	Inputs:		( eSystemHardPoint )	-	One of several constants that determine
											which parts of a ship are affected
				( float )				-	The percentage requested for the hardpoint
				( iTruth )				-	kTrue if rounding should occur; kFalse, otherwise
	Outputs:	N/A
*/
void
tShipInfo::mSetSystemHealthFromCurr(	eSystemHardPoint	System,
										float				Percentage,
										iTruth				RoundUp )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;


	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kSetPercentageFromCurr;
	info.fInt32[2] = System;
	info.fInt32[3] = RoundUp;
	
	info.fFloat[0] = Percentage;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Does damage to random hardpoints of this ship.

	Inputs:		( float )	-	The percentage requested for the hardpoint
	Outputs:	N/A
*/
void
tShipInfo::mDamageRandomInternals( float percentage )
{
	tShipSystemsInfo info;
	
	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipSystems;
	info.fInt32[1] = kSetPercentageFromOrig;
	info.fInt32[2] = kRandomInternalsHardPoint;
	info.fInt32[3] = 0;
	info.fFloat[0] = percentage;
	info.fShipEntityID = fShipID;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Does damage to random hardpoints of this ship.

	Inputs:		( int32 )	-	Number of hardpoints that should get hit				
	Outputs:	N/A
*/
void
tShipInfo::mDamageRandomInternals( int numHits )
{
	tShipSystemsInfo info;
	
	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipSystems;
	info.fInt32[1] = kDamageSystem;
	info.fInt32[2] = kRandomInternalsHardPoint;
	info.fInt32[3] = numHits;
	info.fShipEntityID = fShipID;

	gScriptInterface.mEntityMgr( info );
}


/*
	Purpose:	Accessor for viewing the health of a hardpoint on a ship.

	Inputs:		( eSystemHardPoint )	-	One of several constants that determine
											which parts of a ship are affected
	Outputs:	N/A
*/
void
tShipInfo::mGetHealthOfSystem( eSystemHardPoint System )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kGetHealthOfSystem;
	info.fInt32[2] = System;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Accessor for viewing the max health of a hardpoint on a ship.

	Inputs:		( eSystemHardPoint )	-	One of several constant that determine
											which parts of a ship are affected
	Outputs:	N/A
*/
void
tShipInfo::mGetMaxHealthOfSystem( eSystemHardPoint System )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;
	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kGetMaxHealthOfSystem;
	info.fInt32[2] = System;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tShipInfo::mGetPercentAtMarkOriginal_Trigger(	eSystemHardPoint	System,
												float				AtThisMark,
												iTruth				OrLower )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kGetPercentAtMarkOriginal_Trigger;
	info.fInt32[2] = System;
	info.fInt32[3] = OrLower;

	info.fFloat[0] = AtThisMark;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tShipInfo::mClearPreviousTriggers( eSystemHardPoint System )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipSystems;

	info.fInt32[1] = kClearPreviousSysTriggers;
	info.fInt32[2] = System;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of non-transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that describe
													which item is in question
				( int32 )						-	The amount to remove
	Outputs:	N/A
*/
void
tShipInfo::mRemoveItemFromStores(	eAccessShipStores_ItemTypes	Item,
									int32						Amount )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kRemoveItemFromStores;
	info.fInt32[2] = Item;
	info.fInt32[3] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of transporter items on a ship.

	Inputs:		( eScriptTransportItems )	-	One of several constants that describe
												which item is in question
				( int32 )					-	The amount to remove
	Outputs:	N/A
*/
void
tShipInfo::mRemoveItemFromStores(	eScriptTransportItems	TransporterItem,
									int32					Amount )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores_Transporter;

	info.fInt32[1] = kRemoveItemFromStores;
	info.fInt32[2] = TransporterItem;
	info.fInt32[3] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of non-transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that overall
													describe the item in question
				( eAccessShipStores_Subypes )	-	One of several constants that describe
													the non-transporter item in question
				( int32 )						-	The amount to add
	Outputs:	N/A
*/
void
tShipInfo::mAddItemToStores(	eAccessShipStores_ItemTypes	Item,
								eAccessShipStores_SubTypes	Subtype,
								int32						Amount)
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kAddItemToStores;
	info.fInt32[2] = Item;
	info.fInt32[3] = Subtype;
	info.fInt32[4] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that overall
													describe the item in question
				( eScriptTransportItems )		-	One of several constants that describe
													the transporter item in question
				( int32 )						-	The amount to add
	Outputs:	N/A
*/
void
tShipInfo::mAddItemToStores(	eAccessShipStores_ItemTypes	Item,
								eScriptTransportItems		TransporterItem,
								int32						Amount)
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipStores_Transporter;
	info.fShipEntityID = fShipID;

	info.fInt32[1] = kAddItemToStores;
	info.fInt32[2] = TransporterItem;
	info.fInt32[3] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of non-transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that overall
													describe the item in question
				( eAccessShipStores_SubTypes )	-	One of several constants that describe
													the non-transporter item in question
				( int32 )						-	The amount to modify to
	Outputs:	N/A
*/
void
tShipInfo::mSetItemInStores(	eAccessShipStores_ItemTypes	Item,
								eAccessShipStores_SubTypes	Subtype,
								int32						Amount)
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kSetItemInStores;
	info.fInt32[2] = Item;
	info.fInt32[3] = Subtype;
	info.fInt32[4] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Modifier for the amount of transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that overall
													describe the item in question
				( eScriptTransportItems )		-	One of several constants that describe
													the transporter item in question
				( int32 )						-	The amount to modify to
	Outputs:	N/A
*/
void
tShipInfo::mSetItemInStores(	eAccessShipStores_ItemTypes	Item,
								eScriptTransportItems		TransporterItem,
								int32						Amount)
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipStores_Transporter;
	info.fShipEntityID = fShipID;

	info.fInt32[1] = kSetItemInStores;
	info.fInt32[2] = TransporterItem;
	info.fInt32[3] = Amount;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Invokes a callback for the amount of non-transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that describe
													the non-transporter item in question
	Outputs:	N/A
*/
void
tShipInfo::mGetAmountOfStore( eAccessShipStores_ItemTypes Item )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kGetAmountOfStore;
	info.fInt32[2] = Item;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Invokes a callback for the amount of transporter items on a ship.

	Inputs:		( eScriptTransportItems )	-	One of several constants that describe the
												transporter item in question
	Outputs:	N/A
*/
void
tShipInfo::mGetAmountOfStore( eScriptTransportItems TransporterItem )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipStores_Transporter;

	info.fShipEntityID = fShipID;

	info.fInt32[1] = kGetAmountOfStore;
	info.fInt32[2] = TransporterItem;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Invokes a callback for the maximum amount of non-transporter items on a ship.

	Inputs:		( eAccessShipStores_ItemTypes )	-	One of several constants that describe
													the non-transporter item in question
	Outputs:	N/A
*/
void
tShipInfo::mGetMaxAmountOfStore( eAccessShipStores_ItemTypes Item )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kGetMaxAmountOfStore;
	info.fInt32[2] = Item;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Invokes a callback for the maximum amount of transporter items on a ship.

	Inputs:		( eScriptTransportItems )	-	One of several constants that describe the
												transporter item in question
	Outputs:	N/A
*/
void
tShipInfo::mGetMaxAmountOfStore( eScriptTransportItems Item )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kGetMaxAmountOfStore;
	info.fInt32[2] = Item;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tShipInfo::mGetAmountAtMark_Trigger(	eAccessShipStores_ItemTypes	Item,
										float						AtThisMark,
										iTruth						OrLower )
{
	tShipSystemsInfo info;
	info.fShipEntityID = fShipID;

	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kGetAmountAtMark_Trigger;
	info.fInt32[2] = Item;
	info.fInt32[3] = OrLower;

	info.fFloat[0] = AtThisMark;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tShipInfo::mClearPreviousTriggers( eAccessShipStores_ItemTypes Item )
{
	tShipSystemsInfo info;
	info.fShipEntityID = fShipID;

	info.fAction = kAccessShip;
	info.fInt32[0] = kAccessShipStores;

	info.fInt32[1] = kGetAmountAtMark_Trigger;
	info.fInt32[2] = Item;
	
	gScriptInterface.mEntityMgr( info );	
}


/*
	Purpose:	Sets a ship's shuttle and missile counts.  -1 does nothing.

	Inputs:		( int32 )	-	Amount of marines on this ship
				( int32 )	-	Amount of spare parts on this ship
				( int32 )	-	Amount of mines on this ship
				( int32 )	-	Amount of shuttles on this ship
				( int32 )	-	Amount of fighter bays on this ship   < 4
				( int32 )	-	Amount of fighters in each bay   < 4
				( std::string )	-	Spec name of the fighter to use
				( int32 )	-	Amount of type I, standard missiles on this ship
				( int32 )	-	Amount of type I, medium missiles on this ship
				( int32 )	-	Amount of type I, fast missiles on this ship
				( int32 )	-	Amount of type IV, standard missiles on this ship
				( int32 )	-	Amount of type IV, medium missiles on this ship
				( int32 )	-	Amount of type IV, fast missiles on this ship
	Outputs:	N/A
*/
void
tShipInfo::mSetShipLoadout(	int32	Marines,
							int32	SpareParts,
							int32	Mines,
							int32	Shuttles,
							int32	FighterBays,
							int32	FightersPerBay,
							std::string		FighterSpec,
							int32	MissileTypeIStandard,
							int32	MissileTypeIMedium,
							int32	MissileTypeIFast,
							int32	MissileTypeIVStandard,
							int32	MissileTypeIVMedium,
							int32	MissileTypeIVFast )
{
	if ( Marines >= 0 )
		mSetItemInStores( kMarines, kStandard, Marines );
	if ( SpareParts >= 0 )
		mSetItemInStores( kSpareParts, kStandard, SpareParts );
	if ( Mines >= 0 )
		mSetItemInStores( kSmall_Mines, kStandard, Mines );
	if ( Shuttles >= 0 )
		mSetShuttles( Shuttles );
	if ( ( FighterBays >= 0 ) && ( FightersPerBay > 0 ) )
		mSetFighters( FighterBays, FightersPerBay, FighterSpec);
	if ( MissileTypeIStandard >= 0 )
		mSetItemInStores( kMissilesTypeI, kStandard, MissileTypeIStandard );
	if ( MissileTypeIMedium >= 0 )
		mSetItemInStores( kMissilesTypeI, kSpeed_Med, MissileTypeIMedium );
	if ( MissileTypeIFast >= 0 )
		mSetItemInStores( kMissilesTypeI, kSpeed_Fast, MissileTypeIFast );
	if ( MissileTypeIVStandard >= 0 )
		mSetItemInStores( kMissilesTypeIV, kStandard, MissileTypeIVStandard );
	if ( MissileTypeIVMedium >= 0 )
		mSetItemInStores( kMissilesTypeIV, kSpeed_Med, MissileTypeIVMedium );
	if ( MissileTypeIVFast >= 0 )
		mSetItemInStores( kMissilesTypeIV, kSpeed_Fast, MissileTypeIVFast );
}

/*
	Purpose:	Sets the number of shuttles on this ship.

	Inputs:		( int32 )	-	The new number of shuttles on this ship.
	Outputs:	N/A
*/
void
tShipInfo::mSetShuttles( int32 Shuttles )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kSetShuttlebay;
	info.fInt32[1] = Shuttles;

	gScriptInterface.mEntityMgr( info );
}


/*
	Purpose:	Sets the number of fighters and type on this ship.

	Inputs:		( int32 )	-	The new number of fighter bays on this ship
				( int32 )	-	The new number of fighters per bay
				( std::string )		-	The spec of the fighter
	Outputs:	N/A
*/
void
tShipInfo::mSetFighters(	int32	FighterBays,
							int32	FightersPerBay,
							std::string	FighterSpec )
{
	tShipSystemsInfo info;
	info.fAction = kAccessShip;
	info.fShipEntityID = fShipID;

	info.fInt32[0] = kSetFighterbay;
	info.fInt32[1] = FighterBays;
	info.fInt32[2] = FightersPerBay;
	if ( FighterSpec.empty() )
		info.fTextPtr = NULL;
	else
		info.fTextPtr = FighterSpec.c_str();

	gScriptInterface.mEntityMgr( info );
}


/*
	Purpose:	Accessor for the BPV of this ship without the loadout.

	Inputs:		N/A
	Outputs:	( int32 )	-	The total BPV of this ship without loadout.
*/
int32
tShipInfo::mGetBPV( void )
{
	return 0;
}

/*
	Purpose:	Accessor for the BPV of this ship with the loadout.

	Inputs:		N/A
	Outputs:	( int32 )	-	The total BPV of this ship with loadout.
*/
int32
tShipInfo::mGetTotalBPV( void )
{
	return 0;	//	UNUSED!!!
}



//	4Nov99	zdrummond	Added Comment Header and Footer's
//	29Apr00	dsuleski	Added mSetCustomID(...) for ships given to the script through the meta
//	2May00	dsuleski	Overloaded ! operator for ship iterator
//	19May00	dsuleski	Updating mGetAmountOfStore with detection message
//	26May00	dsuleski	Overloaded ++ prefix operator to increment the tShipIterator before evalulating
//	8Jul00	dsuleski	Documentation update
//	21May01	dsuleski	Added various functionality for ship loadout