//	<Name>	State.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "State.h"
#include "sfcAssert.h"
#include "ScriptInterface.h"

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tState::tState( void ) :
	fActor( NULL ),
	fStateID( kInvalidStateID )
{
	fMissionInfo = gScriptInterface.mMission();
}

/*
	Purpose:	Initialization of a state.

	Inputs:		( tActor* )		-	The actor that is connected to the state in question
				( iStateID )	-	The ID associated with the state in question
	Outputs:	N/A
*/
void
tState::mSetup(	tActor*		Actor,
				iStateID	StateID )
{
	fActor = Actor;
	fStateID = StateID;
}

/*
	Purpose:	Will set a timer within a state.

	Inputs:		( eTimerIntervalTypes )	-	Can be one of many constants such as "kTimerTurnsSet"
				( int32 )				-	The ID of the timer being created
				( float )				-	A scalar determining the time at which a timer should
											be triggered.
	Outputs:	N/A
*/
void
tState::mSetTimedInterval(	eTimerIntervalTypes	Type,
							int32				TimerID,
							float				Interval )
{
	Assert( fActor );
	if( fActor == NULL )
	{
		return;
	}

	fActor->mSetTimedInterval( Type, TimerID, Interval, fStateID );
}

/*
	Purpose:	An evaluator that compares a state to a "typeid()".

	Inputs:		( type_info& )	-	Constant reference to the "typeid()" in question
	Outputs:	( iTruth )		-	kTrue if the state and the "typeid()" in question are the same;
									kFalse, otherwise
*/
iTruth
tState::operator==( const type_info& StateType )
{
	return (typeid( *this ) == StateType);
}

/*
	Purpose:	Accesses the human-readable name of a state.

	Inputs:		N/A
	Outputs:	( char* )	-	Constant pointer to a list of characters, which contain the name
								of the state in question
*/
const char*
tState::mGetName( void )
{
	const type_info& typeID = typeid( *this );
	return typeID.raw_name();
}

/*
	Purpose:	An accessor to the ID of a state.

	Inputs:		N/A
	Outputs:	( iStateID )	-	The ID associated with the state in question
*/
iStateID
tState::mGetStateID( void )
{
	return fStateID;
}

/*
	Purpose:	Initialization of a state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tState::mBeginState( void )
{}

/*
	Purpose:	Cleanup of a state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tState::mEndState( void )
{}

/*
	Purpose:	Message cracker for ships that have current events triggered.

	Inputs:		( tShipInfo* )	-	The ship, if applicable, that is related to the event in question.
				( tEventInfo& )	-	Constant reference to a structure that contains information
									pertaining to the event in question
	Outputs:	N/A
*/
void
tState::mHandleShipEvent(	tShipInfo*						Ship,
							const tEventMgr::tEventInfo&	EventInfo )
{	
	switch( EventInfo.fEvent ) 
	{
		case kNotifyAtDeath:
			mOnDeath( Ship );
			break;
		
		case kNotifyOnCapture:
			Ship->mSetTeam( static_cast< eTeamID >( EventInfo.fEventItem[0] ) );
			mOnCapture( Ship );
			break;
		
		case kNotifyDisengaged:
			mOnDisengaged( Ship );
			break;
		
		case kNotifyAtDistance:
			mAtDistance( Ship );
			break;
		
		case kNotifyAtDestination:
			mAtDestination( Ship );
			break;
		
		case kNotifyAtTimedInterval:
			mOnTimedInterval( EventInfo.fEventItem[2] );
			break;
		
		case kNotifyCommButtonPress:
		{
			tShipInfo* targetShip = fMissionInfo->mFindShipID( EventInfo.fEventItem[1] );
			mOnCommButtonPress( Ship, EventInfo.fEventItem[0], targetShip );
		}
		break;
		
		case kNotifyDamage:
		{
			tShipInfo* hostileShip = fMissionInfo->mFindShipID( EventInfo.fEventItem[0] );
			mOnDamage( Ship, hostileShip, static_cast< eDamageToShip >( EventInfo.fEventItem[1] ) );
		}
		break;
		
		case kNotifyDeepScan:
			mOnDeepScan( Ship, Ship->mGetDeepScanStatus() );
			break;
		
		case kNotifyAlertStatusChange:
			mOnAlertStatusChange( Ship, Ship->mGetAlertStatus() );
			break;
		
		case kNotifyProbeLaunch:
			mOnProbeLaunch( Ship );
			break;
		
		case kNotifyBeginOrbit:
			mOnBeginOrbit( Ship );
			break;
		
		case kNotifyEndOrbit:
			mOnEndOrbit( Ship );
			break;
		
		case kNotifyBeginTractor:
		{
			//ZDNF this is wrong way to get target info for a tractor
			tShipInfo* target = fMissionInfo->mFindShipID( Ship->mGetShipTargetID() );
			mOnBeginTractor( target, Ship );
		}
		break; 

		
		case kNotifyEndTractor:
		{
			//ZDNF this is wrong way to get target info for a tractor
			tShipInfo* target = fMissionInfo->mFindShipID( Ship->mGetShipTargetID() );
			mOnEndTractor( target, Ship );
		}
		break;
		
		case kNotifyFlashingButtonPressed:
			mOnFlashingButtonPressed();
			break;
		
		case kNotifyTransportedItem:
		{
			tShipInfo* transportTo = fMissionInfo->mFindShipID( EventInfo.fEventItem[1] );	
			tShipInfo* transportFrom = fMissionInfo->mFindShipID( EventInfo.fEventItem[2] );	
			mOnTransportedItem( transportTo, transportFrom, static_cast< eScriptTransportItems >( EventInfo.fEventItem[0] ) );
		}
		break;
		
		case kNotifyChangingShip:
			mOnChangingShip( Ship );
			break;
	
		case kNotifySpecialTransport:
			mOnSpecialTransport( Ship );
			break;

		case kNotifyShipSystemUpdate:
		{
			eShipUpdateType updateType = static_cast< eShipUpdateType >( EventInfo.fEventItem[0] );
			eSystemHardPoint system = static_cast< eSystemHardPoint >( EventInfo.fEventItem[1] );	
			if( updateType == kShipCurrentValue || updateType == kShipMaxValue )
			{
				mOnShipSystemUpdate( Ship, updateType, system, EventInfo.fEventItem[2] );
			}
			else if( updateType == kShipCurrentPercent || updateType == kShipMaxPercent )
			{
				mOnShipSystemUpdate( Ship, updateType, system, EventInfo.fEventItem_Float[0] );
			}
			else
			{
				Assert( kFalse );
			}

			break;
		}

		case kNotifyShipStoresUpdate:
		{
			eShipUpdateType updateType = static_cast< eShipUpdateType >( EventInfo.fEventItem[0] );
			eAccessShipStores_ItemTypes item = static_cast< eAccessShipStores_ItemTypes >( EventInfo.fEventItem[1] );
			int32 subItem = EventInfo.fEventItem[2];	//	 will cast below

			switch ( EventInfo.fEventItem[4] )
			{
			case kSupplySubItem:
				{
					mOnShipStoresUpdate( Ship, updateType, item, static_cast< eAccessShipStores_SubTypes >( subItem ), EventInfo.fEventItem[3] );
				}
				break;

			case kTransporterSubItem:
			default:
				{
					mOnShipStoresUpdate( Ship, updateType, item, static_cast< eScriptTransportItems >( subItem ), EventInfo.fEventItem[3] );
				}
			}
			break;
		}
		
		case kNotifyPlayerLeaving:
		{
			mOnPlayerLeaving( EventInfo.fEventItem[0], static_cast< eTeamID >( EventInfo.fEventItem[1] ) );
//			if ( EventInfo.fEventItem[2] )		// extra
//				mOnHostChanged();
		}
		break;

		//
		//	This event is only sent to the host.  In this case, set this script as the host.
		//
		case kNotifyHostChanged:
			gScriptInterface.mSetHost( kTrue );
//			mOnPlayerLeaving( -1, static_cast< eTeamID >( EventInfo.fEventItem[2] ) );
			mOnHostChanged();
			break;

		case kNotifyCommStateChanged:
			mOnCommStateChange( reinterpret_cast< tActor* >( EventInfo.fEventItem[0] ), EventInfo.fEventItem[1], static_cast< eStateEvent >( EventInfo.fEventItem[2] ) );
			mOnCommStateChange( reinterpret_cast< tActor* >( EventInfo.fEventItem[0] ), EventInfo.fEventItem[1], EventInfo.fEventItem[3], static_cast< eStateEvent >( EventInfo.fEventItem[2] ) );
			break;

		case kNotifyStateChanged:
		{
			tActor* actor = reinterpret_cast< tActor* >( EventInfo.fEventItem[0] );
			tState* state = actor->mGetState( EventInfo.fEventItem[1] );
			mOnStateChange( actor, state, (eStateEvent)EventInfo.fEventItem[2] );
			break;
		}

		case kNotifyNewTeam:
		{
			tTeamInfo* team = reinterpret_cast< tTeamInfo* >( EventInfo.fEventItem[0] );

			mOnNewTeam( team );
			break;
		}
	}
}

/*
	Purpose:	Will be called when an object is destroyed.

	Inputs:		( tShipInfo* )	-	If the object is a ship, then this parameter is the ship in
									question; otherwise, it is NULL
	Outputs:	N/A
*/
void
tState::mOnDeath( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when an object is captured.

	Inputs:		( tShipInfo* )	-	If the object is a ship, then this parameter is the ship in
									question; otherwise, it is NULL
	Outputs:	N/A
*/
void
tState::mOnCapture( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when an object has disengaged.

	Inputs:		( tShipInfo* )	-	The disengaged ship in question
	Outputs:	N/A
*/
void
tState::mOnDisengaged( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when an object is within a predetermined distance from a predetermined
				point in space.

	Inputs:		( tShipInfo* )	-	The ship in question that is tracking its distance from a point
	Outputs:	N/A
*/
void
tState::mAtDistance( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when an object has reached a predetermined destination.

	Inputs:		( tShipInfo* )	-	The ship that has reach the destination
	Outputs:	N/A
*/
void
tState::mAtDestination( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when a timer for its "fChildActor" has been triggered.

	Inputs:		( int32 )	-	The ID of the timer in question
	Outputs:	N/A
*/
void
tState::mOnTimedInterval( int32 TimerID )
{}

/*
	Purpose:	Will be called when a human player selects a comm button.

	Inputs:		( tShipInfo* )	-	The ship owned by the human player in question
				( int32 )		-	This is the index into the array of messages used when first
									registering a comm state
				( tShipInfo* )	-	The ship that "owns" the comm button
	Outputs:	N/A
*/
void
tState::mOnCommButtonPress(	tShipInfo*	Ship,
							int32		ButtonID,
							tShipInfo*	TargetShip )
{}

/*
	Purpose:	Will be called when an object takes damage.

	Inputs:		( tShipInfo* )		-	The ship that received the damage
				( tShipInfo* )		-	The ship that inflicted the damage
				( eDamageToShip )	-	One of several constants that describes how the damage
										came about
	Outputs:	N/A
*/
void
tState::mOnDamage(	tShipInfo*		DamagedShip,
					tShipInfo*		HostileShip,
					eDamageToShip	DamageType )
{}

/*
	Purpose:	Will be called whenever the "Deep Scan" button is pressed.

	Inputs:		( tShipInfo* )	-	The ship that is doing the deep scan
				( iTruth )		-	kTrue if the "Deep Scan" mechanism was orginally off; otherwise,
									kFalse if the "Deep Scan" mechanism was originally on
	Outputs:	N/A
*/
void
tState::mOnDeepScan(	tShipInfo*	Ship,
						iTruth		DeepScanOn )
{}

/*
	Purpose:	Will be called whenever a human player changes alert status.

	Inputs:		( tShipInfo* )		-	The currently controlled ship during the status change
				( eAlertLevels )	-	Can be kAlertGreen, kAlertYellow, or kAlertRed
	Outputs:	N/A
*/
void
tState::mOnAlertStatusChange(	tShipInfo*		Ship,
								eAlertLevels	AlertStatus )
{}

/*
	Purpose:	Will be called upon launching a probe.

	Inputs:		( tShipInfo* )	-	The ship that has launched the probe
	Outputs:	N/A
*/
void
tState::mOnProbeLaunch( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when a ship first begins an orbit.

	Inputs:		( tShipInfo* )	-	The ship in question that was commanded to orbit
	Outputs:	N/A
*/
void
tState::mOnBeginOrbit( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when a ship ends an orbit.

	Inputs:		( tShipInfo* )	-	The ship in question that was commanded to stop orbitting
	Outputs:	N/A
*/
void
tState::mOnEndOrbit( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called when a ship is first connected to a foreign tractor beam.

	Inputs:		( tShipInfo* )	-	The ship that is being tractored
				( tShipInfo* )	-	The ship that initiated the tractoring
	Outputs:	N/A
*/
void
tState::mOnBeginTractor(	tShipInfo*	TractorTarget,
							tShipInfo*	TractoringShip )
{}

/*
	Purpose:	Will be called when a ship is disconnected from a foreign tractor beam.

	Inputs:		( tShipInfo* )	-	The ship that was disconnected from the tractor beam
				( tShipInfo* )	-	The ship that ended the tractoring
	Outputs:	N/A
*/
void
tState::mOnEndTractor(	tShipInfo*	TractorTarget,
						tShipInfo*	TractoringShip )
{}

/*
	Purpose:	Will be called upon a human pressing a flashing button.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tState::mOnFlashingButtonPressed( void )
{}

/*
	Purpose:	Will be called upon the exchange of a transporter item.

	Inputs:		( tShipInfo* )				-	The ship that is receiving the transporter item
				( tShipInfo* )				-	The ship that is losing the transporter item
				( eScriptTransportItems )	-	One constant of several that describe the various
												transporter items
	Outputs:	N/A
*/
void
tState::mOnTransportedItem(	tShipInfo*				ShipTo,
							tShipInfo*				ShipFrom,
							eScriptTransportItems	Item )
{}

/*
	Purpose:	Will be called when a human decides to change primary control over a ship.

	Inputs:		( tShipInfo* )	-	The new ship being moved to
	Outputs:	N/A
*/
void
tState::mOnChangingShip( tShipInfo* Ship )
{}

/*
	Purpose:	?

	Inputs:		
	Outputs:	
*/
void
tState::mOnSpecialTransport( tShipInfo* Ship )
{}

/*
	Purpose:	Will be called whenever a ship hardpoint is explicitly modified from the script level.

	Inputs:		( tShipInfo* )			-	The ship in question
				( eShipUpdateType )		-	Can be one of several constants that describe what 'value' means
				( eSystemHardPoint )	-	The hardpoint that was affected
				( int32 )				-	The value of the damage or healing invoked
	Outputs:	N/A
*/
void
tState::mOnShipSystemUpdate(	tShipInfo*			Ship,
								eShipUpdateType		Type,
								eSystemHardPoint	System,
								int32				Value )
{}

/*
	Purpose:	Will be called whenever a ship hardpoint is explicitly modified from the script level.

	Inputs:		( tShipInfo* )			-	The ship in question
				( eShipUpdateType )		-	Can be one of several constants that describe what 'percent' means
				( eSystemHardPoint )	-	The hardpoint that was affected
				( float )				-	The percentage of the damage or healing invoked
	Outputs:	N/A
*/
void
tState::mOnShipSystemUpdate(	tShipInfo*			Ship,
								eShipUpdateType		Type,
								eSystemHardPoint	System,
								float				Percent )
{}

/*
	Purpose:	Will be called upon a change in item stores if explicitly invoked through the script level.

	Inputs:		( tShipInfo* )					-	The ship in question
				( eShipUpdateType )				-	Can be one of several constants that describe what 'value' means
				( eAccessShipStores_ItemTypes )	-	The overall description of the item in question
				( eAccessShipStores_SubTypes )	-	The type of non-transporter item in question
				( int32 )						-	The number of units remaining in question
	Outputs:	N/A
*/
void
tState::mOnShipStoresUpdate(	tShipInfo*					Ship,
								eShipUpdateType				Type, 
								eAccessShipStores_ItemTypes	Item,
								eAccessShipStores_SubTypes	SubItem,
								int32						Value )
{}

/*
	Purpose:	Will be called upon a change in item stores if explicitly invoken through the script level.

	Inputs:		( tShipInfo* )					-	The ship in question
				( eShipUpdateType )				-	Can be one of several constants that describe what 'value' means
				( eAccessShipStores_ItemTypes )	-	The overall description of the item in question
				( eScriptTransportItems )		-	The type of transporter item in question
				( int32 )						-	The number of units remaining in question
	Outputs:	N/A
*/
void
tState::mOnShipStoresUpdate(	tShipInfo*					Ship,
								eShipUpdateType				Type,
								eAccessShipStores_ItemTypes	Item,
								eScriptTransportItems		SubItem,
								int32						Value )
{}

/*
	Purpose:	Will be called when a human player leaves un-gracefully from a multiplayer game.

	Inputs:		( int32 )	-	The ID associated with the player in question who left the game
				( eTeamID )	-	The ID associated with the team that the player was on
	Outputs:	N/A
*/
void
tState::mOnPlayerLeaving( int32 PlayerNumber, eTeamID TeamID )
{}

/*
	Purpose:	Will be called when a host has left abruptly.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tState::mOnHostChanged( void )
{}

/*
	Purpose:	Will be called upon the entrance of a new team while in the midst of a script.

	Inputs:		( tTeamInfo* )	-	The team in question
	Outputs:	N/A
*/
void
tState::mOnNewTeam( tTeamInfo* Team )
{}

/*
	Purpose:	Will be called upon any explicit changes in state on any level.

	Inputs:		( tActor* )		-	The actor who "owns" the state in question
				( tState* )		-	The state in question
				( eStateEvent )	-	kEndingState if leaving the state in question; otherwise,
									kStartingState if coming into the state in question
	Outputs:	N/A
*/
void
tState::mOnStateChange(	tActor*		Actor,
						tState*		State,
						eStateEvent	EventType )
{}

/*
	Purpose:	Will called upon a change in comm buttons.

	Inputs:		( tActor* )		-	The actor who "owns" the comm state in question
				( int32 )		-	The ID associated with the state in question
				( eStateEvent )	-	kEndingState if leaving the state in question; otherwise,
									kStartingState if coming into the state in question
	Outputs:	N/A
*/
void
tState::mOnCommStateChange(	tActor*		Actor,
							int32		State,
							eStateEvent	EventType )
{}

/*
	Purpose:	Will called upon a change in comm buttons.

	Inputs:		( tActor* )		-	The actor who "owns" the comm state in question
				( int32 )		-	The ID associated with the state in question
				( int32 )		-	The ID of the buttons that caused the state change
				( eStateEvent )	-	kEndingState if leaving the state in question; otherwise,
									kStartingState if coming into the state in question
	Outputs:	N/A
*/
void
tState::mOnCommStateChange(	tActor*		Actor,
							int32		State,
							int32		ButtonID,
							eStateEvent EventType )
{}


//	8Nov99	zdrummond	Added Comment Header and Footer's
//	8Jul00	dsuleski	Documentation update
