//	<Name>	TeamInfo.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "ScriptInterface.h"
#include "sfcAssert.h"
#include "State.h"
#include "ShipScriptDescription.h"

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tTeamInfo::tCommMessage::tCommMessage( void ) :
	fMessageMode( kShipMode ),
	fSourceTeamID( kNoTeam ),
	fTeamID( kNoTeam ),
	fShipID( -1 ),
	fButtonNameID( -1 ),
	fMessageIDStart( -1 ),
	fMessageIDEnd( -1 ),
	fNextState( -1 ),
	fMsgType( kMissionScriptMessage )
{}

/*
	Purpose:	Constructor.

	Inputs:		( int32 )				-	The ID associated with the next comm state in the chain
				( int32 )				-	The ID associated with the team that owns the comm state
				( int32 )				-	The ID associated with the ship that owns the comm state
				( int32 )				-	The index into the array of messages that is the text on the button
				( int32 )				-	The index into the array of messages that is the message for the button
				( int32 )				-	The index into the array of messages used for randomizing
				( eScriptMessageTypes )	-	One of several constants that describe where the text should
											be displayed when the comm button is pressed
	Outputs:	N/A
*/
tTeamInfo::tCommMessage::tCommMessage(	int32				NextState,
										int32				SourceTeamID,
										int32				ShipID,
										int32				ButtonNameID,
										int32				MessageIDStart,
										int32				MessageIDEnd,
										eScriptMessageTypes	MsgType ) :
	fNextState( NextState ),
	fMessageMode( kShipMode ),
	fSourceTeamID( SourceTeamID ),
	fShipID( ShipID ),
	fTeamID( kNoTeam ),
	fButtonNameID( ButtonNameID ),
	fMessageIDStart( MessageIDStart ),
	fMessageIDEnd( MessageIDEnd ),
	fMsgType( MsgType )
{}

tClassAllocator<tTeamInfo> gTeamInfoAllocator;

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tTeamInfo::tTeamInfo( void ) :
	fTeam( kNoTeam ),
	fRace( kNoRaceType ),
	fCombatBPV( 0 ),
	fTeamLeaving( kFalse ),
	fNextCallbackID( -1 ),
	fTeamSetup( kFalse ),
	fShipClassType( NULL ),
	fVictoryCondition( NULL ),
	fVictoryLevel( kDraw ),
	fStatus( kGameInProgress ),
	fCurrentlyControlledShip( NULL ),
	fTag( kNoTeamTag ),
	fHasEndedHost( kFalse ),
	fHasEndedClient( kFalse )
{
	//
	//	setup AI info
	//
	int32 i = kMaxTeams;
	while( i-- )
	{
		fAITargets.fRelationships[i] = kUnknownRelationship;
		fAITargets.fTargetGoals[i] = kGoalIgnore;
		fAITargets.fEPVFactors[i] = 1.0;
	}

	//
	//	Setup comm state info.
	//
	for ( int32 j = 0; j < kMaxCommObjects; j++ )
	{
		fIDField[j] = 0;
		fCurCommState[j] = -1;
	}
}

/*
	Purpose:	Initialization for a team once certain information is known that was not
				known during its constructor.

	Inputs:		( eTeamID )		-	The ID associated with the team in question
				( eRaceName )	-	One of several constants that describe the race
									of the team in question
				( iTruth )		-	kTrue if this team is under human control;
									kFalse, otherwise
				( eTeamTag )	-	For multiplayer games, this can help determine the
									relationship between various teams
	Outputs:	N/A
*/
void
tTeamInfo::mSetTeamInfoData(	eTeamID		Team,
								eRaceName	Race,
								iTruth		HumanControlled,
								eTeamTag	Tag )
{
	//
	//	What does it mean if this asserts?
	//
	Assert( fTeamSetup == kFalse );

	fTeamSetup = kFalse;
	fTeam = Team;
	fRace = Race;
	fNextCallbackID =( Team * (0x7fffffff / kMaxTeams) ) + 1;
	fHumanControlled = HumanControlled;
	fTag = Tag;
	
	fAITargets.fRelationships[Team] = kTeamSelfAware;
}

/*
	Purpose:	Not used.

	Inputs:		
	Outputs:	
*/
void
tTeamInfo::mSetTeamFlag( int32 Flag )
{
	fFlag = Flag;
}

/*
	Purpose:	Not used

	Inputs:		
	Outputs:	
*/
int32
tTeamInfo::mGetTeamFlag( void )
{
	return fFlag;
}

/*
	Purpose:	Once all traits are known about a team, its ships are created, etc...it is
				time to inform the script that the team is ready to go.  As a result, the
				this is the place where last minute changes can be made.  Please note that
				if this function is overridden in a script, then you must call this function
				first, before any other code.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mTeamSetupComplete( void )
{
	//
	//	Register the team
	//
	gEventMgr.mRegisterActor( this );							

	//
	//	Then register the ships
	//
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		gEventMgr.mRegisterActor( *ship );

	//
	//	Setup the ships first
	//
	for( ship = mGetFirstShip(); *ship != NULL; ++ship )
		ship->mSetupShip();
	
	fTeamSetup = kTrue;	
}

/*
	Purpose:	Destructor

	Inputs:		N/A
	Outputs:	N/A
*/
tTeamInfo::~tTeamInfo( void )
{
	while( fShipList.size() )
	{
		tShipInfo* Ship = fShipList.back();
		delete Ship;
		Ship = NULL;
		fShipList.pop_back();
	}
	fShipList.erase(fShipList.begin(), fShipList.end());
}

/*
	Purpose:	Tells a team which victory condition object to append itself to.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mSetupVictoryCondition( void )
{}

/*
	Purpose:	Initialization for a team state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mSetupState( void )
{}

/*
	Purpose:	Initialization for team comm states.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mSetupCommState( void )
{}

/*
	Purpose:	Initialization for ship creation.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mCreateShipsForTeam( void )
{}

/*
	Purpose:	Each team can have its own debriefing list based on its
				level of victory.

	Inputs:		( eVictoryLevels )	-	Can be one of several constants such as "kVictory"
										that describe the team's level of victory.
	Outputs:	N/A
*/
void
tTeamInfo::mSetupDebriefingList( eVictoryLevels VictoryLevel )
{}

/*
	Purpose:	When a ship is chosen by a player through the engine, this is
				where that ships description can be captured and modified to
				the script's liking.

	Inputs:		( tShipScriptDescription& )	-	Constant reference of a ship that was
												created through the engine
	Outputs:	N/A
*/
void
tTeamInfo::mCreateNewShip( const tShipScriptDescription& ShipScriptDescription )
{
	mCreateShip( typeid( tShipInfo ), 
				 ShipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G + ShipScriptDescription.fTeamID,
				 ShipScriptDescription.fMetaDatabaseID,
				 -1, 0, 0, -1, -1,
				 ShipScriptDescription.fCustomName.c_str(),
				 kDefaultShipOptions );
}

/*
	Purpose:	Not used.

	Inputs:		
	Outputs:	
*/
void
tTeamInfo::mCreateNewShip( void )
{}

/*
	Purpose:	Accessor for the ID associated with a team.

	Inputs:		N/A
	Outputs:	( eTeamID )	-	The ID associated with the team in question
*/
eTeamID
tTeamInfo::mGetTeam( void )
{
	return fTeam;
}

/*
	Purpose:	Accessor for the tag associated with a team.

	Inputs:		N/A
	Outputs:	( eTeamTag )	-	The tag associated with the team in question
*/
eTeamTag
tTeamInfo::mGetTag( void )
{
	return fTag;
}

/*
	Purpose:	Evaluates whether the setup for a team is completed.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the team has been setup; kFalse, otherwise
*/
iTruth
tTeamInfo::mIsTeamSetup( void )
{
	return fTeamSetup;
}

/*
	Purpose:	Evaluates whether a human owns a team.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the team in question is controlled by a
								human; kFalse, otherwise
*/
iTruth
tTeamInfo::mIsTeamHumanControlled( void )
{
	return fHumanControlled>0;
}

/*
	Purpose:	Accessor for the race associated with a team.

	Inputs:		N/A
	Outputs:	( eRaceName )	-	One of several constants that describes which
									race is associated with the team in question
*/
eRaceName
tTeamInfo::mGetRace( void )
{
	return fRace;
}

/*
	Purpose:	Accessor for the head of the ship list for a team.

	Inputs:		N/A
	Outputs:	( tShipIterator )	-	The head of the ship list if it exist;
										otherwise, dereferencing the return value
										will produce NULL if no ships exist
*/
tShipIterator
tTeamInfo::mGetFirstShip( void )
{
	return tShipIterator( fShipList.begin(), fShipList.end() );
}

/*
	Purpose:	Accessor for a particular ship on a team.

	Inputs:		( int32 )		-	The index requested in the list of ship on the team
	Outputs:	( tShipInfo* )	-	The ship requested if it exists; otherwise, NULL
*/
tShipInfo*
tTeamInfo::mGetThisShip( int32 ShipIndex )
{
	if( ShipIndex > fShipList.size () )
		return NULL;

	ShipIndex--;
	if( ShipIndex < 0 )
		ShipIndex = 0;

	return( fShipList[ShipIndex] );
}

/*
	Purpose:	Accessor to a ship on the team that matches the given index.

	Inputs:		( int32 )		-	Index into the list of ships that will match
									the callback ID of the requested ship
	Outputs:	( tShipInfo* )	-	The ship requested if it exists; otherwise, NULL
*/
tShipInfo*
tTeamInfo::mFindShipIndex( int32 CallbackID )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		if( ship->mGetCallbackID() == CallbackID )
			return *ship;

	return NULL;
}

/*
	Purpose:	Accessor to a ship on the team that matches the given ID.

	Inputs:		( int32 )		-	The ID of the requested ship
	Outputs:	( tShipInfo* )	-	The ship requested if it exists; otherwise, NULL
*/
tShipInfo*
tTeamInfo::mFindShipID( int32 ShipID )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		if( ship->mGetShipID() == ShipID )
			return *ship;

	return NULL;
}

/*
	Purpose:	Accessor to a ship on the team that matches the given custom ID.

	Inputs:		( int32 )		-	The custom ID of the requested ship
	Outputs:	( tShipInfo* )	-	The ship requested if it exists; otherwise, NULL
*/
tShipInfo*
tTeamInfo::mFindShipCustomID( int32 CustomID )
{
	for ( tShipIterator ship = mGetFirstShip(); *ship != NULL; ship++ )
		if ( ship->mGetCustomID() == CustomID )
			return *ship;

	return NULL;
}

/*
	Purpose:	This is the place where a ship is actually added to a team.

	Inputs:		( tCreateShipInfo& )	-	Reference of a structure that contains info on
											the ship to be added to the team in question
	Outputs:	( iTruth )				-	kTrue if the ship was successfully added;
											kFalse, otherwise
*/
iTruth
tTeamInfo::mAddShipToTeamList( tCreateShipInfo& Info )
{
	//
	//	call back into the game to create the ship
	//
	gScriptInterface.mCreateSpecShip( Info );

	if( Info.fSelectedClassType == kNoClassType )
		return kFalse;

	//
	//	add this ship to the teams list if it was created
	//
	tShipInfo* ship = NULL;
	if( Info.fShipData.fShipInfoClassID == kNoAllocatorID )
		ship = new tShipInfo();
	else
		ship = dynamic_cast<tShipInfo*>( tDynamicAllocator::Instance()->mAllocateClass( Info.fShipData.fShipInfoClassID ) );

	//
	//	If this assert fails, it means you did not define an allocator for your ship, in each ship you
	//	define you must add tClassAllocator<tFillInTheShipName> gFillInTheShipNameAllocator; to the
	//	shipsource file ; OR that you passed in an invalid type_info, such as a team class typeid
	//
	Assert( ship );

	if( !ship )
		return kFalse;

	ship->mSetupShipInfo(	fTeam, fRace, Info.fShipData.fScriptCallbackID, Info.fShipData.mGetShipPosition(), Info.fShipData.fCustomID, Info.fCustomName,
							Info.fShipData.fStartX, Info.fShipData.fStartY, Info.fShipData.fEndX, Info.fShipData.fEndY, Info.fShipData.fShipOptions );
	fShipList.push_back( ship );
	ship->mSetClassType( Info.fSelectedClassType );

	ship->mSetShipBPV( Info.fShipData.fShipBPV );
	mAddToCombatBPV( Info.fShipData.fShipBPV );

	return kTrue;
}

/*
	Purpose:	This is the palce where a ship is actually removed from a team.

	Inputs:		( tShipInfo* )	-	A description of the ship to removed from a team
	Outputs:	N/A
*/
void
tTeamInfo::mRemoveShipFromTeamList( tShipInfo* Ship )
{
	tShipContainer::iterator foundShip;

	//
	//	Go through the ship list to find the ship that matches the above parameter.
	//
	for ( tShipContainer::iterator i = fShipList.begin(); i != fShipList.end(); i++ )
	{
		if ( Ship->mGetActorID() == ( *i )->mGetActorID() )
		{
			foundShip = i;
			break;
		}
	}

	//
	//	Now remove that ship from the list.
	//
	gEventMgr.mRemoveActor( ( *foundShip )->mGetActorID() );
	fShipList.erase( foundShip );
}

/*
	Purpose:	Sets up ship information before actually telling the ship
				to be added to a particular team.

	Inputs:		( type_info& )			-	Constant reference of a "typeid()"
				( tCreateShipInfo& )	-	A structure that contains the trait info
											on the ship to be created
				( iMapPosition )		-	One of several constants that describe where
											the ship is to be created
				( int32 )				-	The custom ID, if any, assigned to the ship
				( int32 )				-	X-coordinate of the start position of the ship
				( int32 )				-	Y-coordinate of the start position of the ship
				( int32 )				-	X-coordinate of the end position of the ship
				( int32 )				-	Y-coordinate of the end position of the ship
				( eRace )				-	The race of this ship if not supplied by the script
	Outputs:	N/A
*/
void
tTeamInfo::mCreateTeamShip(	const type_info&	ShipClass,
							tCreateShipInfo&	Info,
							iMapPosition		Position,
							int32				CustomID,
							int32				StartX,
							int32				StartY,
							int32				EndX,
							int32				EndY,
							eRace				Race )
{
	if( Position == kStartPosition_Specified )		  
	{
		if( StartX < 1 )
			StartX = 1;
		if( StartY < 1 )
			StartY = 1;
		if( EndX < 1 )
			EndX = StartX;
		if( EndY < 1 )
			EndY = StartY;
	}

	
	Info.fShipRace = ( ( Race == kNoRace ) ? fRace : Race );
	Info.fShipData.fTeam = fTeam;
	Info.fShipData.mSetShipPosition( Position );
	Info.fShipData.fStartX = StartX - 1;
	Info.fShipData.fStartY = StartY - 1;
	Info.fShipData.fEndX = EndX - 1;
	Info.fShipData.fEndY = EndY - 1;
	Info.fShipData.fScriptCallbackID = fNextCallbackID;
	Info.fShipData.fCreateShadowShip = kFalse;
	Info.fShipData.fCustomID = CustomID;
	Info.fShipData.fShipInfoClassID = tDynamicAllocator::Instance()->mGetAllocatorID( ShipClass );
	Info.fShipData.fHumanControlled = fHumanControlled;

	//
	//	Figure out the starting offset positioning of the ship.
	//
	if ( Position != kStartPosition_Specified )
		mDetermineStartPositionOffset( &( Info.fShipData.fShipOptions ), Position );

	if( mAddShipToTeamList( Info ) )
		fNextCallbackID++;
}

/*
	Purpose:	Will determine the offset of a ship starting position on this team.

	Inputs:		( eShipOptions* )	-	The memory that will contain the new offset data
				( iMapPosition )	-	The position of the ship coming in
	Outputs:	N/A
*/
void
tTeamInfo::mDetermineStartPositionOffset(	eShipOptions*	Options,
											iMapPosition	Position )
{
	int32	intOptions = static_cast< int32 >( *Options );
	if ( ( intOptions & kStartPositionCanBeOffset ) && !fTeamSetup )
	{
		if ( intOptions & kStartPositionOffsetMask )
		{
			;	//	everything is fine -- the scripter explicitly told the ship to appear here
		}
		else
		{
			//
			//	Try the origin first.  If a ship on this team is taking that up, then find a safe
			//	position for this ship.  If none, then start in the origin.
			//
			iTruth	originClear = kTrue;
			for ( tShipIterator ship = mGetFirstShip(); *ship != NULL; ship++ )
			{
				if ( ( ship->mGetShipOptions() & kStartPositionOffsetMask ) == kStartPositionOffsetOrigin )
				{
					if ( ship->mGetStartPositionIndex() == Position )
					{
						originClear = kFalse;
					}
				}
			}
			if ( originClear )
			{
				intOptions |= kStartPositionOffsetOrigin;
			}
			else
			{
				intOptions &= ~kStartPositionOffsetMask;
				int32	newDirection = kStartPositionOffsetE;
				iTruth	isGoodDirection = kTrue;
				while ( kTrue )
				{
					isGoodDirection = kTrue;
					for ( tShipIterator ship = mGetFirstShip(); *ship != NULL; ship++ )
					{
						int32 thisShipDirection = ship->mGetShipOptions() & kStartPositionOffsetMask;
						if ( thisShipDirection == newDirection )
						{
							if ( ship->mGetStartPositionIndex() == Position )
							{
								isGoodDirection = kFalse;
							}
						}
					}
					if ( isGoodDirection )
						break;
					else
					{
						if ( newDirection == kStartPositionOffsetOrigin )
							break;
						newDirection >>= kStartPositionBitAlignment;
						newDirection++;
						newDirection %= kNumStartPositionOffsets;
						newDirection <<= kStartPositionBitAlignment;
					}
				}
				intOptions = isGoodDirection ? intOptions | newDirection : intOptions | kStartPositionOffsetOrigin;
			}
		}
	}
	else
	{
		intOptions |= kStartPositionOffsetOrigin;
	}

	*Options = static_cast< eShipOptions >( ( static_cast< int32 >( *Options ) ) | intOptions );
}

/*
	Purpose:	Creates a ship from a structure that describes the ship to
				be created.

	Inputs:		( type_info& )	-	Constant reference of a "typeid()"
				( tShipInGame )	-	Constant reference of a structure that 
									contains info on the ship to be created
	Outputs:	N/A
*/
void tTeamInfo::mCreateShip(	const type_info&	ShipClass,
								const tShipInGame&	ShipData)
{

	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ShipData.fClassName;
	info.fShipData.fCustomName = ShipData.fCustomName;
	info.fShipData.fShipOptions = ShipData.fShipOptions;
	info.fShipData.fMetaDatabaseID = ShipData.fMetaDatabaseID;
	//
	//	You just created one ship, you can't group around
	//
	iMapPosition mapPos = ShipData.mGetShipPosition();
	mCreateTeamShip( ShipClass, info, mapPos, ShipData.fCustomID, ShipData.fStartX, ShipData.fStartY, ShipData.fEndX, ShipData.fEndY );
}

/*
	Purpose:	Creates a ship based on the given ship classes and percentages.

	Inputs:		( type_info& )		-	Constant reference of a "typeid()"
				( int32 )			-	The minimum BPV of the ship
				( eClassTypes )		-	One of several constants that describe the
										type of ship to be created
				( int32 )			-	Range from 0 to 100; higher the number, the
										more chance that the type of ship chosen will
										be a variant of the default ship
				( iMapPosition )	-	One of several constants that describe where
										the ship will spawn
				( int32 )			-	The custom ID assigned to the ship, if any
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
				( eRace )			-	Race of the ship if specified by the script
	Outputs:	N/A
*/
void
tTeamInfo::mCreateShip( const type_info&	ShipClass,
						int32				TotalBPV,
						eClassTypes			ShipType,
						int32				VariantPercent,
						iMapPosition		Position,
						int32				CustomID,
						int32				StartX,
						int32				StartY,
						int32				EndX,
						int32				EndY,
						const char*			CustomName,
						eShipOptions		ShipOptions,
						eRace				Race )
{
	mCreateFleet( ShipClass, 1, TotalBPV, ShipType, ShipType, VariantPercent, 
				  Position, CustomID, StartX, StartY, EndX, EndY, CustomName, ShipOptions, Race );
}
/*
	Purpose:	Creates a ship based on the range of ship classes and percentages.

	Inputs:		( type_info& )		-	Constant reference of a "typeid()"
				( int32 )			-	The minimum BPV of the ship
				( eClassTypes )		-	The low end of the valid class types
				( eClassTypes )		-	The high end of the valid class types
				( int32 )			-	Range from 0 to 100; highter the number, the
										more chance that the type of ship chosen will
										be a variant of the default ship
				( iMapPosition )	-	One of several constants that describe where
										the ship will spawn
				( int32 )			-	The custom ID assigned to the ship, if any
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
				( int32 )			-	ID of the ship passed from Meta engine				
	Outputs:	N/A
*/
void
tTeamInfo::mCreateShip( const type_info&	ShipClass,
						int32				TotalBPV,
						eClassTypes			SmallestHullSize,
						eClassTypes			LargestHullSize,
						int32				VariantPercent,
						iMapPosition		Position,
						int32				CustomID,
						int32				StartX,
						int32				StartY,
						int32				EndX,
						int32				EndY,
						const char*			CustomName,
						eShipOptions		ShipOptions )
{
	mCreateFleet( ShipClass, 1, TotalBPV, SmallestHullSize, LargestHullSize, VariantPercent,
				  Position, CustomID, StartX, StartY, EndX, EndY, CustomName, ShipOptions );
}


/*
	Purpose:	Creates a ship based on the name in the spec file.

	Inputs:		( type_info& )		-	Constant reference of a "typeid()"
				( char* )			-	Constant pointer to a list of characters that makes
										up the name of the primary key in the spec file
				( iMapPosition )	-	One of several constants the describe where the ship
										will spawn
				( int32 )			-	The custom ID assigned to the ship, if any
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
				( int32 )			-	ID of the ship passed from Meta engine
				
	Outputs:	N/A
*/
void
tTeamInfo::mCreateShip( const type_info&	ShipClass,
						const char*			ClassName,
						iMapPosition		Position,
						int32				MetaDatabaseID,
						int32				CustomID,
						int32				StartX,
						int32				StartY,
						int32				EndX,
						int32				EndY,
						const char*			CustomName,
						eShipOptions		ShipOptions )
{
	tCreateShipInfo info;

	info.fShipData.fClassType = kNoClassType;
	info.fShipData.fClassName = ClassName;
	info.fShipData.fCustomName = CustomName;
	info.fShipData.fShipOptions = ShipOptions;
	info.fShipData.fMetaDatabaseID = MetaDatabaseID;
	mCreateTeamShip( ShipClass, info, Position, CustomID, StartX, StartY, EndX, EndY );
}

/*
	Purpose:	Creates a variable number of variable types of ships.

	Inputs:		( type_info& )		-	Constant reference of a "typeid()"
				( int32 )			-	The total number of ships requested
				( int32 )			-	The minimum BPV of the ship
				( eClassTypes )		-	The low end of the valid class types
				( eClassTypes )		-	The high end of the valid class types
				( int32 )			-	Range from 0 to 100; highter the number, the
										more chance that the type of ship chosen will
										be a variant of the default ship
				( iMapPosition )	-	One of several constants that describe where
										the ship will spawn
				( int32 )			-	The custom ID assigned to the ship, if any
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
				( eRace )			-	Race of the ship if not specified by the script
	Outputs:	N/A
*/
void
tTeamInfo::mCreateFleet( const type_info&	ShipClass,
						 int32				TotalShips,
						 int32				TotalBPV,
						 eClassTypes		SmallestHullSize,
						 eClassTypes		LargestHullSize,
						 int32				VariantPercent,
						 int32				Position,
						 int32				CustomID,
						 int32				StartX,
						 int32				StartY,
						 int32				EndX,
						 int32				EndY,
						 const char*		CustomName,						 
						 eShipOptions		ShipOptions,
						 eRace				Race )
{
	Assert( TotalShips < kMaxShips );
	if ( TotalShips >= kMaxShips )
		return;

	tCreateShipInfo info[kMaxShips];

	for( int32 i = 0; i < TotalShips; i++ )
	{
		info[i].fShipData.fClassType = LargestHullSize;
		info[i].fTotalBPV = TotalBPV;
		info[i].fVariant = VariantPercent;
		info[i].fClassTypeLast = SmallestHullSize;
		info[i].fShipData.fCustomName = CustomName;
		info[i].fShipData.fShipOptions = ShipOptions;
		info[i].fShipData.fMetaDatabaseID = 0;
	}
	
	mCreateTeamFleet( ShipClass, TotalShips, info, Position, CustomID, StartX, StartY, EndX, EndY, NULL, kDefaultShipOptions, Race );
}

/*
	Purpose:	Creates a variable number of ship based on a structure containing
				the traits of these ships.

	Inputs:		( type_info& )		-	Constant reference of a "typeid()"
				( int32 )			-	The total number of ships requested
				( tCreateShipInfo )	-	List of the traits associated with the ships
				( iMapPosition )	-	One of several constants that describe where
										the ship will spawn
				( int32 )			-	The custom ID assigned to the ship, if any
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
				( eRace )			-	Race of the ship if not specified by the script
	Outputs:	N/A
*/
void
tTeamInfo::mCreateTeamFleet(	const type_info&	ShipClass,
								int32				ShipCount,
								tCreateShipInfo		Info[], 
								iMapPosition		Position,
								int32				CustomID,
								int32				StartX, 
								int32				StartY, 
								int32				EndX,
								int32				EndY,
								const char*			CustomName,
								eShipOptions		ShipOptions,
								eRace				Race )
{
	if( Position != kStartPosition_Specified )
	{
		unsigned char MapObject = 'G' + Position - kStartPosition_G;
		gScriptInterface.mFindMapObject( MapObject, &StartX, &StartY );
		MapObject += 'g' - 'G';
		gScriptInterface.mFindMapObject( MapObject, &EndX, &EndY );
	}

	int32	sX = StartX;
	int32	sY = StartY;
	int32	eX = EndX;
	int32	eY = EndY;
	int32	Edge = 0;		// 0=top, 1=right, 2=bottom, 3=left
	int32	EdgeStep = 2;
	int32	EdgeCount = 0;
	int32	Xstep;
	int32	Ystep;

	for( int32 i = 0; i < ShipCount; i++ )
	{
		mCreateTeamShip( ShipClass, Info[i], Position, CustomID, sX, sY, eX, eY, Race );

		//
		//	test to step the next start position out in a spiral pattern
		//
		if( Position == kStartPosition_Specified )
		{
			//
			//	test to start a new edge
			//
			if( EdgeCount == 0 )
			{
				//
				//	test to reset edge back to top edge and lengthen edge step
				//
				if( Edge > 3 )
				{
					Edge = 0;				// start top edge
					EdgeStep += 2;
				}
				switch( Edge )
				{
					case 0:					// starting top edge
					case 3:					// starting left edge
						Xstep = 0;
						Ystep = -1;
						break;
					case 1:					// starting right edge
						Xstep = 0;
						Ystep = 1;
						break;
					case 2:					// starting bottom edge
						Xstep = -1;
						Ystep = 0;
						break;
				}
				EdgeCount = EdgeStep;
			}

			//
			//	move to new position
			//
			sX += Xstep;
			sY += Ystep;
			eX += Xstep;
			eY += Ystep;

			//
			//	setup for next edge
			//
			if( EdgeCount == EdgeStep )
			{
				if( Edge == 0 )
				{
					Xstep = 1;
					Ystep = 0;
				}
				Edge++;
			}
			EdgeCount--;
		}
		else
		{
			Position++;
		}
	}
}

/*
	Purpose:	Creates a ship based on the traits of another ship.

	Inputs:		( tShipInfo* )	-	The ship to be duplicated
				( iMapPosition )	-	One of several constants that describe where
										the ship will spawn
				( int32 )			-	X-coordinate of the start position
				( int32 )			-	Y-coordinate of the start position
				( int32 )			-	X-coordinate of the end position
				( int32 )			-	Y-coordinate of the end position
				( char* )			-	Constant pointer to a list of characters that
										makes up the name of the ship, if any
				( iTruth )			-	kTrue means that the ship can be added to another
										team's fleet upon capture; kFalse, otherwise
	Outputs:	N/A
*/
void
tTeamInfo::mDuplicateShip(	tShipInfo*		Ship, 
							iMapPosition	Position, 
							int32			StartX, 
							int32			StartY, 
							int32			EndX, 
							int32			EndY,
							const char*		CustomName,
							eShipOptions	ShipOptions )
{
	if( Ship )
	{
		tCreateShipInfo info;

		info.fShipData.fClassType = kNoClassType;
		info.fShipData.fClassName = Ship->mGetClassName();
		info.fShipData.fCustomName = NULL;
		info.fShipData.fShipOptions = ShipOptions;
		mCreateTeamShip( typeid( *Ship ), info, Position, info.fShipData.fCustomID, StartX, StartY, EndX, EndY );
	}
}

//------------------------------------------------------------------------------------------------
// returns number of ships in play -- seems to not work with current iterator >> use mShipsRemaining()
/*
	Purpose:	

	Inputs:		
	Outputs:	
*/
int32
tTeamInfo::mGetShipsInPlayCount( void )
{
	tShipIterator ship = mGetFirstShip();
	tShipInPlayEval inPlayEval;
	ship.mAddEvaluator( &inPlayEval );

	if( *ship == NULL )
		return -1;			// no ships created yet for this team

	int32 count = 0;
	for( ; *ship != NULL; ++ship )
		count++;

	return count;
}

/*
	Purpose:	Evaluates if all ships on a team have been created.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if all ships have been initialized;
								kFalse, otherwise
*/
iTruth
tTeamInfo::mAllShipsReady( void )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		if( ship->mTestShipStatus (kShipCreated) == kFalse )
			return kFalse;

	return kTrue;
}

/*
	Purpose:	Evaluates if all ships on a team are in their positions.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if all ships are in position;
								kFalse, otherwise
*/
iTruth
tTeamInfo::mAllInPosition( void )
{
	tShipIterator ship = mGetFirstShip();
	tShipInPlayEval inPlayEval;
	ship.mAddEvaluator( &inPlayEval );
	
	for( ; *ship != NULL; ++ship )
		if( ship->mGetShipCallbackEvents() & kNotifyAtDestination )
			return kFalse;

	return kTrue;
}

/*
	Purpose:	Will tell all ships on a team to disengage.

	Inputs:		( eDisengageMethods )	-	One of several constants that describe how
											the ships should go about disengaging
				( eAIPriority )			-	One of several constants that tells the AI how
											likely these ships will actually carry out
											this order
	Outputs:	N/A
*/
void
tTeamInfo::mDisengageShips(	eDisengageMethods	Method,
							eAIPriority			Priority )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		ship->mDisengage(Method, Priority);
}

/*
	Purpose:	Will tell all ships to defend another ship.

	Inputs:		( tShipInfo* )	-	The ship that is to be defended
				( float )		-	The distance from the defendee that an enemy has
									to come within for a pursuit to ensue
				( float )		-	The distance at which the defender will follow
									the defendee
				( eAIPriority )	-	One of several constants that tells the AI how
									likely these ships will actually defend
	Outputs:	N/A
*/
void
tTeamInfo::mDefendEntity(	tShipInfo*	Entity,
							float		ThreatRadius,
							float		FollowRadius,
							eAIPriority	Priority )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		ship->mDefendEntity( Entity, ThreatRadius, FollowRadius, Priority );
}

/*
	Purpose:	Removes any AI commands specified through the script to all ships
				on a team.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mClearCommands( void )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		ship->mClearCommands();
}

/*
	Purpose:	Accessor for the BPV of the team.

	Inputs:		N/A
	Outputs:	( int32 )	-	The total BPV of all ships on a team
*/
int32
tTeamInfo::mGetCombatBPV( void )
{
	return fCombatBPV;
}

/*
	Purpose:	Accessor for the BPV of the team with loadout.

	Inputs:		N/A
	Outputs:	( int32 )	-	The total BPV of all ships on a team with loadout.
*/
int32
tTeamInfo::mGetTotalCombatBPV( void )
{
	int32 totalBPV = 0;		//	UNUSED!!!

	for ( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		totalBPV += ship->mGetTotalBPV();

	return totalBPV;
}
/*
	Purpose:	Modifier for the BPV of the team.

	Inputs:		( int32 )	-	Amount to modify the BPV of the team
	Outputs:	N/A
*/
void
tTeamInfo::mAddToCombatBPV( int32 ShipBPV )
{
	fCombatBPV += ShipBPV;
}

/*
	Purpose:	Determines what BPV has been done as a result of damage from
				another team.

	Inputs:		( tTeamInfo* )	-	The enemy team in question
	Outputs:	( int32 )		-	The determined damage BPV
*/
int32
tTeamInfo::mAssessDamages( tTeamInfo* EnemyTeam )
{
	if( EnemyTeam )
	{
		eRaceName EnemyRace = EnemyTeam->mGetRace();
		float EPVFactor = EnemyTeam->mGetEPV( fTeam );

		if( EnemyTeam->mIsTeamLeaving() )
			return 0;
		if( mIsTeamLeaving() )
			return mGetCombatBPV() * EPVFactor;

		int32 DamageBPV = 0;
		for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
			DamageBPV += ship->mAssessDamage( EnemyRace ) * EPVFactor;

		return DamageBPV;
	}
	return 0;
}

/*
	Purpose:	Removes a ship as a primary target from all ships on a team.

	Inputs:		( tShipInfo* )	-	The ship to remove as a target
	Outputs:	N/A
*/
void
tTeamInfo::mRemoveShipAsTarget( tShipInfo* TargetShip )
{
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		ship->mSetShipTarget( TargetShip, kNeverPriority );
}

/*
	Purpose:	Will determine why a team is no longer in play.

	Inputs:		N/A
	Outputs:	( iNotifyEvents )	-	One of several bits that will describe what
										has happened to the team
*/
iNotifyEvents
tTeamInfo::mGetTeamStatus( void )
{
	if( mIsTeamLeaving() )
		return kNotifyPlayerLeaving;

	int32 disengaged_count = 0;
	int32 captured_count = 0;
	
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
	{
		if( ship->mTestShipStatus( kShipDisengaged ) )
			disengaged_count++;
		else if( ship->mTestShipStatus( kShipCaptured ) )
			captured_count++;
	}

	if( disengaged_count )
		return kNotifyDisengaged;
	else if( captured_count )
		return kNotifyOnCapture;

	return kNotifyAtDeath;
}

/*
	Purpose:	Sets this teams relations to another team.

	Inputs:		( eTeamID )				-	The ID associated with the team in question
				( eTeamRelationship )	-	One of several constants that describes possible
											relationships between teams
				( eTargetGoals )		-	One of several constants that describes possible
											goals between teams
				( float )				-	?
	Outputs:	N/A
*/
void
tTeamInfo::mSetRelations(	eTeamID				Team,
							eTeamRelationship	Relations,
							eTargetGoals		Goals,
							float				EPVFactor )
{
	fAITargets.fRelationships[Team] = Relations;
	fAITargets.fTargetGoals[Team] = Goals;
	fAITargets.fEPVFactors[Team] = EPVFactor;

	//
	//	let the game know of this change
	//
	tShipSystemsInfo info;

	info.fAction = kSetTeamRelation;
	info.fInt32[0] = Relations;
	info.fInt32[1] = fTeam;					// *this team
	info.fInt32[2] = Team;
	info.fInt32[3] = Goals;
	info.fFloat[0] = EPVFactor;

	gScriptInterface.mEntityMgr( info );
}

/*
	Purpose:	Accessor for the relationship between this team and another team.

	Inputs:		( eTeamID )				-	The ID associated with the team in question
	Outputs:	( eTeamRelationship )	-	One of several constants that describes possible
											relationships between teams
*/
eTeamRelationship
tTeamInfo::mGetRelations( eTeamID Team )
{
	if( Team > kNoTeam && Team < kMaxTeams )
		return( fAITargets.fRelationships[Team] );

	return kUnknownRelationship;
}

/*
	Purpose:	Accessor for the goals between this team and another team.

	Inputs:		( eTeamID )			-	The ID associated with the team in question
	Outputs:	( eTargetGoals )	-	One of several constants that describes possible
										goals between teams
*/
eTargetGoals
tTeamInfo::mGetGoal( eTeamID Team )

{
	if( Team > kNoTeam && Team < kMaxTeams )
		return( fAITargets.fTargetGoals[Team] );

	return kGoalIgnore;
}

/*
	Purpose:	Not used.

	Inputs:		
	Outputs:	
*/
float
tTeamInfo::mGetEPV( eTeamID Team )

{
	if( Team > kNoTeam && Team < kMaxTeams )
		return( fAITargets.fEPVFactors[Team] );

	return 0.0;
}

/*
	Purpose:	Accessor for the number of ships on a team dependant upon
				the ship's status.

	Inputs:		( eShipStatus )	-	One of several constants that describe the
									current status of a ship
	Outputs:	( int32 )		-	The number of ships on a team that met the
									status criteria
*/
int32
tTeamInfo::mGetShipCount( eShipStatus Status )
{
	int32 count = 0;
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
		if( ship->mTestShipStatus (Status) )
			count++;

	return count;
}

/*
	Purpose:	Determines the total amount of damage a team has received as a whole.

	Inputs:		N/A
	Outputs:	( float )	-	The total damage requested
*/
float
tTeamInfo::mGetTotalDamagePercent( void )
{
	int32 count = 0;
	float total_damage = 0.0;
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
	{
		count++;
		if( ship->mTestShipStatus (kShipCaptured) == kTrue )
			total_damage += 100.0;
		else
			total_damage += ship->mGetDamagePercent();
	}
	if( count )
		return total_damage / count;

	return 100.0;
}

/*
	Purpose:	Will inform the script that a team is leaving the mission.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tTeamInfo::mSetTeamLeaving( void )
{
	fTeamLeaving = kTrue;
	if( fStatus == kGameInProgress )
		mSetGameStatus( static_cast< eGameStatus >( kPlayerLoses | kLeftEarly ) );
}

/*
	Purpose:	Evaluates whether a team has left a mission or not.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the team has left; kFalse, otherwise
*/
iTruth
tTeamInfo::mIsTeamLeaving( void )
{
	return fTeamLeaving;
}

/*
	Purpose:	Determines how many ships are left in play on a team.  Those ships that have been
				captured, disengaged, or destroyed are not counted towards this total.

	Inputs:		N/A
	Outputs:	( int32 )	-	The number of ships in play requested
*/
int32
tTeamInfo::mShipsRemainingInPlay( void )
{
	eTeamID	thisTeamID = mGetTeam();
	int32	numShips = 0;

	for ( int32 i = kTeam1; i <= kTeam20; i++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( !team )
			continue;

		tShipIterator	ship = team->mGetFirstShip();
		tShipInPlayEval	x;

		ship.mAddEvaluator( &x );
		for ( ; *ship != NULL; ship++ )
		{
			if ( ship->mGetTeam() == thisTeamID )
				numShips++;
		}
	}

	return numShips;
}

/*
	Purpose:	Determines how many ships are left in play and under control on a team.  Only ships that
				originally started on this team will be counted.

	Inputs:		N/A
	Outputs:	( int32 )	-	The number of ships in play requested
*/
int32
tTeamInfo::mShipsRemainingInPlayFromOriginalTeam( void )
{
	eTeamID thisTeamID = mGetTeam();
	int32	numShips = 0;

	for ( int32 i = kTeam1; i <= kTeam20; i++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( !team )
			continue;

		tShipIterator	ship = team->mGetFirstShip();
		tShipInPlayEval	x;

		ship.mAddEvaluator( &x );
		for ( ; *ship != NULL; ship++ )
		{
			if ( ship->mGetOriginalTeam() == thisTeamID )
				numShips++;
		}
	}

	return numShips;
}

/*
	Purpose:	Determines how many ships are current under control by a certain team regardless of what
				a ship's original team was. (Looks like this does the same as mShipsRemainingInPlay)

	Inputs:		N/A
	Outputs:	( int32 )	-	The number of ships under this team's control
*/
int32
tTeamInfo::mShipsCurrentlyControlled( void )
{
	eTeamID	thisTeamID = mGetTeam();
	int32	numShips = 0;

	for ( int32 i = kTeam1; i <= kTeam20; i++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( !team )
			continue;

		tShipIterator	ship = team->mGetFirstShip();
		tShipInPlayEval	x;

		ship.mAddEvaluator( &x );
		for ( ; *ship != NULL; ship++ )
		{
			if ( ( ship->mGetTeam() == thisTeamID ) && ( !ship->mTestShipStatus( kShipDisengaged ) ) )
				numShips++;
		}
	}

	return numShips;
}

/*
	Purpose:	Determines the largest ship class type on a team.

	Inputs:		N/A
	Outputs:	( eClassTypes )	-	One of several constants that describe the type
									of ship
*/
eClassTypes
tTeamInfo::mGetMaxClassType( void )
{
	eClassTypes Max = kNoClassType;
	
	for( tShipIterator ship = mGetFirstShip(); *ship != NULL; ++ship )
	{
		eClassTypes type = ship->mGetClassType();
		if( type > Max )
			Max = type;
	}
	return Max;
}

/*

	Purpose:	Looks for the specified ID in the table.  If the ID is not found, then it will add the
				new ID into a slot in the table.  Regardless of which happens, the index into the table
				will be returned.  If the max number of entries in the table has been exhausted, -1 is
				returned instead as the index.

	Inputs:		( int32 )	-	The ID associated with the comm button in question
				( iTruth )	-	kTrue if it OK to write over previous data; kFalse, otherwise
	Outputs:	( int32 )	-	The index into the array, if valid; otherwise, -1
*/
int32
tTeamInfo::mFindCommIndex(	int32	ID,
							iTruth	Overwrite )
{
	//
	//	Try to find the shipID as being stored.
	//
	for ( int32 i = 0; i < kMaxCommObjects; i++ )
	{
		if ( !fIDField[i] )
			break;
		if ( fIDField[i] == ID )
			return i;
	}

	//
	//	If we got here, then the shipID was not found and, instead, should be added to the list.  But first,
	//	let's make certain we have not overstepped the bounds of the array.
	//
	if ( Overwrite )
	{
		if ( i < kMaxCommObjects )
		{
			fIDField[i] = ID;
			return i;
		}
	}

	//
	//	If we are down here, then the above must have failed and an "invalid" index into the array should
	//	be returned informing the user that it was both not found and room had run out in the array.
	//
	return -1;
}

/*
	Purpose:	Sets up a comm button on a ship.  By registering this button, we are informing the
				other actors that we want to get messages from this button.

	Inputs:		( int32 )				-	The customized state of the comm message
				( int32 )				-	If this button is pressed, what is the next state
				( int32 )				-	The ID associated with the ship in question
				( int32 )				-	The index in the list of messages for this button
				( int32 )				-	The index in the list of messages for the message on this button
				( int32 )				-	The high end in the index of messages for random messages
				( eScriptMessageTypes )	-	One of several constants that describe where the
											text for the message should appear on the screen
	Outputs:	N/A
*/
void
tTeamInfo::mRegisterCommMessage(	int32				StateID,
									int32				NextState,
									int32				ShipID, 
									int32				ButtonNameID,
									int32				MessageIDStart, 
									int32				MessageIDEnd,
									eScriptMessageTypes MsgType ) 
{
	Assert( StateID < kMaxCommStates );
	if( StateID >= kMaxCommStates )
		return;

	tCommMessage newCommMessage( NextState, mGetTeam(), ShipID, ButtonNameID, MessageIDStart, MessageIDEnd, MsgType );

	int32 shipIndex = mFindCommIndex( ShipID, kTrue );
	if ( shipIndex != -1 )
		fCommStates[StateID][shipIndex].push_back( newCommMessage );
}

/*

	Purpose:	The comm state now needs the ship or team id sent to it.  For this function to determine
				whether the id is of ship or team, pass "kCommTargetShip" or "kCommTargetTeam", then pass
				in the state id like always.

	Inputs:		( int32 )	-	The ID of the ship that owns the comm state in question
				( int32 )	-	The ID of the comm state that should be activated
				( int32 )	-	?
	Outputs:	N/A
*/
void
tTeamInfo::mGotoCommState(	int32	ID,
							int32	StateID,
							int32	ButtonID )
{
	Assert( StateID < kMaxCommStates );
	if( StateID >= kMaxCommStates )
		return;

	int32 index = mFindCommIndex( ID );

	if ( index == -1 )
		return;

	//
	//	Clear up old comm state
	//
	if( fCurCommState[index] > -1 )
	{
		for( tCommMsgList::iterator commMsg = fCommStates[fCurCommState[index]][index].begin();
			 commMsg != fCommStates[fCurCommState[index]][index].end();
			 commMsg++ )
		{
			tShipInfo* ship = fMissionInfo->mFindShipID( commMsg->fShipID );
			Assert( ship );
			if( ship )
				ship->mClearCommMsg( fTeam, commMsg->fButtonNameID );
		}

		//
		//	Tell the world that we our leaving this comm state
		//
		tEventMgr::tEventInfo eventInfo;
		eventInfo.fEvent = kNotifyCommStateChanged;
		eventInfo.fEventItem[0] = (int32)this;
		eventInfo.fEventItem[1] = fCurCommState[index];
		eventInfo.fEventItem[2] = tState::kEndingState;
		eventInfo.fEventItem[3] = ButtonID;
		gEventMgr.mPostEvent( eventInfo );
	}

	fCurCommState[index] = StateID;


	//
	//	Setup new state
	//
	for( tCommMsgList::iterator commMsg = fCommStates[fCurCommState[index]][index].begin();
		 commMsg != fCommStates[fCurCommState[index]][index].end();
		 commMsg++ )
	{
		tShipInfo* ship = fMissionInfo->mFindShipID( commMsg->fShipID );

		//
		//	If you hit this assert, you might be using the wrong register comm message function
		//	Make sure you want this comm message to be associated with a ship
		//
		Assert( ship );

		if( ship )
			ship->mSetCommButton( fTeam, commMsg->fButtonNameID );
	}

	//
	//	Tell the world that we are starting this comm state
	//
	tEventMgr::tEventInfo eventInfo;
	eventInfo.fEvent = kNotifyCommStateChanged;
	eventInfo.fEventItem[0] = (int32)this;
	eventInfo.fEventItem[1] = fCurCommState[index];
	eventInfo.fEventItem[2] = tState::kStartingNewState;
	eventInfo.fEventItem[3] = ButtonID;
	gEventMgr.mPostEvent( eventInfo );
}

/*
	Purpose:	

	Inputs:		
	Outputs:	
*/
int32
tTeamInfo::mGetCommState( int32 ID )
{
	int32 index = mFindCommIndex( ID );
	return fCurCommState[index];
}

/*
	Purpose:	This will be called first before any other events.  What this does is to
				take care of any comm button related items, then pass along the same
				information to the eventual actor.

	Inputs:		( tShipInfo* )	-	The ship that may have to do with the event at hand
				( eEventInfo* )	-	Constant reference of a structure that describes the
									event at hand
	Outputs:	N/A
*/
void
tTeamInfo::mHandleShipEvent(	tShipInfo*						Ship,
								const tEventMgr::tEventInfo&	EventInfo )
{
	//
	//	Only handle event if this team is still in the game
	//
	if( fStatus != kGameInProgress )
		return;

	int32 newState = -1;
	tTeamInfo* sourceTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( EventInfo.fEventItem[3] ) );

	int32 index = -1;
	if ( EventInfo.fEvent == kNotifyCommButtonPress )
		if ( mFindShipID( EventInfo.fEventItem[1] ) )
			index = sourceTeam->mFindCommIndex( EventInfo.fEventItem[1] );

	if ( index != -1 )
	{
		if( EventInfo.fEvent == kNotifyCommButtonPress && sourceTeam->fCurCommState[index] >= 0 )
		{
			//
			//	Go through ship messages
			//
			iTruth msgDisplayed = kFalse;

			for( tCommMsgList::iterator commMsg = sourceTeam->fCommStates[sourceTeam->fCurCommState[index]][index].begin();
				 commMsg != sourceTeam->fCommStates[sourceTeam->fCurCommState[index]][index].end();
				 commMsg++ )
			{
				if( ( commMsg->fButtonNameID == EventInfo.fEventItem[0] ) && ( commMsg->fShipID == EventInfo.fEventItem[1] ) )
				{
					msgDisplayed = kTrue;

					//
					//	If there is no message requested for display, then do not display because that will waste space in the on-screen buffer.
					//
					if ( commMsg->fMessageIDStart != kNoMsg )
					{
						if( commMsg->fMessageIDEnd == kNoRandomMsg )
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( commMsg->fSourceTeamID ), commMsg->fMessageIDStart, commMsg->fMsgType );
						else
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( commMsg->fSourceTeamID ), fMissionInfo->mRandomInt32( commMsg->fMessageIDStart, commMsg->fMessageIDEnd ), commMsg->fMsgType );
					}
					
					if( commMsg->fNextState >= 0 )
					{
						newState = commMsg->fNextState;
						break;
					}
				}
			}
		}
		else if( EventInfo.fEvent == kNotifyAtTimedInterval &&
				 EventInfo.fEventItem[1] != kInvalidStateID &&
				 EventInfo.fEventItem[1] != fCurState->mGetStateID() )
		{
			//
			//	A event meant for a particular state went off, but we are not in that state
			//
			::OutputDebugString( "A event meant for a particular state went off, but we are not in that state\n" );
			return;
		}
	}

	if( newState > -1 )
		sourceTeam->mGotoCommState( EventInfo.fEventItem[1], newState, EventInfo.fEventItem[0] );

	tActor::mHandleShipEvent( Ship, EventInfo );
}

/*
	Purpose:	Accessor for the current status of the game.

	Inputs:		N/A
	Outputs:	( eGameStatus )	-	One of several constants that describes the
									status of the game
*/
eGameStatus
tTeamInfo::mGetGameStatus( void )
{
	return fStatus;
}

/*
	Purpose:	Modifies the status of the a team.

	Inputs:		( eGameStatus )	-	One of several constants that describes the
									status of the game
	Outputs:	N/A
*/
void
tTeamInfo::mSetGameStatus( eGameStatus Status )
{
	fStatus = Status;
	fMissionInfo->fStatusArray[mGetTeam()] = Status;

	if( fStatus != kGameInProgress && fVictoryCondition )
	{
		if ( gScriptInterface.mIsHost() )
		{
			//
			//	Let the source know that the status of this team has changed and that the proper dialog should
			//	appear as a result.
			//
			fMissionInfo->mInvokeGameStatus( kFalse, fTeam, kFalse, gScriptInterface.mIsHost() );
			fHasEndedHost = kTrue;
			fHasEndedClient = kTrue;
		}
		else
		{
			fMissionInfo->mInvokeGameStatus( kFalse, fTeam, kFalse, gScriptInterface.mIsHost() );
			fHasEndedClient = kTrue;
		}
	}
}

/*
	Purpose:	Updates which ship is currently controlled by a human on a team.

	Inputs:		( tShipInfo* )	-	The newly controlled ship
	Outputs:	N/A
*/
void
tTeamInfo::mSetCurrentlyControlledShip( tShipInfo* Ship )
{
	fCurrentlyControlledShip = Ship;
}

/*
	Purpose:	Accessor for the currently controlled ship by a human on a team.

	Inputs:		N/A
	Outputs:	( tShipInfo* )	-	The ship requested
*/
tShipInfo*
tTeamInfo::mCurrentlyControlledShip( void )
{
	return fCurrentlyControlledShip;
}

/*

	Purpose:	Retrieves the info about the next scheduled mission.

	Inputs:		N/A
	Outputs:	( tMissionScheduler* )	-	The class that contains the info on the next mission in the queue

*/
tTeamInfo::tMissionScheduler*
tTeamInfo::mGetNextMissionInfo( void )
{
	return &fMissionScheduler;
}

/*

	Purpose:	Sets the next mission data.

	Inputs:		( tVictoryCondition* )	-	The info on this team's victory
	Outputs:	N/A

*/
void
tTeamInfo::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{}

//	4Nov99	zdrummond	Added Comment Header and Footer's
//	18Apr00	dsuleski	Overloaded mCreateNewShip(...) to be more like the old scripting interface
//	28Apr00	dsuleski	new mShipsRemaining() on team level counts ships in play
//	29Apr00	dsuleski	can now get the current commstate of a team
//	2May00	dsuleski	put back in old functionality into finding a ship on a team through its custom ID
//	26May00	dsuleski	mGetFirstShipInPlay() causes crash with local evaluator -- got rid of it
//	15Jun00	dsuleski	Added function to remove a ship from a team list
//	16Jun00	dsuleski	Added accessor for fCurrentlyControlledShip
//	3Jul00	dsuleski	Comm messages no longer need to display messages if not requested to do so
//	8Jul00	dsuleski	Documentation update
//	25Sep00	dsuleski	Created tMissionScheduler and appropriate helper functions within tTeamInfo