//	<Name>	VictoryCondition.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "sfcAssert.h"
#include "NormalVictoryState.h"
#include "ScriptInterface.h"


/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tVictoryCondition::tVictoryCondition( void ) :
	fTeam( NULL )
{}

/*
	Purpose:	Constructor that sets the team associated with it.

	Inputs:		( tTeamInfo* )	-	The team attached to this victory condition
	Outputs:	N/A
*/
tVictoryCondition::tVictoryCondition( tTeamInfo* Team ) :
	fTeam( Team ),
	fVictoryState( NULL )
{}

/*
	Purpose:	Constructor that sets a whole lot of various attributes for the script.

	Inputs:		( int32 )		-	The value given when an astounding victory is achieved
				( int32 )		-	The value given when a victory is achieved
				( int32 )		-	The value given when a draw is achieved
				( int32 )		-	The value given when a defeat is achieved
				( int32 )		-	The value given when a devastating defeat is achieved
				( int32 )		-	The value given for capturing an enemy (obsolete)
				( int32 )		-	The value given when forfeiting a mission
				( tTeamInfo* )	-	A pointer to the team structure
				( int32 )		-	ID associated with the player team (obsolete)
				( int32 )		-	ID associated with the enemy team (obsolete)
	Outputs:	N/A
*/
tVictoryCondition::tVictoryCondition(	int32		AstoundingVictory,
										int32		Victory,
										int32		Draw,
										int32		Defeat,
										int32		DevastatingDefeat,
										int32		CaptureBonus,
										int32		UnknownVictoryPoints,
										tTeamInfo*	Team,
										int32		PlayerTeamID,
										int32		EnemyTeamID ) :
	fTeam( Team ),
	fPlayerTeamID( PlayerTeamID ),
	fEnemyTeamID( EnemyTeamID ),
	fVictoryState( NULL ),
	fCaptureBouns( CaptureBonus ),     
	fAstoundingVictory( AstoundingVictory ),
	fVictory( Victory ),          
	fDraw( Draw ),             
	fDefeat( Defeat ),           
	fDevastatingDefeat( DevastatingDefeat ),
	fUnknownVictoryPoints( UnknownVictoryPoints )
{}

/*
	Purpose:	By registering a victory state, you are telling the API that the state in question
				should receive all messages from all actors from this point on.

	Inputs:		( tVictoryState* )	-	A state object with some special methods for victories
	Outputs:	N/A
*/
void
tVictoryCondition::mRegisterState( tVictoryState* State )
{
	tActor::mRegisterState( State );

	State->mSetTeam( fTeam );
}

/*
	Purpose:	This method tells an actor to catch messages in the state in question.

	Inputs:		( type_info& )	-	The state class in question used with "typeid()"
	Outputs:	N/A
*/
void
tVictoryCondition::mGotoState( const type_info& StateTypeID )
{
	tActor::mGotoState( StateTypeID );
	fVictoryState = dynamic_cast<tVictoryState*>( fCurState );

	Assert( fVictoryState );
}

/*
	Purpose:	Figures out the level of success for a particular team.

	Inputs:		( eTeamID )			-	The ID associated with the team in question
	Outputs:	( eVictoryLevels )	-	Can be kAstoundingVictory, kVictory, kDraw, kDefeat, or
				kDevastatingDefeat
*/
eVictoryLevels
tVictoryCondition::mCalcVictoryStatus( eTeamID ID )
{
	Assert( fVictoryState );
	if( fVictoryState )
	{
		return fVictoryState->mCalcVictoryStatus( ID );
	}
	
	return kDraw; 
}

/*
	Purpose:	An accessor to the prestige points for a particular victory
				level.

	Inputs:		( eVictoryLevels )	-	The level of victory in question
	Outputs:	( int32 )			-	The number of prestige points associated
										with the level of victory in question
*/
int32
tVictoryCondition::mGetPrestigePoints( eVictoryLevels Level )
{
	Assert( fVictoryState );
	if( fVictoryState )
	{
		if ( Level >= kAstoundingVictory )
		{
			return fVictoryState->mGetPrestigePoints( Level );
		}
		else
		{
			return fVictoryState->mLookupPrestigeValue( Level );
		}
	}
	
	return 0; 
}

/*
	Purpose:	An accessor to the bonus prestige points received so far
				with the team associated with this victory condition.

	Inputs:		N/A
	Outputs:	( int32 )	-	The number of bonus prestige points accumulated
*/
int32
tVictoryCondition::mGetBonusPrestige( void )
{
	Assert( fVictoryState );
	if( fVictoryState )
	{
		return fVictoryState->mGetBonusPrestige();
	}
	
	return 0; 
}

/*
	Purpose:	Will return the prestige value associated with a special victory level.

	Inputs:		( eVictoryLevels )	-	The level of victory in question
	Outputs:	( int32 )			-	The number of prestige points associated
										with the level of victory in question
*/
int32
tVictoryCondition::mLookupPrestigeValue( eVictoryLevels Level )
{
	Assert( fVictoryState );
	if ( fVictoryState )
	{
		return fVictoryState->mLookupPrestigeValue( Level );
	}
	return 0;
}

/*
	Purpose:	Registering a state tells all actors to send out its messages to
				the newly registered state.

	Inputs:		( tState* )	-	The state in question
	Outputs:	N/A
*/
void
tVictoryCondition::mRegisterState( tState* State )
{
	Assert( kFalse );
}

/*
	Purpose:	Meant as an initialization of a newly registered state.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tVictoryCondition::mSetupState( void )
{
	Assert( fPlayerTeamID != kNoTeam && fEnemyTeamID != kNoTeam );

	tNormalVictoryState* normalVictoryState = new tNormalVictoryState( fCaptureBouns, fAstoundingVictory, fVictory, fDraw, fDefeat, fDevastatingDefeat );

	if( fEnemyTeamID != kNoTeam )
	{
		normalVictoryState->mAddEnemySide( fEnemyTeamID );
	}
	
	if( fPlayerTeamID != kNoTeam )
	{
		normalVictoryState->mAddPlayerSide( static_cast< eTeamID >( fPlayerTeamID ), kTrue );
	}

	mRegisterState( normalVictoryState );
	
	mGotoState( typeid( tNormalVictoryState ) );	
}

/*
	Purpose:	An accessor to the team associated with this victory condition.

	Inputs:		N/A
	Outputs:	( tTeamInfo* )	-	The team requested
*/
tTeamInfo*
tVictoryCondition::mGetTeam( void )
{
	return fTeam;
}

//	15Nov99	zdrummond	Added Comment Header and Footer's
//	21Apr00	dsuleski	Added default constructor which is needed for custom victory conditions
//	04Jun00	dsuleski	No longer needed the enemy/player id in the constructor
//	08Jul00	dsuleski	Documentation update