/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	for ( int i = 0; i < 6; i++ )
	{
		fShipsDestroyed[i] = 0;
		fTeamWon[i] = true;
	}
	for ( i = 0; i < kNumStartingPositions; i++ )
	{
		fPosUsed[i] = kFalse;
	}
	fNumTeams = 0;
}

/*

	Purpose:	Will end the game upon when only one team is left.  Essentially, all players (human or
				otherwise) are initialized as already won.  As ships are knocked out of the game, they
				are set to a losing condition and their game ends and should theorectically end when
				there is only one ship left standing.  However, since teams can be dynamically changed
				according to the player, rather than check for one ship standing, we need to check if all
				of the ships standing are from the same team tag.  If so, then we run through each of those
				remaining teams and set a winning victory condition for each.

*/
void tWorldConditions::mCheckForGameEnd( void )
{

	iTruth	remainingShipsOnSameTeam = true;
	int32	lastWinningTeam = -1;

	//
	//	Find out if all the ships left standing are from the same team.
	//
	for ( int i = 0; i < fNumTeams; i++ )
	{
		//
		//	If we already have found a winning team, then lastWinningTeam must be the most recent winning
		//	team we have found.
		//
		if ( lastWinningTeam != -1 )
		{
			tMissionInfo*	thisMission = gScriptInterface.mMission();

			tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			Assert( currentTeam );
			if ( !currentTeam )
				continue;
			tTeamInfo* previousTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( lastWinningTeam ) );
			Assert( previousTeam );
			if ( !previousTeam )
				continue;

			//
			//	If the two ships do not match team tags, then we know that a fight is still brewing and
			//	we should not end any games as winners yet.
			//
			if ( mTeamWon( i ) )
			{
				if ( currentTeam->mGetTag() != previousTeam->mGetTag() )
				{
					remainingShipsOnSameTeam = false;
					break;
				}
			}
		}

		//
		//	If the current team still has the won flag standing, then save the ID so that we can compare
		//	on the next pass through.
		//
		if ( mTeamWon( i ) )
		{
			lastWinningTeam = i;
		}
	}

	//
	//	If we have hit a winning condition (all ships remaining on same team), then we should end the game
	//	for all remaining players with a winning condition.
	//
	if ( remainingShipsOnSameTeam )
	{
		for ( i = 0; i < fNumTeams; i++ )
		{
			if ( mTeamWon( i ) )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
			}
		}
	}
}

/*

	Purpose:	Returns a random starting position for each of the ships.  Only once all starting positions
				are used up (G - Z), will a starting position be available for reuse.

*/
iMapPosition tWorldConditions::mGenerateRandomStartPosition( void )
{
	//
	//	Make certain that there is at least one starting point that is still valid.  If not, then reset
	//	all starting points to be valid.
	//
	iTruth	valid = kFalse;
	for ( int i = kFirstStartPosition; ( i < kFirstStartPosition + kNumStartingPositions ) && ( !valid ) ; i++ )
	{
		if ( !fPosUsed[i] )
			valid = kTrue;
	}
	if ( !valid )
	{
		for ( i = 0; i < kNumStartingPositions; i++ )
		{
			fPosUsed[i] = kFalse;
		}
	}

	//
	//	Now find a valid starting point.
	//
	while ( true )
	{
		int32 guess = gScriptInterface.mMission()->mRandomInt32( 0, kNumStartingPositions );
		if ( !fPosUsed[guess] )
		{
			fPosUsed[guess] = kTrue;
			return static_cast< iMapPosition >( guess + kStartPosition_G );
		}
	}

	//
	//	Should never get here.
	//
	return -1;
}

int32 tWorldConditions::mShipsDestroyed( int x )
{
	return ( x >= 0 && x <= 5 )
		? fShipsDestroyed[x] : 0 ;
}
bool tWorldConditions::mTeamWon( int x )
{
	return ( x >= 0 && x <= 5 )
		? fTeamWon[x] : 0;
}

void tWorldConditions::mIncrementShipsDestroyed( int x )
{
	if ( x < 0 || x > 5 )
		return;
	fShipsDestroyed[x]++;
}

void tWorldConditions::mSetTeamWon( int x, bool y )
{
	if ( x < 0 || x > 5 )
		return;
	fTeamWon[x] = y;
}