/*
	BaseVictoryState.cpp

	This module contains the definition for the tBaseVictoryState class.
*/

#include "Constants.h"
#include "Uni_X4M1AThief.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "EnemyTeam.h"
#include "EnemyShip.h"
#include "TransportTeam.h"

// Default constructor initializes all member variables for the scenario.

tBaseVictoryState::tBaseVictoryState(tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns, int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat) :
	tChildActorVictoryState<tBaseVictory>(astoundingVictory, victory, draw, defeat, devastatingDefeat),
	fNumTransportsCaptured(0),
	fNumTransportsLeft(NUM_TRANSPORTS),
	fEnemyDestroyed(kFalse),
	fEnemyCaptured(kFalse),
	fEnemySurrendered(kFalse),
	fEnemyAttackedAfterSurrender(kFalse),
	fTractoringATransport(kFalse)
{}

/*
	This method checks the race of the player and adds the appropriate bonus
	prestige for defeating the enemy by use of non-lethal force. At present,
	only the Federation can earn this bonus.
*/

void tBaseVictoryState::AddNonLethalBonus()
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	// A Federation player gets a special bonus for avoiding lethal
	// use of force in fulfilling the mission objectives.

	if (playerTeam->mGetRace() == kFederation)
		mAddToBonusPrestige(PRESTIGE_NON_LETHAL_BONUS * 3);
}	

/*
	Returns kTrue if the enemy guard ship has been defeated or kFalse otherwise.
	The formula, while it should be obvious from the code below, is pretty
	simple; the enemy is defeated if the ship is destroyed or captured. Also,
	the enemy is defeated if he has surrendered and the player has honored that
	surrender.
*/

iTruth tBaseVictoryState::IsEnemyDefeated()
{
	if( fEnemyDestroyed || fEnemyCaptured || (fEnemySurrendered && !fEnemyAttackedAfterSurrender) )
	{
		return( 1 );
	}
	else
	{
		return( 0 );
	}
}

/*
	This method sets the random chatter timer if the appropriate preprocessor 
	directive is defined.
*/

void tBaseVictoryState::mBeginState()
{
//#if defined(kRandomChatter)
//	mSetTimedInterval(kTimerSecondsSet, kRandChatID, RANDOM_CHATTER_INTERVAL);
//#endif
}

void tBaseVictoryState::mEndState()
{}

/*
	Victory levels for this mission are pretty much on a do-or-die basis. If
	the player gets all the transports and defeats the enemy, then he's won
	an astounding victory. Getting the required number and defeating the enemy
	is mere victory. Anything less is essentially a failure; there is no such
	thing as a "draw" for this mission.
*/

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus(eTeamID ID)
{
	// TODO: figure out what to do with the ID.
	iTruth fEnemyDefeated = IsEnemyDefeated();

	if ((fNumTransportsCaptured == NUM_TRANSPORTS) && fEnemyDefeated)
		return kAstoundingVictory;
	else if ((fNumTransportsCaptured >= MIN_TRANSPORTS_TO_CAPTURE) && fEnemyDefeated)
		return kVictory;
	else if ((fNumTransportsCaptured >= MIN_TRANSPORTS_TO_CAPTURE) || fEnemyDefeated)
		return kDefeat;
	else
		return kDevastatingDefeat;
}

/*
	This event handler deals with (1) the death of the enemy ship by adding
	the necessary prestige and potentially ending the mission, (2) the
	death of a transport by decrementing the number of transports in play and
	potentially ending the mission, and (3) the death of player ships by
	ending the mission if none are left.
*/

void tBaseVictoryState::mOnDeath(tShipInfo* ship)
{
	// If it's the enemy destroyer, then:
	//
	//	(1) Set the enemy destroyed flag.
	//	(2) Show the appropriate message about ejecting debris and so forth.
	//	(3) If any transports are left, show the message about the secondary
	//		objective being complete.
	//	(4) Otherwise, show either the all objectives completed" or the "we've 
	//		done all we can" message depending on whether the player has
	//		captured enough transports. Then end the game with player wins.

	if (ship->mGetTeam() == kEnemyTeam && ship->mGetCustomID() == tEnemyTeam::kEnemyShip)
	{
		fEnemyDestroyed = kTrue;
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemyShipDestroyed);
		mAddToBonusPrestige(PRESTIGE_ENEMY_DESTROYED * 3);

		// If there are any transports remaining to be dealt with, then
		// show the message only that the secondary objective is accomplished.
		// Otherwise, if the minimum number have been captured for success,
		// then show that all objectives have been accomplished.

		if (fNumTransportsLeft > 0)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kSecondObjectiveCompleted);
		else 
		{
			if (fNumTransportsCaptured > MIN_TRANSPORTS_TO_CAPTURE)
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAllObjectivesCompleted);
			else
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDoneAllWeCan);

			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerWins);
		}
	}

	// If the original ship team is the transport team, just decrement the
	// counter for transports still in play and end the game if the enemy
	// ship is also defeated.

	else if (ship->mGetOriginalTeam() == kTransportTeam)
	{
		if ((--fNumTransportsLeft == 0) && IsEnemyDefeated())
			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerWins);
	}

	// If it's the player's last ship, end the mission with player failure.

	else if (ship->mGetTeam() == kPlayerTeam)
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(ship->mGetTeam());
		if (!team->mShipsRemainingInPlay())
			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerDestroyed);
	}
}

/*
	This event handler deals with (1) the capture of the player's ship by ending
	the game if none remain, and (2) the capture of the enemy destroyer by
	adding appropriate bonus prestige and potentially ending the mission if
	no transports are left in play.
*/

void tBaseVictoryState::mOnCapture(tShipInfo* ship)
{
	// If the player's ship is captured, end the mission with player failure.

	if (ship->mGetOriginalTeam() == kPlayerTeam)
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(ship->mGetTeam());
		if (!team->mShipsRemainingInPlay())
			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerCaptured);
	}

	// If the enemy destroyer is captured, set the captured flag.

	if ( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ship->mGetCustomID() == tEnemyTeam::kEnemyShip)
	{
		fEnemyCaptured = kTrue;
		AddNonLethalBonus();
		mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED * 3);

		if (fNumTransportsLeft == 0)
			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerWins);
	}
}

/*
	This event handler deals with (1) the disengaging of a transport by
	adding the appropriate prestige, incrementing the transports captured
	counter and potentially ending the mission if none are left in play,
	and (2) the player's ship disengaging, which ends the scenario and
	checks for the possibility of tractoring a transport.
*/

void tBaseVictoryState::mOnDisengaged(tShipInfo* ship)
{
	// If the ship disengaging is one of the transports, then:
	//
	// (1) Show the proper message to the player.
	// (2) Increment the number of transports captured.
	// (3) Decrement the number of transports left in play.

	if (ship->mGetOriginalTeam() == kTransportTeam)
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportDisengaged);
		mAddToBonusPrestige(PRESTIGE_PER_TRANSPORT * 3);
		++fNumTransportsCaptured;

		// If all the transports are to safety, we end the mission if the 
		// secondary objective has also been met.

		if ((--fNumTransportsLeft == 0) && IsEnemyDefeated())
			gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerWins);
	}

	// If the ship disengaging is the player ship, then end the mission.

	else if (ship->mGetTeam() == kPlayerTeam && ship->mGetCustomID() == tPlayerTeam::kPlayerShip)
	{
		// If the player is tractoring a transport, be sure to give him
		// credit for it.

		if (fTractoringATransport)
		{
			mAddToBonusPrestige(PRESTIGE_PER_TRANSPORT * 3);
			++fNumTransportsCaptured;
		}

		gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerDisengaged);
	}
}

void tBaseVictoryState::mAtDistance(tShipInfo* ship)
{}
void tBaseVictoryState::mAtDestination(tShipInfo* ship)
{}
void tBaseVictoryState::mOnCommButtonPress(tShipInfo* ship, int32 buttonID, tShipInfo* targetShip)
{}
void tBaseVictoryState::mOnDamage(tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType)
{}
void tBaseVictoryState::mOnDeepScan(tShipInfo* ship, iTruth deepScanOn)
{}
void tBaseVictoryState::mOnAlertStatusChange(tShipInfo* ship, eAlertLevels alertStatus)
{}
void tBaseVictoryState::mOnProbeLaunch(tShipInfo* ship)
{}
void tBaseVictoryState::mOnBeginOrbit(tShipInfo* ship)
{}
void tBaseVictoryState::mOnEndOrbit(tShipInfo* ship)
{}

// This method sets fTractoringATransport to kTrue whenever the player begins tractoring a transport.

void tBaseVictoryState::mOnBeginTractor(tShipInfo* tractorTarget, tShipInfo* tractoringShip)
{
	int32 iShipID = tractoringShip->mGetCustomID();
	int32 iCargoID = tractorTarget->mGetCustomID();

	if (iShipID == tPlayerTeam::kPlayerShip && 
		tTransportTeam::kTransport1 <= iCargoID && iCargoID <= tTransportTeam::kTransport4)
	{
		fTractoringATransport = kTrue;
	}
}

// This method sets fTractoringATransport to kFalse whenever the player ends tractoring a transport.

void tBaseVictoryState::mOnEndTractor(tShipInfo* tractorTarget, tShipInfo* tractoringShip)
{
	int32 iShipID = tractoringShip->mGetCustomID();
	int32 iCargoID = tractorTarget->mGetCustomID();

	if (iShipID == tPlayerTeam::kPlayerShip && 
		tTransportTeam::kTransport1 <= iCargoID && iCargoID <= tTransportTeam::kTransport4)
	{
		fTractoringATransport = kFalse;
	}
}

void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem(tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item)
{}
void tBaseVictoryState::mOnChangingShip(tShipInfo* ship)
{}
void tBaseVictoryState::mOnSpecialTransport(tShipInfo* ship)
{}
void tBaseVictoryState::mOnPlayerLeaving(int playerNumber)
{}
void tBaseVictoryState::mOnHostChanged()
{}

// This method handles the random chat timer.

void tBaseVictoryState::mOnTimedInterval(int32 timerID)
{
/*
#if defined(kRandomChatter)
	if (timerID == kRandChatID)
	{
		switch (fMissionInfo->mRandomInt32(0, 5))
		{
		case 0:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat1_msg);	break;
		case 1:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat2_msg);	break;
		case 2:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat3_msg);	break;
		case 3:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat4_msg);	break;
		case 4:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat5_msg);	break;
		case 5:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat6_msg);	break;
		}

		float fInterval = RANDOM_CHATTER_INTERVAL + 
						  (float)(fMissionInfo->mRandomInt32(0, (int32)RANDOM_CHATTER_RANGE));
		fChildActor->mSetTimedInterval(kTimerSecondsSet, timerID, fInterval);
	}
#endif
*/
}

void tBaseVictoryState::mOnStateChange(tActor* actor, tState* stateID, eStateEvent eventType)
{}

/*
	This method probably shouldn't be here, but it works, so I presently see
	no reason to fix what ain't broke. Handling the surrender of the enemy
	ship occurs in the tEnemyShipBaseState::mOnCommButtonPress method, but
	since the surrender affects the victory conditions, this method watches
	for comm state changes to the two comm states that correspond to the
	enemy ship having surrendered and having been attacked after surrender.
*/

void tBaseVictoryState::mOnCommStateChange(tActor* actor, int32 state, int32 buttonID, eStateEvent eventType)
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

	if (eventType == kStartingNewState && actor == playerTeam && buttonID == k_DS_ISC_btn)
	{
		// If the comm state is the surrendered state, then set up the
		// prestige bonus and set the flag.

		if (state == tEnemyShip::kCommState2)
		{
			AddNonLethalBonus();
			mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED * 3);
			fEnemySurrendered = kTrue;

			// If the minimum number of transports have been captured,
			// then show that all objectives have been accomplished.
			// Otherwise, tell the player we've done all we can.

			if (fNumTransportsLeft == 0)
			{
				if (fNumTransportsCaptured > MIN_TRANSPORTS_TO_CAPTURE)
					fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAllObjectivesCompleted);
				else
					fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDoneAllWeCan);
				gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, kPlayerWins);
			}
		}

		// If the comm state is the attacked after surrendered state,
		// then adjust the prestige and set the flag.

		else if (state == tEnemyShip::kCommState1)
		{
			mAddToBonusPrestige(PRESTIGE_FALSE_SURRENDER * 3);
			fEnemyAttackedAfterSurrender = kTrue;
		}
	}
}
