/*
	EnemyShipBaseState.cpp

	This module contains the definition of the tEnemyShipBaseState class,
	which does a fair amount of handling for the enemy ship.
*/

#include "Constants.h"
#include "Uni_X4M1AThief.h"
#include "EnemyShipBaseState.h"
#include "MissionText.h"
#include "TeamInfo.h"
#include "EnemyShip.h"

// Default constructor initializes all variables for the mission.

tEnemyShipBaseState::tEnemyShipBaseState() :
	fHailed(kFalse),
	fCaptured(kFalse),
	fSurrendered(kFalse),
	fAttackedAfterSurrender(kFalse)
{
}

void tEnemyShipBaseState::mBeginState()
{}
void tEnemyShipBaseState::mEndState()
{}
void tEnemyShipBaseState::mOnDeath( tShipInfo* ship )
{}

/*
	This method handles the capture of the enemy ship. When captured, an
	appropriate message will be displayed, and the enemy ship will be
	ordered to disengage from the map.
*/

void tEnemyShipBaseState::mOnCapture( tShipInfo* ship )
{
	if (fChildActor->mGetCustomID() == ship->mGetCustomID())
	{
		fChildActor->mDisengage(kByMapEdge, kAbsolutePriority);
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kEnemyShipCaptured);
	}
}

void tEnemyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyShipBaseState::mAtDestination( tShipInfo* ship )
{}

/*
	This method handles comm buttons for the enemy ship. The hail message is
	changed after the initial hail, and the ship will surrender if and only
	if at least it's damage is greater than or equal to SURRENDER_THRESHOLD.
*/

void tEnemyShipBaseState::mOnCommButtonPress(tShipInfo* ship, int32 buttonID, tShipInfo* targetShip)
{
	if (fChildActor->mGetCustomID() == targetShip->mGetCustomID())
	{
		if (buttonID == k_HAIL_ISC_btn)
		{
			// When the hail button is pressed, if the ship has already been
			// captured, has surrendered or has been attacked after having
			// surrendered, then we do nothing. Only the comm state for a
			// surrender can change, and this will happen only when the ship
			// is attacked again.

			if (fCaptured || fSurrendered || fAttackedAfterSurrender)
				return;

			if (!fHailed)
			{
				float fDamage = targetShip->mGetDamagePercent();
				tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

				if (fDamage < SURRENDER_THRESHOLD)
					playerTeam->mGotoCommState(targetShip->mGetShipID(), tEnemyShip::kCommState6);
				else
					playerTeam->mGotoCommState(targetShip->mGetShipID(), tEnemyShip::kCommState7);

				fHailed = kTrue;
			}
		}
		else if (buttonID == k_DS_ISC_btn)
		{
			// If we've been captured or have already surrendered or been
			// attacked after surrender, we can simply exit the handler.

			if (fCaptured || fSurrendered || fAttackedAfterSurrender)
				return;

			// Otherwise, we get the damage percent and surrender if it is 
			// greater than the required threshold.

			float fDamage = targetShip->mGetDamagePercent();

			if (fDamage >= SURRENDER_THRESHOLD)
			{
				tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
				playerTeam->mGotoCommState(targetShip->mGetShipID(), tEnemyShip::kCommState2, k_DS_ISC_btn);
				tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
				fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kSecondObjectiveThroughSurrender);
				// TODO: figure out the float value's meaning
				enemyTeam->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamNeutral, kGoalIgnore, 1.0);	
				targetShip->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
				fSurrendered = kTrue;
			}
		}
	}
}

/*
	This method handles damage to the ship after the enemy has surrendered.
	If the player is really treacherous, then the enemy ship will rejoin the
	fight and it will cost the player prestige.
*/

void tEnemyShipBaseState::mOnDamage(tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType)
{
	if (fChildActor->mGetCustomID() == damagedShip->mGetCustomID())
	{
		// If we've already been attacked after surrender, then ignore it.

		if (fAttackedAfterSurrender)
			return;

		// If we are in a surrendered state, let's get revenge.

		else if (fSurrendered)
		{
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kAttackedAfterSurrender);
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
			playerTeam->mGotoCommState(damagedShip->mGetShipID(), tEnemyShip::kCommState1, k_DS_ISC_btn);
			tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
			damagedShip->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 1.0f);
			enemyTeam->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamAtWar, kGoalDestroy, 1.0);	
			fAttackedAfterSurrender = kTrue;
		}
	
		// Otherwise, we might need to modify the current comm state if badly
		// damaged enough to make surrender an attractive option.

		else if (damagedShip->mGetDamagePercent() >= SURRENDER_THRESHOLD)
		{
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
			if (fHailed)
				playerTeam->mGotoCommState(damagedShip->mGetShipID(), tEnemyShip::kCommState7);
			else
				playerTeam->mGotoCommState(damagedShip->mGetShipID(), tEnemyShip::kCommState5);
		}

		// Finally, reset the callback for damage notification.

		damagedShip->mSetShipCallbackEvents(kNotifyDamage);
	}
}

void tEnemyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipBaseState::mOnFlashingButtonPressed()
{}
void tEnemyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tEnemyShipBaseState::mOnHostChanged()
{}
void tEnemyShipBaseState::mOnTimedInterval( int32 timerID )
{}
void tEnemyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

