/*
	TransportBaseState.cpp

	This module contains the definition for the tTransportBaseState class,
	which handles the capturing and damage of transports.
*/

#include <limits>
#include "Uni_X4M1AThief.h"
#include "TransportBaseState.h"
#include "MissionText.h"

void tTransportBaseState::mBeginState()
{}
void tTransportBaseState::mEndState()
{}
void tTransportBaseState::mOnDeath( tShipInfo* ship )
{}

/*
	This method is responsible for handling the capture of a transport.
	When a transport is captured, the follow steps must be taken:

	(1) The ship maximum speed must be restored to 31 so that it can move.
	(2) The ship shields must be restored.
	(3) The ship must be ordered to leave the map at best possible speed.
	(4) A message should be provided to acknowledge the capture.
	(5) The proper comm state should be set.
*/

void tTransportBaseState::mOnCapture(tShipInfo* ship)
{
	if (fChildActor->mGetCustomID() == ship->mGetCustomID())
	{
		fChildActor->mSetShipMaxSpeed(31);
		fChildActor->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 1.0f);
		fChildActor->mDisengage(kByMapEdge, kAbsolutePriority);
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportCaptured);
		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		playerTeam->mGotoCommState(fChildActor->mGetShipID(), tTransport::kCommState2);
	}
}

void tTransportBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tTransportBaseState::mAtDistance( tShipInfo* ship )
{}
void tTransportBaseState::mAtDestination( tShipInfo* ship )
{}
void tTransportBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}

/*
	This method is responsible for handling damage to the transports after
	they've been captured by the player. As the damage increases, the tone
	of the replies changes to let the player know whether the transports
	are in serious trouble and require assistance.
*/

void tTransportBaseState::mOnDamage(tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType)
{
	if (fChildActor->mGetCustomID() == damagedShip->mGetCustomID() &&
		fChildActor->mGetTeam() == kPlayerTeam)
	{
		float fDamage = fChildActor->mGetDamagePercent();
		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

		// Note: when testing for zero damage, it is necessary to test against
		// the C++ library defined lower limit due to float precision. In short,
		// the first case of the if statement is for zero damage.

		if (fDamage < std::numeric_limits<float>::epsilon())
			playerTeam->mGotoCommState(damagedShip->mGetShipID(), tTransport::kCommState2);
		else if (fDamage < 30.0f)
			playerTeam->mGotoCommState(damagedShip->mGetShipID(), tTransport::kCommState3);
		else if (fDamage < 60.0f)
			playerTeam->mGotoCommState(damagedShip->mGetShipID(), tTransport::kCommState4);
		else
			playerTeam->mGotoCommState(damagedShip->mGetShipID(), tTransport::kCommState5);

		damagedShip->mSetShipCallbackEvents(kNotifyDamage);
	}
}

void tTransportBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTransportBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTransportBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnFlashingButtonPressed()
{}
void tTransportBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTransportBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tTransportBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tTransportBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tTransportBaseState::mOnHostChanged()
{}
void tTransportBaseState::mOnTimedInterval(int32 timerID)
{}
void tTransportBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTransportBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
