//	<Name>	tcrandsequence.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	Interface for the standard c random number generator and a linear Congruential algorithm 

//	Note: See tsequence.h for additional functions

#ifndef tcrandsequence_h
#define tcrandsequence_h 1

#include "tsequence.h"
#include "trandsequencedefs.h"


class tCRandomSequence : public tSequencePart
{
public:
	
	// random number generator types; to be expanded as more algorithms are added
	typedef enum
	{
		kStandardCAlgorithm,		// use the standard generator
		kLinearCongruentialAlgorithm
	} tGenerator;

	// constructors and deconstructors
	tCRandomSequence(int32u initialSeed = 1, int32u historySize = 0);

	tCRandomSequence(int8u bitSize, int32u initialSeed = 1, int32u historySize = 0);

	tCRandomSequence(tGenerator algorithm, int32u initialSeed = 1, int32u historySize = 0);

	tCRandomSequence(int32u initialSeed, int32u historySize, tGenerator algorithm, int8u bitSize);
	
	~tCRandomSequence(void);
		
	// set and get the random number generator being used
	tGenerator	mGetGenerator(void)					{ return fAlgorithm; }

	// returns a random integer from 0 to upper_limit-1
	int32u		mGetRandomInteger(int32 upper_limit) { return (int32)(upper_limit*(double)mQuickGet()/(double)fLargestValue); }; 

	// returns a random double or float in the range of 0 to .9999999999999999999...
	double		mGetRandomDouble() { return (double)mQuickGet()/(double)fLargestValue; }; 
	float		mGetRandomFloat() { return (float)((double)mQuickGet()/(double)fLargestValue); };

	// generally speaking this is what mQuickGet() needs to be divided by to give the
	//	above values...
	int32u		mGetLargestValue() { return fLargestValue; };

		
	virtual void	mSetSeed(int32u newSeed);


	int32u mGetLastValueGenerated() { return fLastValueGenerated; };

protected:

	tGenerator	fAlgorithm;	// the default generator to use
	int8u		fBitSize;	// the bit-size of the numbers to generate
	int32u		fLargestValue; // the maximum number
	int32u		fLastValueGenerated; // a quick way to get the last generated number
	int32u		fModulusM;
	int32u		fMultiplierA;
	int32u		fIncrementC;
 /*
 x_{n+1} = (a x_n + c) mod m
The mod refers to modulo integer division. this algorithm depends on four numbers, a, c, m, and x_0, called the seed. For computational speed, m is almost always chosen to be 2^p where p is the number of bits in an integer. 
Topics in the classroom: 

Representing integers in base 2. 
Example with m = 8. 
Theory for the length of the cycle of a generator
A mixed linear congruential generator will have period m if and only if 
c is relatively prime to m 
a mod p = 1 for every prime factor p of m 
a mod 4 = 1 if 4 is a factor of m 
*/

	// generates a random number
	virtual int32u	mGenerateNumber(void);
	
private: 

#if rng_tests

//--------------------------------------------------------------------------------
// functions for testing a random number generator; see xrandsequence.cpp for 
// examples of how to use
// 

public:

	// data structure for a simple linked list 
	struct sHistoryNode
	{
		int32u number;			// the randomly generated number
		sHistoryNode *link;		// a link to the previous number
	};
	
	// setup the tests for the RNG; needs the number range being used
	void	mInitializeTests(int32u numberRange);
	
	// returns how many times each number occured; needs an array of size fNumberRange
	void	mGetHistogram(int32u *histogram);
	
	// returns the longest run of each number generated; needs an array of size fNumberRange
	void	mGetLongestRun(int32u *longestRun);
	
	// returns the full history of random numbers generated; needs an array of the size of
	// the total number of random numbers that were generated
	void 	mGetFullHistory(int32u *fullHistory);
	
	// returns kTrue if the algorithm can generate all the numbers within the bit-size
	// once before repeating, kFalse otherwise
	iTruth	mAllNumbersGenerated(int32u seed);

protected:

	int32u fNumberRange;		// the number range being used
	sHistoryNode *fFullHistory;	// the full history of numbers generated
	
#endif
};

#endif

//	<History>
//	13Jul98	oschlunk		Created off of trandsequence.
//	15Jul98	oschlunk		Add a linear congruential algorithm -- it's not perfect, but it should be good enough for
//							the sorts of things we are doing....	 

