//	<Name>	tpart.h

//	<Legal>	Copyright (c) 1995 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	classes for managing individual system "parts" or components

#pragma once

#include "tbasic.h"
#include "tlearn.h"

#ifndef	tpart_h
#define	tpart_h	1

#include "tpartdefs.h"

typedef enum
{
	kPartUnregistered = 0,
	
	kPartRegistered,
	
	kPartValidInactive,
	kPartValidActive,
	
	kPartStateLim
}	ePartState;

class	tPart
{
	public:
			// methods/fields required of parts
									tPart			( void );	// defaults to call mRegisterSelf()
							virtual	~tPart			( void );

			// implemented by root class only, but public (no overrides for any of these)
			
					void	mRegisterSelf	( void );
		static		void	mValidateParts	( void );
		static		void	mActivateParts	( void );
		static		void	mDeactivateParts( void );
		
		static		void	mResetPartCachePtrList		( void );
		static		void	mAddToPartCachePtrList		( tPart ** addrOfCachePtr );
		static		void	mInvalidatePartCachePtrs	( void );

		static		iCount	mCountCommonParts			( tPart *partPtr );
		static		tPart*	mGetIndexedCommonPart		( tPart *partPtr, iIndex commonPartIndex );
		static		void	mSubstituteCommonPart		( tPart* oldPart, tPart* newPart );
		
			// implemented by concrete subclasses (overrides expected)
			// but not implemented for abstract subclasses for semi obvious reasons
			
		virtual		iTruth	mIsValid	( void ) = 0;				// override: check system dependencies (only once)
		virtual		iTruth	mIsReady	( void ) = 0;				// override: check inter-part dependencies (maybe often)		
		
		virtual		iTruth	mIsCommon	( tPart* otherPart )		// override: check for common base class against another part
			{ return (otherPart==this); };							// someday this will become pure virtual. get ready!
																	//	(this was added for dynamic part support as well as startup robustness)
																	
		virtual		void	mActivate	( void ) = 0;				// override: only call if first two conditions are met
		virtual		void	mDeactivate	( void ) = 0;				// override: only call if previously activated

																	
					
			// fields
		
		ePartState		fPartState;
		
		int32s			fPartGroupIdent;	// used to group commonly based classes.
											// (parts with common bases should have equal Ident values)
						
			// exceptions
			
		class	tPartStartupException	{};
		class	tPartBindingException	{};
		
	protected:
		static		int32u	fStaticInitCheck;
		
			// list of static part objects that registered at startup time
		static		iCount	fStaticPartRegCount;
		static		tPart	*fStaticPartRegList[ kPartCountMax ];

			// list of parts that had validated system needs (ready to attempt activation)
		static		iCount	fStaticPartValidCount;
		static		tPart	*fStaticPartValidList[ kPartCountMax ];
		
			// list of parts in the order they were actually activated.
			// this also gets used to deactivate in the reverse order.
		static		iCount	fStaticPartActivateCount;
		static		tPart	*fStaticPartActivateList[ kPartCountMax ];		

			// list of "addresses of part cache pointers"
		static		iCount	fStaticPartCachedPtrCount;
		static		tPart	**fStaticPartCachedPtrList[ kPartCachedPtrCountMax ];			// note, two stars
};


	// macro to generate code in each abstract interface (subclass of tPart).
	// the method name generated is of the form Bind_<classname>( void )
	// this is accomplished through use of the ## operator which allows for symbol concatenation.

#define	GenAbstractBinding(classx)								\
		static classx *Search_##classx ( void )					\
		{														\
			static	classx	*partCachePtr = (classx*) nil;		\
			if (partCachePtr)	return	partCachePtr;			\
			else												\
			{													\
				for( int idx = 0; idx<tPart::fStaticPartActivateCount; idx++)			\
				{																		\
					classx	*bindResult = dynamic_cast< classx *>(tPart::fStaticPartActivateList[idx] );	\
					if (bindResult)														\
					{																	\
						tPart::mAddToPartCachePtrList( (tPart**)&partCachePtr );		\
						partCachePtr = bindResult;										\
						return bindResult;												\
					}												\
				}													\
				throw	tPartBindingException();					\
				return (kFalse);									\
			}														\
		};															\
		static iTruth Test_##classx( void )							\
		{															\
			try	{ classx *test = classx::Search_##classx();	}		\
			catch (tPartBindingException)	{ return	kFalse;	}	\
			return kTrue;											\
		}


	// macro to do part lookups based on the name of an abstract ancestor class
	// (relies on the ## concatenation effect)

#define	BindPart(classx)	( classx::Search_##classx() )
#define	TestPart(classx)	( classx::Test_##classx() )
#define InstantiatePart(classx) classx inst_of_##classx; iTruth validity_of_##classx = (inst_of_##classx).mIsValid(); 

	// macro to construct mIsCommon implementations quickly
#define	IsCastableAs(classptr,classx)	(dynamic_cast< classx* >(classptr) != NULL)

#endif	// end tpart_h block




//	<History>
//	05Oct95	rbarris		Started.
//	10Jan97	rbarris		renamed for new Tiger project, simplified startup model
//  14Jan97 mhertogh	added return of kFalse after the exception when rtti could not find a compatible type (needed for MSVC++)
//	17Jan97	rbarris		simplified usage a bit, changed some names. Added tPartBindingException. No more need for to call mRegisterSelf().
//	20Jan97	rbarris		better naming "is valid" and "is ready".
//  20Jan97 mhertogh	casted nil to classx* for MSVC++.
//	24Feb97 rbarris		fixed angle brackets on initial #includes. Needs to be quotes.
//	14Jun97	rbarris		added new stuff for dynamic part switching.
//	01Jul97 bsnyder		Added InstantiatePart.  Not having the validity stuff caused some linkers to not link in parts.