//	<Name>	tvectormath.h
//
//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	classes and functions for working with 3D vector/matrices
//
#ifndef	tvectormath_h
#define	tvectormath_h	1

#if SystemMac7
	#include	<fp.h>
#endif

#if SystemWin32
	#include	<math.h>
#endif

#ifdef __BORLANDC__
    #pragma warn -inl   // remove "inline function not expanded" warnings
#endif

//--------------------------------------------------------------------------------
// Ensure our base 3D data types are packed tightly
//
#if CompilerMetrowerks && SystemMac7
	#pragma options align=mac68k
#elif (CompilerMetrowerks && SystemWin32) | CompilerBorland | CompilerMicrosoft | CompilerWatcom
	#pragma pack(push)
	#pragma pack(1)
#else
	#error	"Don't know how to pack structures for your compiler"
#endif

//--------------------------------------------------------------------------------
// Data type used to represent angles. This type and its inherent range differs
// from radian or degree measure.
//
typedef short	tAngle;

//--------------------------------------------------------------------------------
// Used for fast conversion of floats to short int, simply add 
// kFloatToInt to _f and use the macro FLT2INT_INT to retrieve the short int
//
// examples:
//
//	tFlt2Int	fi;		fi._f = 3.1235f + kFloatToInt;
//	short int	si;		si = FLT2INT_INT(fi);
//
typedef union
{
	float	_f;
	short	_short[2];
} tFlt2Int;

// Macro to retrieve the proper short in above union
//
#if EndianLittle	// defined by tlearn.h
	#define	FLT2INT_INT(a)	((a)._short[0])
#elif EndianBig
	#define	FLT2INT_INT(a)	((a)._short[1])
#else
	#error	"Don't know the endian-ness of your machine"
#endif

//--------------------------------------------------------------------------------
//
struct tPoint3
{
public:
	float	x, y, z;

public:
    // Constructors, destructor
    //
	inline tPoint3(const float aX, const float aY, const float aZ)
	{
		x = aX; y = aY; z = aZ;
    }
    inline tPoint3(const tPoint3 &p)
    {
        x = p.x; y = p.y; z = p.z;
    }
	inline tPoint3()
	{
		// do nothing, leave uninitialized
	}

    // Accessor functions for individual X,Y,Z components
    //
    inline float &operator[](const int i)              { return (&x)[i];	}
    inline const float &operator[](const int i) const  { return (&x)[i];	}
	inline void mSet(const float aX, const float aY, const float aZ)
	{
		x = aX; y = aY; z = aZ;
    }

    // Unary negation
    //
    tPoint3 operator-() const;

    // Addition of two vectors
    //
    tPoint3 operator+(const tPoint3 &a) const;
    tPoint3 &operator+=(const tPoint3 &a);

    // Subtraction of two vectors
    //
    tPoint3 operator-(const tPoint3 &a) const;
    tPoint3 &operator-=(const tPoint3 &a);

    // Vector cross product, note usage of '%' operator
    //
    tPoint3 operator%(const tPoint3 &a) const;

    // Vector dot product
    //
    float operator*(const tPoint3 &a) const;

    // Scalar multiplication of a vector
    //
    tPoint3 operator*(const float d) const;
    tPoint3 &operator*=(const float d);

    // Scalar division of a vector (use multiplication by reciprocal
    // for optimimum performance instead)
    //
    tPoint3 operator/(const float d) const;
    tPoint3 &operator/=(const float d);

    // Vector equivalence tests
    //
    iTruth operator==(const tPoint3 &a) const;
    iTruth operator!=(const tPoint3 &a) const;

    // Assignment
    //
    inline tPoint3 &operator=(const tPoint3 &a)
	{
		x = a.x; y = a.y; z = a.z;
        return(*this);
	}

    // Methods
    //
    tPoint3	mCrossProduct(const tPoint3 &a) const;
    float	mDotProduct(const tPoint3 &a) const;
    float	mLength(void) const;
    iTruth	mNormalize(void);

};

inline tPoint3 operator*(const float scalar, const tPoint3 &a)
{
    return(a*scalar);
}

//--------------------------------------------------------------------------------
//
struct tPoint2
{
    union   { float x; float u; };
    union   { float y; float v; };
};

struct tPoint4
{
    float x, y, z, w;
};

struct tPoint6
{
	tPoint3	fXYZ;
	tPoint3	fNormal;
};

//--------------------------------------------------------------------------------
// Restore default structure packing
//
#if CompilerMetrowerks && SystemMac7
	#pragma options align=reset
#elif (CompilerMetrowerks && SystemWin32) | CompilerBorland | CompilerMicrosoft | CompilerWatcom
	#pragma pack(pop)
#endif

//--------------------------------------------------------------------------------
//
//
typedef float	tMtx4x4[4][4];
class tMatrix
{
protected:
	union
	{
		float		fM[4][4];
		tMtx4x4		fMtx;
	};
    iTruth			fIdentity;  // kTrue if member fM is identity
    iTruth			f3x4;       // kTrue if member fM is really a 3x4 submatrix

public:
    enum eInitMode
    {
        kUninitialized  = 0,
        kIdentity3x4    = 1,
        kIdentity4x4    = 2
    };

    // Constructors, destructor
    //
    inline tMatrix(const tMatrix &b)
    {
        *this = b;
    }

    inline tMatrix(eInitMode initMode = tMatrix::kIdentity3x4)
    {
        if (initMode != tMatrix::kUninitialized)
        {
            mIdentity();
            f3x4 = initMode == kIdentity3x4;
        }
    }

	inline tMatrix(
		const float xx, const float xy, const float xz, const float xw,
		const float yx, const float yy, const float yz, const float yw,
		const float zx, const float zy, const float zz, const float zw,
		const float wx, const float wy, const float wz, const float ww)
	{
		fIdentity = f3x4 = kFalse;
		fM[0][0] = xx;	fM[0][1] = xy;	fM[0][2] = xz;	fM[0][3] = xw;
		fM[1][0] = yx;	fM[1][1] = yy;	fM[1][2] = yz;	fM[1][3] = yw;
		fM[2][0] = zx;	fM[2][1] = zy;	fM[2][2] = zz;	fM[2][3] = zw;
		fM[3][0] = wx;	fM[3][1] = wy;	fM[3][2] = wz;	fM[3][3] = ww;
	}

	inline tMatrix(
		const float xx, const float xy, const float xz, const float xw,
		const float yx, const float yy, const float yz, const float yw,
		const float zx, const float zy, const float zz, const float zw)
	{
		fIdentity	= kFalse; 
		f3x4		= kTrue;
		fM[0][0] = xx;	fM[0][1] = xy;	fM[0][2] = xz;	fM[0][3] = xw;
		fM[1][0] = yx;	fM[1][1] = yy;	fM[1][2] = yz;	fM[1][3] = yw;
		fM[2][0] = zx;	fM[2][1] = zy;	fM[2][2] = zz;	fM[2][3] = zw;
		fM[3][0] = fM[3][1] = fM[3][2] = (float)0; fM[3][2] = (float)1;
	}

public:
	// Sets the matrix to identity
	//
    void mIdentity(void);

	// Following is provided to allow direct access to the 4x4 float matrix
	// and is intended for use only for those functions that must have
	// direct access. A motion capture player is one example
	//
	inline tMtx4x4 &mGetMtx4x4(void)
	{
		fIdentity = kFalse;		// giving write access implies identity can no
		return(fMtx);			// longer be guaranteed
	}

	// Following is provided to allow direct access to the 4x4 float matrix
	// and is intended for read only use
	//
	inline const tMtx4x4 &mGetMtx4x4(void) const
	{
		return(fMtx);
	}

	// Following is provided to allow direct read-only access to the elements
	// of the matrix. This is required by the OpenGL routines
	//
	inline const float *mGetMtx(void) const
	{
		return(&fMtx[0][0]);
	}

    // Return the swapped row/column inverse of the matrix
    //
    tMatrix mGetTransposition(void) const;

    // Transpose the matrix (swap rows with columns)
    //
    void mTranspose(void);

	// Retrieve an item
	//
	inline float mGetItem(const unsigned kRow, const unsigned kCol) const	{	return(fM[kRow][kCol]);	}

    // Set to or concatenate a rotation matrix around X, Y, Z respectively
    //
    void mRotX(const tAngle angle);
    void mRotY(const tAngle angle);
    void mRotZ(const tAngle angle);
	void mRotXRadians(const float radians);
	void mRotYRadians(const float radians);
	void mRotZRadians(const float radians);

	// Set to or concatenate a scaling matrix
	//
    void mScale(const float x, const float y, const float z);

    // Set to or concatenate a translation matrix
    //
    void mTranslate(const float x, const float y, const float z);

    // Force translation components of the vector
    //
    void mForceTranslate(const float x, const float y, const float z);

	// Add individual translation components
	//
    void mPostTranslate(const float x, const float y, const float z);

    // Matrix multiplcation (you can mix 3x4 and 4x4 matrices)
    //
    void mMult(const tMatrix &a);                   // this  = this * a
    void mMult(tMatrix &a, const tMatrix &b) const; // a = this * b

    // Transform a point using the matrix
    //
    void mTransform(tPoint3 &p) const;
    void mTransform(tPoint3 &out, const tPoint3 &in) const;
    void mTransform(tPoint4 &out, const tPoint3 &in) const;
    void mTransform3x3(tPoint3 &p) const;
	void mTransform3x3(tPoint3 &outPt, const tPoint3 &inPt) const;

    // Inverse transform a point (uses the transpose of the matrix)
    //
	void mInverseTransform(tPoint3 &pt3) const;
    void mInverseTransform(tPoint3 &out, const tPoint3 &in) const;
    void mInverseTransform3x3(tPoint3 &out, const tPoint3 &in) const;

	// Methods that should only be used in very special circumstances
	//
	inline void	mSet3x4Flag(const iTruth bIs3x4)	{	f3x4 = (iTruth)(bIs3x4 ? kTrue : kFalse);	}
	inline iTruth	mGet3x4Flag(void) const			{	return(f3x4);								}
};

//--------------------------------------------------------------------------------
// Constants
//
const double	kPi = 3.14159265358979323846;
const float		kFloatToInt = (float)(1.5f * (1l << 23));
const tMatrix	kIdentityMtx(tMatrix::kIdentity3x4);

enum
{
	kAngle1		= 32,			// one angle (32 seconds per degree)
	kAngle90	= kAngle1 * 64,	// 64*32 seconds per 90 degrees
	kAngle180	= kAngle90 * 2,	
	kAngle270	= kAngle90 * 3,
	kAngle360	= kAngle90 * 4,
	kAngleMask	= kAngle360 - 1
};

//--------------------------------------------------------------------------------
// Trig functions
//
extern tAngle ArcTangent(const float x, const float y);

inline float Sine(const tAngle angle)
{
	extern float gSinCosTable[];
	return( gSinCosTable[ angle & kAngleMask ]);
}

inline float Cosine(const tAngle angle)
{
	extern float gSinCosTable[];
    return( gSinCosTable[ (angle & kAngleMask) + kAngle90 ] );
}

inline float Tangent(const tAngle angle)
{
	extern float gTangentTable[];
    return( gTangentTable[ (angle & kAngleMask) ] );
}

inline float tAngleToRadians(const tAngle angle)
{
    return( (float) ((angle & kAngleMask) * (kPi / kAngle180)) );
}

inline tAngle DegreesTotAngle(float deg)
{
    return( (tAngle) ((kAngle360 / 360.0) * deg) );
}

inline tAngle RadiansTotAngle(float rad)
{
    return( (tAngle) ((kAngle180 / kPi) * rad) );
}

inline double tAngleToDegrees(tAngle angle)
{
	return( (double)angle * (360.0 / (double)kAngle360) );
}

//--------------------------------------------------------------------------------
// Misc functions
//
extern float frand(void);	// Returns random value between 0.0f and 1.0f


#endif	// endif on tvectormath_h

//--------------------------------------------------------------------------------
//	<History>
//
// 22Aug97	rbarris		derived from dmp3d.h - should be completely interchangeable
// 22Sep97	rbarris		re-synchronized, added two methods and changed all tReal to 'float'.
// 17Oct97	dmandap		restored original contents of dmp3d.h into this file
// 07Nov97	dmandap		Removed function headers and other descriptive commenting.
//						Note: Source code and compiler directives unmodified.
// 24Mar98	dmandap		Created 'kFloatToInt', 'kPi' and 'kIdentityMtx' constants.
// 18Apr99	dmandap		Added frand() function
// 18May99	dmandap		Removed tLineSegment and tFlt2Int4, moved most inline functions out of here
//						and placed into tvectormath.cpp
//

