//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include "DataStoreInput.h"

#if !defined( _INLINE_DataStoreInput )
	// Non-Inline section
namespace nDataStore
{

tInput::iInputFlags tInput::fDefaultInputFlags = tInput::kReadCRC | tInput::kValidateCRC | tInput::kReadVersion;

}

#else
	// In-line section

#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>

#pragma warning( disable:4800 )
namespace nDataStore
{

tInput::tInput( int32u initSize )
	: fGetPos( 0 )
	, fCleanUpBuffer( true )
	, fInputFlags( fDefaultInputFlags )
{
	fBuffer = new tBuffer( initSize );
	validate( fBuffer ); 
}

tInput::tInput( const tBuffer* buffer )
	: fGetPos( 0 )
	, fBuffer( buffer )
	, fCleanUpBuffer( false )
	, fInputFlags( fDefaultInputFlags )
{
	validate( fBuffer ); 	
}


tInput::~tInput()
{
	Close( fCleanUpBuffer ); 
}

tInput::tInput( const tInput& copy ) :
	fBuffer( NULL )
{
	*this = copy;
}

tInput& tInput::operator=( const tInput& copy )
{
	if( this == &copy )
	{
		return *this;
	}

	if( fBuffer )
	{
		delete fBuffer;
		fBuffer = NULL;
	}

	fGetPos = copy.fGetPos;
	fCleanUpBuffer = true;

	if( copy.fBuffer )
	{
		fBuffer = new tBuffer( *(copy.fBuffer) );	
	}
	
	return *this;
}

void tInput::SetDefaultFlags( iInputFlags newDefaultFlags )
{
	fDefaultInputFlags = newDefaultFlags;
}

void tInput::AddFlags( iInputFlags newFlags, bool replaceNewFlags )
{
	if( replaceNewFlags )
	{
		fInputFlags = newFlags;
	}
	else
	{
		fInputFlags |= newFlags;
	}
}

void tInput::RemoveFlags( iInputFlags flagsToRemove )
{
	fInputFlags &= ~flagsToRemove;	
}

tInput::iInputFlags	tInput::GetCurFlags() const
{
	return fInputFlags;
}

bool tInput::CheckFlags( iInputFlags flags ) const
{
	if( fInputFlags & flags )
	{
		return true;
	}
	else
	{
		return false;
	}
}

const tBuffer* tInput::GetBuffer() const
{
	return fBuffer;
}

tBuffer* tInput::Close( bool cleanUpBuffer ) 
{
	tBuffer* value = NULL;

	if( fBuffer ) 
	{
		if( cleanUpBuffer )
		{	
			delete fBuffer;
			fBuffer = NULL;
		}
		else
		{
			value = const_cast<tBuffer*>( fBuffer );
			fBuffer = NULL;
		}
	} 

	return value;
}

void tInput::SetGetPos( int32 pos )
{
	if( pos > 0 )
	{
		fGetPos = pos;
	}
	else 
	{
		fGetPos = fBuffer->Size() + pos;
		validate( fGetPos >= 0 );
		if( fGetPos < 0 )
		{
			fGetPos = 0;
		}
	}
}

byte tInput::Get()
{
	return fBuffer->Get( fGetPos++ );
}

void tInput::Read( byte* data, int32 size )
{
	fBuffer->Read( data, size, fGetPos );

	fGetPos += size;
}

void tInput::Rewind()
{
	fGetPos = 0;
}

//TEMPLATE FUNCTIONS
template< class tType >
tInput& operator>>( tInput& input, std::basic_string< tType >& string )
{
	string.erase();

	int32u size = 0;
	input.Read( (byte*)&size, sizeof( size ) );
	
	for( int i = 0; i < size; i++ )
	{
		tType type;
		input >> type;
		string += type;
	}

	return input;
}

template< class tType >
tInput& operator>>( tInput& input, std::vector< tType >& value )
{
	value.clear();

	int32u size = 0;
	input.Read( (byte*)&size, sizeof( size ) );
	
	value.reserve( size );
	
	for( int i = 0; i < size; i++ )
	{
		tType type;
		input >> type;
		value.push_back( type );
	}

	return input;
}


template< class tType >
tInput& operator>>( tInput& input, std::list< tType >& value )
{
	value.clear();

	int32u size = 0;
	input.Read( (byte*)&size, sizeof( size ) );
	
	for( int i = 0; i < size; i++ )
	{
		tType type;
		input >> type;
		value.push_back( type );
	}

	return input;
}

template< class tType >
tInput& operator>>( tInput& input, std::deque< tType >& value )
{
	value.clear();

	int32u size = 0;
	input.Read( (byte*)&size, sizeof( size ) );

	value.reserve( size );
	
	for( int i = 0; i < size; i++ )
	{
		tType type;
		input >> type;
		value.push_back( type );
	}

	return input;
}

template< class tIDType, class tValueType, class tCompareType >
tInput& operator>>( tInput& input, std::map< tIDType, tValueType, tCompareType >& value )
{
	value.clear();

	int32u size = 0;
	input.Read( (byte*)&size, sizeof( size ) );
	
	for( int i = 0; i < size; i++ )
	{
		tIDType first;
		tValueType second;
		input >> first >> second;
		value[ first ] = second;
	}

	return input;
}

template< class tType >
tInput& operator>>( tInput& input, tInterlockedScalar< tType >& value )
{
	tType holdValue;
	input >> holdValue;
	value = holdValue;

	return input;
}

tInput& tInput::operator>>( tInput& (*func)( tInput& input ) ) 
{
	return func( *this );
}

tInput& tInput::operator>>( tBuffer& buffer )
{
	int32u size = 0;
	Read( (byte*)&size, sizeof( size ) );
	
	for( int32u i = 0; i < size; i++ )
	{
		buffer.Put( Get() );
	}  
	
	return *this;
}

tInput& tInput::operator>>( std::string& string )
{
	string.erase();

	int32u size = 0;
	Read( (byte*)&size, sizeof( size ) );

	for( int32u i = 0; i < size; i++ )
	{
		string += Get();
	}
	
	return *this;
}


tInput& tInput::operator>>( bool& value )
{
	value = (bool)Get();
	
	return *this;
}

tInput& tInput::operator>>( char& value )
{
	value = (char)Get();
	
	return *this;
}

tInput& tInput::operator>>( int8s& value )
{
	value = (int8s)Get();
	
	return *this;
}

tInput& tInput::operator>>( int8u& value )
{
	value = (int8u)Get();
	
	return *this;
}

tInput& tInput::operator>>( int16u& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( int16s& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( int32u& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( int32s& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( float& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( double& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

tInput& tInput::operator>>( long double& value )
{
	Read( (byte*)&value, sizeof( value ) );
	
	return *this;
}

};

#endif