//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include "DataStoreOutput.h"

#if !defined( _INLINE_DataStoreOutput )

	//Non in-line section
namespace nDataStore
{

tOutput::iOutputFlags tOutput::fDefaultOutputFlags = tOutput::kOutputVersion | tOutput::kOutputCRC;

}

#else
	//In-line section

#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>

namespace nDataStore
{

tOutput::tOutput( int32u initSize )
	: fCleanUpBuffer( true )
	, fOutputFlags( fDefaultOutputFlags )
{
	fBuffer = new tBuffer( initSize );
	validate( fBuffer );	
}

tOutput::tOutput( tBuffer* buffer )
	: fBuffer( buffer )
	, fCleanUpBuffer( false )
	, fOutputFlags( fDefaultOutputFlags )
{
	validate( fBuffer );
}


tOutput::~tOutput()
{
	Close( fCleanUpBuffer ); 
}

tOutput::tOutput( const tOutput& copy ) :
	fBuffer( NULL )
{
	*this = copy;
}

tOutput& tOutput::operator=( const tOutput& copy )
{
	if( this == &copy )
	{
		return *this;
	}

	if( fBuffer )
	{
		delete fBuffer;
		fBuffer = NULL;
	}

	fCleanUpBuffer = true;

	if( copy.fBuffer )
	{
		fBuffer = new tBuffer( *(copy.fBuffer) );	
	}
	
	return *this;
}

void tOutput::SetDefaultFlags( iOutputFlags newDefaultFlags )
{
	fDefaultOutputFlags = newDefaultFlags;
}

void tOutput::AddFlags( iOutputFlags newFlags, bool replaceNewFlags )
{
	if( replaceNewFlags )
	{
		fOutputFlags = newFlags;
	}
	else
	{
		fOutputFlags |= newFlags;
	}
}

void tOutput::RemoveFlags( iOutputFlags flagsToRemove )
{
	fOutputFlags &= ~flagsToRemove;	
}

tOutput::iOutputFlags tOutput::GetCurFlags() const
{
	return fOutputFlags;
}

bool tOutput::CheckFlags( iOutputFlags flags ) const
{
	if( fOutputFlags & flags )
	{
		return true;
	}
	else
	{
		return false;
	}
}

const tBuffer* tOutput::GetBuffer() const
{
	return fBuffer;
}

tBuffer* tOutput::Close( bool cleanUpBuffer ) 
{
	tBuffer* value = NULL;

	if( fBuffer ) 
	{
		if( cleanUpBuffer )
		{	
			delete fBuffer;
			fBuffer = NULL;
		}
		else
		{
			value = fBuffer;	
			fBuffer = NULL;
		}
	}

	return value;
}

void tOutput::Put( byte data )
{
	fBuffer->Put( data );
}

void tOutput::Write( const byte* data, int32 size )
{
	fBuffer->Write( data, size );
}


//TEMPLATE Functions!

template< class tType >
tOutput& operator<<( tOutput& output, const std::basic_string< tType >& value )
{
	int32u size = value.size();
	output.Write( (const byte*)&size, sizeof( size ) );
	
	for( std::basic_string< tType >::const_iterator i = value.begin();
		 i != value.end();
		 i++ )
	{
		output << *i;
	}

	return output;
}

template< class tType >
tOutput& operator<<( tOutput& output, const std::vector< tType >& value )
{
	int32u size = value.size();
	output.Write( (const byte*)&size, sizeof( size ) );
	
	for( std::vector< tType >::const_iterator i = value.begin();
		 i != value.end();
		 i++ )
	{
		output << *i;
	}

	return output;
}

template< class tType >
tOutput& operator<<( tOutput& output, const std::list< tType >& value )
{
	int32u size = value.size();
	output.Write( (const byte*)&size, sizeof( size ) );
	
	for( std::list< tType >::const_iterator i = value.begin();
		 i != value.end();
		 i++ )
	{
		output << *i;
	}

	return output;
}

template< class tType >
tOutput& operator<<( tOutput& output, const std::deque< tType >& value )
{
	int32u size = value.size();
	output.Write( (const byte*)&size, sizeof( size ) );
	
	for( std::deque< tType >::const_iterator i = value.begin();
		 i != value.end();
		 i++ )
	{
		output << *i;
	}

	return output;
}

template< class tIDType, class tValueType, class tCompareType >
tOutput& operator<<( tOutput& output, const std::map< tIDType, tValueType, tCompareType >& value )
{
	int32u size = value.size();
	output.Write( (const byte*)&size, sizeof( size ) );
	
	for( std::map< tIDType, tValueType, tCompareType >::const_iterator i = value.begin();
		 i != value.end();
		 i++ )
	{
		output << i->first << i->second;
	}

	return output;
}


tOutput& tOutput::operator<<( tOutput& (*func)( tOutput& input ) ) 
{
	return func( *this );
}

tOutput& tOutput::operator<<( const tBuffer& buffer )
{
	int32u size = buffer.Size();
	Write( (const byte*)&size, sizeof( size ) );

	for( int32u i = 0; i < size; i++ )
	{
		Put( buffer.Get( i ) );		
	}
	
	return *this;
}

tOutput& tOutput::operator<<( const std::string& string )
{
	int32u size = string.size();
	Write( (const byte*)&size, sizeof( size ) );

	Write( (const byte*)string.data(), size );
	
	return *this;
}

tOutput& tOutput::operator<<( const char* string )
{
	int32u size = strlen( string );
	Write( (const byte*)&size, sizeof( size ) );

	Write( (const byte*)string, size );
	
	return *this;
}

tOutput& tOutput::operator<<( bool value )
{
	Put( (byte)value );
	
	return *this;
}

tOutput& tOutput::operator<<( char value )
{
	Put( (byte)value );
	
	return *this;
}

tOutput& tOutput::operator<<( int8s value )
{
	Put( (byte)value );
	
	return *this;
}

tOutput& tOutput::operator<<( int8u value )
{
	Put( (byte)value );
	
	return *this;
}

tOutput& tOutput::operator<<( int16u value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( int16s value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( int32u value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( int32s value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( float value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( double value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

tOutput& tOutput::operator<<( long double value )
{
	Write( (const byte*)&value, sizeof( value ) );
	
	return *this;
}

};

#endif