//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>

#include "Streamable.h"

#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <Log\LogStream.h>
#include <Crypto\CRC32.h>

namespace nDataStore
{

////////////////////////////////////////////////////////////////////////////////
// tStreamable::tCRCFailed
// 
////////////////////////////////////////////////////////////////////////////////

tStreamable::tCRCFailed::tCRCFailed( const std::string& toObject, const std::string& forObject, const char* file, int32 lineNumber )
	: tBaseException( "tStreamable::tCRCFailed", file, lineNumber )
{
	_func_start( nDataStore::tStreamable::tCRCFailed::tCRCFailed );
	try
	{
		fToObject = toObject;
		fForObject = forObject;
	}
	catch( ... )
	{}
	_func_end;
}

void tStreamable::tCRCFailed::ReportException( const char* caughtDescription ) throw()
{
	_func_start( nDataStore::tStreamable::tCRCFailed::ReportException );
	try
	{
		//ZDNF Fill in real descriptions
		tBaseException::ReportException( caughtDescription ); 
	}
	catch( ... )
	{
	}
	_func_end;
}

// Data
tStreamable::iObjectIDInfoLookup tStreamable::fObjectIDInfoLookup;

tStreamable::tObjectIDInfo::tObjectIDInfo() :
	fCRC( tCRC32::kDefaultSeed )
{_func_start( nDataStore::tStreamable::tObjectIDInfo::tObjectIDInfo );_func_end;}

//-----------------------------------------------------------------------------------------------------

tStreamable::tStreamable( iObjectVersion versionNumber ) :
	fObjectID( tCRC32::kDefaultSeed	),
	fObjectVersion( versionNumber )
{_func_start( nDataStore::tStreamable::tStreamable );_func_end;}

tStreamable::~tStreamable()
{_func_start( nDataStore::tStreamable::~tStreamable );_func_end;}

tStreamable::iObjectID tStreamable::GetObjectID() const
{
	_func_start( nDataStore::tStreamable::GetObjectID );
	if( fObjectID == tCRC32::kDefaultSeed )
	{
		// If we don't know are ID, look it up
		const type_info& objectType = typeid( *this );

		tObjectIDInfo* idInfo = &fObjectIDInfoLookup[ &objectType ];

		if( idInfo->fCRC == tCRC32::kDefaultSeed )
		{ 
			// If the ID was never created, create it now
			tCRC32 crc;
			std::string name = objectType.raw_name();
			idInfo->fObjectName = objectType.name(); 
			idInfo->fCRC = crc.UpdateCRC( tCRC32::kDefaultSeed, name.begin(), name.size() );
		}

		fObjectID = idInfo->fCRC;
	}

	return fObjectID;
	_func_end;
}

tStreamable::iObjectVersion tStreamable::GetVersion() const
{
	_func_start( nDataStore::tStreamable::GetVersion );
	//ZDNF@ The Object Version is currently manual increased, it would be good to 
	// come up with a automatic way to increment it
	return fObjectVersion;
	_func_end;
}

nDataStore::tInput& operator>>( nDataStore::tInput& input, tStreamable& object )
{
	_func_start( nDataStore::operator >> );
	tStreamable::iObjectID objID;

	tStreamable::iObjectVersion defaultVer = object.GetVersion();

	if( input.CheckFlags( tInput::kReadVersion ) )
	{
		input >> object.fObjectVersion;
	}
	else
	{
		object.fObjectVersion = 0;
	}

	if( input.CheckFlags( tInput::kDefaultVersion ) )
	{
		object.fObjectVersion = defaultVer;
	}

	if( input.CheckFlags( tInput::kReadCRC ) )
	{
		input >> objID;

		if( input.CheckFlags( tInput::kValidateCRC ) && objID != object.GetObjectID() )
		{
			// This is the wrong stream for this object, look up the name that
			// the stream was intended for. Then register a error and throw
			const type_info& objectType = typeid( object );
			std::string objectName = objectType.name();
			std::string packetFor;

			for( tStreamable::iObjectIDInfoLookup::iterator i = tStreamable::fObjectIDInfoLookup.begin();
				 i != tStreamable::fObjectIDInfoLookup.end();
				 i++ )
			{
				if( i->second.fCRC == objID )
				{
					packetFor = i->second.fObjectName;
					break;
				}
			}

			throw tStreamable::tCRCFailed( objectName, packetFor, __FILE__, __LINE__ );
		}
	}

	object.StreamIn( input );

	return input;
	_func_end;
}

nDataStore::tOutput& operator<<( nDataStore::tOutput& output, const tStreamable& object )
{
	_func_start( nDataStore::operator << );
	if( output.CheckFlags( tOutput::kOutputVersion ) )
	{
		output << object.GetVersion();
	}
	
	if( output.CheckFlags( tOutput::kOutputCRC ) )
	{
		output << object.GetObjectID();
	}

	object.StreamOut( output );

	return output;
	_func_end;
}

};

