//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Database_h )
#define Database_h

#include <BaseTypes.h>

#if !defined( USE_PCH )

#include <map>
#include <vector>

#endif 

#include <DynamicAllocator.h>
#include <Exceptions\Validate.h>
#include <DataBase\DatabaseExceptions.h>
#include <Thread\Optex.h>
#include <MemberCallback.h>
#include <Timer\HeartbeatTimer.h>

#include "Query.h"
#include "DatabaseID.h"

namespace nDatabase
{

class tDatabaseStorage;
class tDatabaseObject;
class tDatabaseObjectUpdate;

// CLASS PURPOSE: This is a front end to a generic database interface
class tDatabase 
{
public:
//Types
	typedef int32u iLockID;
	enum
	{
		kNotLocked = 0,
		kInitialLockID,

		kLockExpiration = 300, // Locks last five minutes (for now)
	};

//Setup
	// The database takes ownership of the tDatabaseStorage and will
	// clean it up when it needs
	tDatabase( tDatabaseStorage* pStorage );
	virtual ~tDatabase();

//Operations
	virtual void Flush();

//	MEMBER TEMPLATE are defined at the bottom of the class decleration
	//template< class tClass >
	//virtual void AssignObjectToDB( tDatabaseObject* pObject );
	void RecoverObjectFromStorage( tDatabaseObject* pActualObject );
	virtual void RemoveObjectFromDB( tDatabaseObject* pObject );
	
	virtual void Store( const tDatabaseObject* pObject );
	virtual void Store( const tDatabaseObject& object );

//	MEMBER TEMPLATE are defined at the bottom of the class decleration
//	template< class tClass, class tResultClassPtr > 
//	void QueryDB(  const tQuery< tClass >& query, std::vector< tResultClassPtr >* result );
//	template< class tClass, class tResultClassPtr > 
//	bool LockingQueryDB( const tQuery< tClass >& query, std::vector< tResultClassPtr >* result );

//	template< class tnDatabaseClass >
//	void Lock( const tnDatabaseClass* pObject, tnDatabaseClass* pDestObject )	
	virtual void UnLock( tDatabaseObject* pObject );
	virtual void UnLock( const tID& objectID );
	virtual void UpdateObject( const tDatabaseObjectUpdate& update, bool unlock = true ); 

//Accessors
//	template< typename tnClass >
//	void GetObject( const tID& id, tnClass* pObject ) const

protected:
//Types
	class tLockInfo
	{
	public:
	//Fields
		tLockInfo( iLockID lockID, time_t expiration, tDatabaseObject* pObject );
		bool operator<( const tLockInfo& lockInfo ) const; 
		bool operator<( const iLockID lockID ) const; 
		bool operator==( const iLockID lockID ) const; 

		bool fLocked;
		iLockID fLockID;
		time_t fExpiration;
		tDatabaseObject* fpObject;	
	};

	typedef std::vector< tDatabaseObject* > iObjects;
	typedef std::map< tID, tDatabaseObject* > iIDIndex;
	typedef std::map< iAllocatorID, iObjects > iTypeIndex;
	typedef std::deque< tLockInfo > iLocks;
	typedef tMemberCallback< tDatabase, int32u > iCallback; 
	
// Operations
	virtual const tDatabaseObject* GetObject( const tID& id ) const;
	virtual void Lock( tDatabaseObject* pObject ); 
	virtual void MultiLock( iObjects* objects, bool mustLockAll = true ); 
	virtual void RecordLock( iLockID lockID, tDatabaseObject* pObject );
	virtual void CompactLocks();

	void AddObjectToDBIndex( tDatabaseObject* pActualObject, iAllocatorID allocID );

// Events
	void OnTick( int32u context );

//Fields
	mutable tOptex		fAccess;
	iTypeIndex			fTypeIndex;
	iIDIndex			fMasterIndex;
	tDatabaseStorage*	fDatabaseStorage;
	
	iLockID				fNextLockID;
	iLocks				fLocks;

	tTimer*				fAlarm;
	iCallback*			fAlarmCB;

	//State
	bool fSingleThreaded;
	
private:
//Hidden
	tDatabase( const tDatabase& copy );
	tDatabase& operator=( const tDatabase& copy );



// ----- Implementaion of template functions ----- 
// NOTE: This block is not normal, nor is it good. It is here because
// VC 6 has a bug, which does not allow you to have memeber templates
// outside the class decleration. We put it at the end in it's own block
// so we don't uglify the rest of the decleration
public: 
	template< class tClass >
	void AssignObjectToDB( tClass* pObject )
	{
		_func_start( nDatabase::tDatabase::AssignObjectToDB );
		tOptex::tGuard guard( &fAccess );

		validate( pObject );
		iAllocatorID allocID = tDynamicAllocator::Instance()->mGetAllocatorID( typeid( *pObject ) ); 
		validate( allocID != kInvalidAllocatorID );

		tClass* pActualObject = static_cast< tClass* >( tDynamicAllocator::Instance()->mAllocateClass( allocID ) );
		validate( pActualObject );
		*pActualObject = *pObject;

		validate( pActualObject->GetID().IsValid() == false );
		
		//Save the pActualObject, after it is saved it has an ID associated with it
		const tID& newID = fDatabaseStorage->AddToStorage( pActualObject );
		pActualObject->AttachToDB( newID );
		
		// Any new object cannot be locked (so if loaded in, must clear lock)
		pActualObject->SetLockID( kNotLocked );
		
		AddObjectToDBIndex( pActualObject, allocID );

		*pObject = *pActualObject;
		_func_end;
	}


	template< class tClass, class tResultClassPtr > 
	void QueryDB( const tQuery< tClass >& query, tResultClassPtr* result )
	{
		_func_start( nDatabase::tDatabase::QueryDB );
		tOptex::tGuard guard( &fAccess );

		iAllocatorID allocID = tDynamicAllocator::Instance()->mGetAllocatorID( typeid( tClass ) ); 	
		validate( allocID != kInvalidAllocatorID );

		const tDatabase::iObjects& objects = fTypeIndex[allocID];
		
		for( tDatabase::iObjects::const_iterator i = objects.begin();
			 i != objects.end();
			 i++ )
		{
			// We can use a static cast here, because we have already verified 
			// the type before it was inserted into the container
			const tClass* object = static_cast<const tClass*>( *i );	
			if( query( object ) )
			{
				result->push_back( *object );
				result->back().MakeReadOnly();
			}
		} 
		_func_end;
	}

	template< class tClass, class tResultClassPtr > 
	void LockingQueryDB( const tQuery< tClass >& query, tResultClassPtr* result, bool mustLockAll = true )
	{
		_func_start( nDatabase::tDatabase::LockingQueryDB );
		tOptex::tGuard guard( &fAccess );

		iAllocatorID allocID = tDynamicAllocator::Instance()->mGetAllocatorID( typeid( tClass ) ); 	
		validate( allocID != kInvalidAllocatorID );

		const tDatabase::iObjects& objects = fTypeIndex[allocID];
		bool lockFailed = false;

		iObjects holdResult;
		for( tDatabase::iObjects::const_iterator i = objects.begin();
			 i != objects.end() && !lockFailed;
			 i++ )
		{
			// We can use a static cast here, because we have already verified 
			// the type before it was inserted into the container
			tClass* pObject = static_cast<tClass*>( *i );	

			if( query( pObject ) )
			{		
				// We don't want this object deleted while we are waiting for it
				pObject->AddReference();
				holdResult.push_back( pObject );
			}
		} 
		guard.Leave();

		if( holdResult.size() > 0 )
		{ 
			MultiLock( &holdResult, mustLockAll ); 

			for( iObjects::iterator lockedObject = holdResult.begin();
				 lockedObject != holdResult.end();
				 lockedObject++ )
			{
				if ( *lockedObject )
				{
					// We can use a static cast here, because we have already verified 
					// the type before it was inserted into the container
					tClass* pObject = static_cast<tClass*>( *lockedObject );
					if( ! pObject->IsDead() )
					{
						result->push_back( *pObject );
					}
					
					pObject->RemoveReference();
				}
			}
		}
		_func_end;
	}
	
	template< class tnDatabaseClass >
	void Lock( const tnDatabaseClass* pObject, tnDatabaseClass* pDestObject )
	{
		_func_start( nDatabase::tDatabase::Lock );
		tOptex::tGuard guard( &fAccess );

		tnDatabaseClass* pActualDBObject = NULL;
		
		iIDIndex::iterator object = fMasterIndex.find( pObject->GetID() );
		
		validate( object != fMasterIndex.end() );
		//ZD_EXCEPT@@@@ validate( object->second && object->second->mGetAllocatorID() == GetAllocID<tnDatabaseClass>() ); 

		pActualDBObject = reinterpret_cast< tnDatabaseClass* >( object->second );
		guard.Leave();

		Lock( pActualDBObject );

		*pDestObject = *pActualDBObject;
		_func_end;
	}

	template< typename tnClass >
	void GetObject( const tID& id, tnClass* pObject ) const
	{
		_func_start( nDatabase::tDatabase::GetObject );
		tOptex::tGuard guard( &fAccess );

		const tDatabaseObject* pActualObject = GetObject( id );

		//ZD_EXCEPT@@@@ validate( pActualObject && pActualObject->mGetAllocatorID() == GetAllocID<tnClass>() ); 

		const tnClass* pConvert = reinterpret_cast< const tnClass* >( pActualObject ); 
		*pObject = *pConvert;
		pObject->MakeReadOnly();

		_func_end;
	}

	template< class tClass > 
	int32 GetObjectCount( const tQuery< tClass >& query )
	{
		_func_start( nDatabase::tDatabase::GetObjectCount );
		tOptex::tGuard guard( &fAccess );

		int32	result = 0;

		iAllocatorID allocID = tDynamicAllocator::Instance()->mGetAllocatorID( typeid( tClass ) ); 	
		validate( allocID != kInvalidAllocatorID );

		const tDatabase::iObjects& objects = fTypeIndex[allocID];
		
		for( tDatabase::iObjects::const_iterator i = objects.begin();
			 i != objects.end();
			 i++ )
		{
			// We can use a static cast here, because we have already verified 
			// the type before it was inserted into the container
			const tClass* object = static_cast<const tClass*>( *i );	
			if( query( object ) )
			{
				++result;
			}
		}

		return result;
		_func_end;
	}
};


};
namespace db = nDatabase;

#endif


